package controle.eventos;

import java.text.NumberFormat;
import java.util.ArrayList;

import jfan.fan.NeuronioFAN;
import jfan.fan.StoredNetworkType;
import jfan.fan.beans.StatisticBean;
import jfan.fan.events.ValidacaoEvent;
import jfan.fan.events.ValidacaoListener;
import jfan.fan.padroes.IPadrao;
import thinletcommons.MessageDialog;
import visao.gui.telas.classes.EasyFan;
import visao.gui.telas.classes.MatrizConfusaoTela;
import controle.main.App;

public class EvtValidacao implements ValidacaoListener {
	private EasyFan owner;
	private final NumberFormat numFormat = NumberFormat.getInstance();
	private StatisticBean statisticas;
	private int epoca = -1;
	
	public EvtValidacao(EasyFan owner) {
		numFormat.setMaximumFractionDigits(4);
		numFormat.setMinimumFractionDigits(4);
		numFormat.setMaximumIntegerDigits(3);
		numFormat.setMinimumIntegerDigits(3);
		this.owner = owner;
	}
	
	public void validarRede(StoredNetworkType tipo) {
		App.requisitarNormalizacao();
		statisticas = App.getMonitorFAN().validar(tipo);
		atualizarDados(tipo.toString(),"--");
		epoca = -1;
	}
	
	public void validarRede(ArrayList<NeuronioFAN> neuronios) {
		App.requisitarNormalizacao();
		statisticas = App.getMonitorFAN().validar(neuronios);
		atualizarDados("Carregada","--");
		epoca = -1;
	}
	
	public void chamarTelaMatrizConfusao() {
		if (statisticas.getMatrizConfusao() != null) {
			MatrizConfusaoTela mct = new MatrizConfusaoTela(App.getTelaEasyFAN().getFrame(),"Matriz de confusão: Teste",epoca,statisticas.getMatrizConfusao(),statisticas.getHarmonica(),statisticas.getAritmetica(),statisticas.getMaxmin(), "validacao");
			mct.show();
		}
		else {
			MessageDialog msg = new MessageDialog(App.getTelaEasyFAN().getFrame(),"Atenção","É necessário testar uma rede antes.");
			msg.show();
		}
	}
	
	private void atualizarDados(String tipoRede, String epocaValidada) {
		owner.setAcertosValidacao(Integer.toString(statisticas.getAcertos()));
		owner.setErrosValidacao(Integer.toString(statisticas.getErros()));
		owner.setHarmonicaValidacao(numFormat.format(statisticas.getHarmonica()));
		owner.setAritmeticaValidacao(numFormat.format(statisticas.getAritmetica()));
		owner.setMaxMinValidacao(numFormat.format(statisticas.getMaxmin()));
		owner.setTipoRedeValidacao(tipoRede);
		owner.setEpocaValidada(epocaValidada);
		
	}
	
	public void validarCadaEpocas(int i) {
		App.getProjetoConfig().getValidacaoConfig().setCadaEpocaValidar(i);
		App.getMonitorFAN().setCadaEpocaValidar(i);
	}
	
	public void desativarValidarCadaEpocas() {
		App.getProjetoConfig().getValidacaoConfig().setCadaEpocaValidar(0);
		App.getMonitorFAN().setCadaEpocaValidar(0);
	}
	
	public void setTipoRedeValidacao(StoredNetworkType tipoRede) {
		App.getProjetoConfig().getValidacaoConfig().setTipoRedeValidacao(tipoRede);
		App.getMonitorFAN().setTipoRedeValidar(tipoRede);
	}
	

	public void validou(ValidacaoEvent event) {
		statisticas = event.getStatisticas();
		epoca = event.getEpocaValidacao() ;
		atualizarDados(event.getTipoRedeValidacao().toString(), Integer.toString(epoca));
	}

	public void validarRede(ArrayList<NeuronioFAN> neurons, ArrayList<IPadrao> conjunto) {
		//App.requisitarNormalizacao();
		statisticas = App.getMonitorFAN().validar(neurons,conjunto);
		atualizarDados("Carregada","--");
	}

}
