package modelo;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import jfan.fan.StoredNetworkType;
import jfan.fan.temperas.TemperaAleatoria;
import jfan.fan.temperas.TemperaSimuladaFAN;
import jfan.fan.temperas.TipoTempera;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class Projeto {
	
	private String nome;
	
	private RedeConfig rede = new RedeConfig();
	
	private TreinamentoConfig treino = new TreinamentoConfig();
	
	private ValidacaoConfig validacao = new ValidacaoConfig();
	
	private String arquivoAssociado;

	public Projeto(String arquivoConfig) {
		arquivoAssociado = arquivoConfig;
		loadDefaultConfigurationXML(arquivoConfig);
	}
	
	public String getNome() {
		return this.nome;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public RedeConfig getRedeConf() {
		return this.rede;
	}

	public void setRedeConf(RedeConfig rede) {
		this.rede = rede;
	}

	public TreinamentoConfig getTreinoConf() {
		return this.treino;
	}

	public void setTreinoConf(TreinamentoConfig treino) {
		this.treino = treino;
	}

	public ValidacaoConfig getValidacaoConfig() {
		return validacao;
	}

	public void setValidacaoConfig(ValidacaoConfig validacao) {
		this.validacao = validacao;
	}
	
	private boolean loadDefaultConfigurationXML(String file) {
		
		if (file == null) {
			return false;
		}
			
		File arq = new File(file);
		
		if (arq.exists()) {
			try {
				
				// cria a factory e o parser
				SAXParser parser = SAXParserFactory.newInstance().newSAXParser();

				// abre a conexao com o arquivo
				InputSource input = new InputSource(file);

				// inicia o parsing
				parser.parse(input, new LeitorXmlConfig());
				
				return true;
			} 
			catch (ParserConfigurationException ex) {
				// se ocorrer um erro de parsing
				ex.printStackTrace();
			} 
			catch (SAXException ex) {
				// se ocorrer um erro de parsinggerenciadorPadroes = gp;
				ex.printStackTrace();
			} 
			catch (IOException ex) {
				// se ocorrer um erro de io
				ex.printStackTrace();
			}
			
		}
		
		return false;
		
	}
	
	public void saveAsDefaultConfiguration() {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(arquivoAssociado));
			//Cabeçalho
			bw.write("<?xml version=\"1.0\"?>"); bw.newLine();
			//Elemento root
			bw.write("<defaultConfiguration>"); bw.newLine();
			
			//configurações de rede
			bw.write("<networkConfiguration>"); bw.newLine();
			bw.write("\t<suporteConjuntosDifusos>"); 
			bw.write(Integer.toString(rede.getSuporteConjuntosDifusos()));
			bw.write("</suporteConjuntosDifusos>"); bw.newLine();
			bw.write("\t<raioDifuso>"); 
			bw.write(Integer.toString(rede.getRaioDifuso()));
			bw.write("</raioDifuso>"); bw.newLine();
			bw.write("</networkConfiguration>"); bw.newLine();
			
			//configurações de treinamento
			bw.write("<trainningConfiguration>"); bw.newLine();
			bw.write("\t<cadaEpocaRenormalizar>");	
			bw.write(Integer.toString(treino.getCadaEpocaRenormalizar()));
			bw.write("</cadaEpocaRenormalizar>"); bw.newLine();
			bw.write("\t<chanceRenormalizar>"); 
			bw.write(Integer.toString(treino.getChanceRenormalizar()));
			bw.write("</chanceRenormalizar>"); bw.newLine();
			bw.write("\t<cadaEpocaEmbaralharPadroes>"); 
			bw.write(Integer.toString(treino.getCadaEpocaEmbaralharPadroes()));
			bw.write("</cadaEpocaEmbaralharPadroes>"); bw.newLine();
			bw.write("\t<numeroPadroesTreino>");
			bw.write(Integer.toString(treino.getNumeroPadroesTreino()));
			bw.write("</numeroPadroesTreino>"); bw.newLine();
			bw.write("\t<randomizarPesos>");
			bw.write(treino.isRandomizarPesos()?"true":"false");
			bw.write("</randomizarPesos>"); bw.newLine();
			bw.write("\t<pesoBase>");
			bw.write(Integer.toString(treino.getPesoBase()));
			bw.write("</pesoBase>"); bw.newLine();
				//Configurações da têmpera
				bw.write("\t<tempera tipo=\"" +treino.getTipoTempera().toString() + "\">"); bw.newLine();
				bw.write("\t\t<maximo>");
				bw.write(Double.toString(treino.getTempera().getLimiteMaximo()));
				bw.write("</maximo>"); bw.newLine();
				bw.write("\t\t<minimo>");
				bw.write(Double.toString(treino.getTempera().getLimiteMinimo()));
				bw.write("</minimo>"); bw.newLine();
				bw.write("\t\t<step>");
				bw.write(Double.toString(treino.getTempera().getStepValue()));
				bw.write("</step>"); bw.newLine();
				bw.write("\t</tempera>"); bw.newLine();
			bw.write("\t<cadaEpocaResetTempera>");
			bw.write(Integer.toString(treino.getCadaEpocaResetTempera()));
			bw.write("</cadaEpocaResetTempera>"); bw.newLine();
			bw.write("\t<cadaEpocaStepTempera>"); 
			bw.write(Integer.toString(treino.getCadaEpocaStepTempera()));
			bw.write("</cadaEpocaStepTempera>"); bw.newLine();
			bw.write("\t<porcentagemPadroesTreino>");
			bw.write(Integer.toString(treino.getPorcentagemPadroesTreino()));
			bw.write("</porcentagemPadroesTreino>"); bw.newLine();
			bw.write("\t<numeroEpocasTreinar>");
			bw.write(Integer.toString(treino.getNumeroEpocasTreinar()));
			bw.write("</numeroEpocasTreinar>"); bw.newLine();
				//Configurações do tempo de parada
				bw.write("\t<tempoParada>"); bw.newLine();
				bw.write("\t\t<dias>");
				bw.write(Integer.toString(treino.getDiasParada()));
				bw.write("</dias>"); bw.newLine();
				bw.write("\t\t<horas>");
				bw.write(Integer.toString(treino.getHorasParada()));
				bw.write("</horas>"); bw.newLine();
				bw.write("\t\t<minutos>");
				bw.write(Integer.toString(treino.getMinutosParada()));
				bw.write("</minutos>"); bw.newLine();
				bw.write("\t\t<segundos>");
				bw.write(Integer.toString(treino.getSegundosParada()));
				bw.write("</segundos>"); bw.newLine();
				bw.write("\t</tempoParada>"); bw.newLine();
			bw.write("</trainningConfiguration>");
			
			//configurações de validação
			bw.write("<validationConfiguration>"); bw.newLine();
			bw.write("\t<cadaEpocaValidar>");
			bw.write(Integer.toString(validacao.getCadaEpocaValidar()));
			bw.write("</cadaEpocaValidar>"); bw.newLine();
			bw.write("\t<tipoRedeValidacao>");
			bw.write(validacao.getTipoRedeValidacao().toString());
			bw.write("</tipoRedeValidacao>"); bw.newLine();
			bw.write("</validationConfiguration>"); bw.newLine();
			
			//fim do elemento root
			bw.write("</defaultConfiguration>");
			
			bw.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	private class LeitorXmlConfig extends DefaultHandler {
		
		private String tag;
		
		private StringBuilder value = new StringBuilder(150);
		
		private Attributes attributes = null;
		
		private int dias,horas,minutos,segundos;
		
		/** comeca um documento novo */
		public void startDocument() { }

		/** comeca uma tag nova */
		public void startElement(String uri,String localName,String tag,Attributes atributos){
			//this.tag.setLength(0);
			//this.tag.append(tag);
			this.tag = tag;
			value.setLength(0);
			attributes = atributos;
		}

		/** recebe os dados de uma tag */
		/**Faz todo o processamento do documento**/
		public void characters(char[] ch, int start, int length) throws SAXException{
			//values.append(ch,start,length);
			/*if (tag.equalsIgnoreCase("suporteConjuntosDifusos")) {
				value.append(ch,start,length);
			}
			else*/ 
			if (tag.equalsIgnoreCase("/suporteConjuntosDifusos")) {
				rede.setSuporteConjuntosDifusos(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/raioDifuso")) {
				rede.setRaioDifuso(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/cadaEpocaRenormalizar")) {
				treino.setCadaEpocaRenormalizar(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/chanceRenormalizar")) {
				treino.setChanceRenormalizar(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/cadaEpocaEmbaralharPadroes")) {
				treino.setCadaEpocaEmbaralharPadroes(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/numeroPadroesTreino")) {
				treino.setNumeroPadroesTreino(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/randomizarPesos")) {
				if (value.toString().equals("true")) {
					treino.setRandomizarPesos(true);
				}
				else {
					treino.setRandomizarPesos(false);
				}
			}
			else if (tag.equalsIgnoreCase("/pesoBase")) {
				treino.setPesoBase(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("tempera")) {
				String s = attributes.getValue(0);
				if (s.equals("TemperaAleatorio")) {
					treino.setTempera(new TemperaAleatoria());
					treino.setTipoTempera(TipoTempera.TemperaAleatoria);
				}
				else if (s.equals("TemperaSimulada")) {
					treino.setTempera(new TemperaSimuladaFAN());
					treino.setTipoTempera(TipoTempera.TemperaSimuladaFAN);
				}
			}
			else if (tag.equalsIgnoreCase("/maximo")) {
				treino.getTempera().setLimiteMaximo(Double.parseDouble(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/minimo")) {
				treino.getTempera().setLimiteMinimo(Double.parseDouble(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/step")) {
				treino.getTempera().setStep(Double.parseDouble(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/cadaEpocaResetTempera")) {
				treino.setCadaEpocaResetTempera(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/cadaEpocaStepTempera")) {
				treino.setCadaEpocaStepTempera(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/porcentagemPadroesTreino")) {
				treino.setPorcentagemPadroesTreino(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/numeroEpocasTreinar")) {
				treino.setNumeroEpocasTreinar(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/dias")) {
				dias = Integer.parseInt(value.toString());
			}
			else if (tag.equalsIgnoreCase("/horas")) {
				horas = Integer.parseInt(value.toString());
			}
			else if (tag.equalsIgnoreCase("/minutos")) {
				minutos = Integer.parseInt(value.toString());
			}
			else if (tag.equalsIgnoreCase("/segundos")) {
				segundos = Integer.parseInt(value.toString());
			}
			else if (tag.equalsIgnoreCase("/tempoParada")) {
				treino.setTempoParada(dias, horas, minutos, segundos);
			}
			else if (tag.equalsIgnoreCase("/cadaEpocaValidar")) {
				validacao.setCadaEpocaValidar(Integer.parseInt(value.toString()));
			}
			else if (tag.equalsIgnoreCase("/tipoRedeValidacao")) {
				String s = value.toString();
				StoredNetworkType snt = StoredNetworkType.BestHarmonicMean;
				if (s.equalsIgnoreCase("Actual")) {
					snt = StoredNetworkType.Actual;
				}
				else if (s.equalsIgnoreCase("BestAritmethicMean")) {
					snt = StoredNetworkType.BestAritmethicMean;
				}
				else if (s.equalsIgnoreCase("BestMaxMin")) {
					snt = StoredNetworkType.BestMaxMin;
				}
				validacao.setTipoRedeValidacao(snt);
			}
			else {
				value.append(ch,start,length);
			}
		}
		
		/**termina uma tag */
		public void endElement(String uri, String localName, String tag){
			//this.tag.setLength(0);
			//this.tag.append("/");
			///this.tag.append(tag);
			this.tag = "/" + tag;
		}

		/** termina o documento */
		public void endDocument() { }
	}
	
	
	
}


