package visao.gui.telas.graficos.classes;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Image;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.util.ArrayList;
import jfan.fan.NeuronioFAN;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import thinlet.Thinlet;
import thinletcommons.AntiAliasedThinlet;
import thinletcommons.ThinletDialog;
import controle.main.App;

public class ComparateFeatureGraphic {
	private Thinlet thinlet;

	private ThinletDialog dialog;

	private Object lblGrafico;

	private JFreeChart freeChart;

	private CategoryDataset dataset = new DefaultCategoryDataset();

	private ArrayList<NeuronioFAN> array;

	private Object cmbCaracteristica;

	private Object cmbTipoNeuronio;

	private int indiceCaracteristica;

	private String[] neuronio = null;

	public ComparateFeatureGraphic(Dialog owner) {
		this.dialog = new ThinletDialog(owner, false);
		this.init();
	}

	public ComparateFeatureGraphic(Dialog owner, String title) {
		this.dialog = new ThinletDialog(owner, title, false);
		this.init();
	}

	public ComparateFeatureGraphic(Frame owner) {
		this.dialog = new ThinletDialog(owner, false);
		this.init();
	}

	public ComparateFeatureGraphic(Frame owner, String title) {
		this.dialog = new ThinletDialog(owner, title, false);
		this.init();
	}

	private void init() {
		this.thinlet = new AntiAliasedThinlet();
		Object panel = null;

		try {
			panel = thinlet.parse("/visao/gui/telas/graficos/ComparateFeatureGraphic.xml",
					this);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		thinlet.add(panel);
		dialog.setContent(thinlet);
		createChart();
		int fim = 0;
		
		try {
			double[][] numeroCarac = App.getMonitorFAN().getNeuroniosAtuais()
					.get(0).getMatrizNeural();
			fim = numeroCarac.length;

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Object choice;

		for (int i = 0; i < fim; i++) {
			choice = thinlet.create("choice");
			thinlet.setString(choice, "text", Integer.toString(i + 1));
			thinlet.add(cmbCaracteristica, choice, i);
		}
		thinlet.setInteger(cmbCaracteristica, "selected", 0);

		updateTipoNeuronio();
		dialog.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				createImage();
			}
		});
	}

	private void createImage() {
		int height = dialog.getHeight() - 60;
		if (height <= 0)
			height = 1;
		thinlet.setIcon(lblGrafico, "icon", (Image) freeChart
				.createBufferedImage(thinlet.getWidth() - 10, height));
	}

	public void show() {
		dialog.pack();
		dialog.setSize(1000, 400);
		dialog.setAlwaysOnTop(true);
		dialog.setLocationRelativeTo(dialog.getOwner());
		dialog.setVisible(true);
		createImage();
	}

	public void initParameters(Object labelImagem, Object cmbCaracteristica,
			Object cmbTipoNeuronio) {
		this.cmbCaracteristica = cmbCaracteristica;
		this.lblGrafico = labelImagem;
		this.cmbTipoNeuronio = cmbTipoNeuronio;
	}

	private void createChart() {

		// create the chart...
		freeChart = ChartFactory.createLineChart(
				"Gr�fico de Compara��o de Caracter�sticas", // chart
				// title
				"Range", // domain axis label
				"Valor", // range axis label
				dataset, // data
				PlotOrientation.VERTICAL, // orientation
				true, // include legend
				true, // tooltips
				false // urls
				);

	}

	public void updateCaracteristica() {
		indiceCaracteristica = thinlet.getSelectedIndex(cmbCaracteristica);
		updateTipoNeuronio();
	}

	public void updateTipoNeuronio() {
		int indiceTipo = thinlet.getSelectedIndex(cmbTipoNeuronio);
		((DefaultCategoryDataset) dataset).clear();
		if (indiceTipo == 0) {
			array = App.getMonitorFAN().getNeuroniosAtuais();
		}
		if (indiceTipo == 1) {
			array = App.getMonitorFAN().getNeuroniosMelhorAritmetica();
		}
		if (indiceTipo == 2) {
			array = App.getMonitorFAN().getNeuroniosMelhorHarmonica();
		}
		if (indiceTipo == 3) {
			array = App.getMonitorFAN().getNeuroniosMelhorMaximoMinimo();
		}
		int fim2 = array.size();
		int fim3 = array.get(0).getMatrizNeural()[0].length;
		double[][] array3 = new double[fim2][fim3];
		double[][] nf = null;
		for (int j=0; j<fim2;j++){
			nf=array.get(j).getMatrizNeural();
		 for (int jj=0; jj<fim3; jj++){
			 array3[j][jj]= nf[indiceCaracteristica][jj];
		 }
		}
		neuronio = new String[array3.length];
		int fim4 = array3.length;
		int fim5 = array3[0].length;
		for (int i = 0; i < fim4; i++) {
			neuronio[i] = "Neur�nio" + Integer.toString(i + 1);
			double somatorio = array.get(0).getSomatorio(indiceCaracteristica+1);
			for (int ii = 0; ii < fim5; ii++) {
		 		String columns = Integer.toString(ii);
					((DefaultCategoryDataset) dataset).addValue(array3[i][ii]/somatorio,
							neuronio[i], columns);
				}
			}
		
		createImage();
	}

	public static void main(String[] args) {
		final ComparateFeatureGraphic cfg = new ComparateFeatureGraphic(
				new Frame(), "Compara��o de Caracter�sicas");
		cfg.show();
		cfg.dialog.addWindowListener(new WindowListener() {

			public void windowActivated(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowClosed(WindowEvent e) {
				System.exit(0);

			}

			public void windowClosing(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowDeactivated(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowDeiconified(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowIconified(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowOpened(WindowEvent e) {
				// TODO Auto-generated method stub

			}

		});
	}

}
