package visao.gui.telas.graficos.classes;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Image;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.util.ArrayList;
import jfan.fan.NeuronioFAN;
import jfan.fan.utilitarias.RecalculadorClasses;
import jfan.io.arquivos.GerenciadorPadroes;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import thinlet.Thinlet;
import thinletcommons.AntiAliasedThinlet;
import thinletcommons.ThinletDialog;
import controle.main.App;

public class NeuronGraphic {
	private Thinlet thinlet;

	private ThinletDialog dialog;

	private Object lblGrafico;

	private JFreeChart freeChart;

	private CategoryDataset dataset = new DefaultCategoryDataset();

	private ArrayList<NeuronioFAN> array;

	private Object cmbNeuronio;

	private Object cmbTipoNeuronio;

	private Object cmbCaracteristica;

	private int indiceNeuronio;

	private int indiceCaracteristica;

	private String[] caracteristica = null;

	public NeuronGraphic(Dialog owner) {
		this.dialog = new ThinletDialog(owner, false);
		this.init();
	}

	public NeuronGraphic(Dialog owner, String title) {
		this.dialog = new ThinletDialog(owner, title, false);
		this.init();
	}

	public NeuronGraphic(Frame owner) {
		this.dialog = new ThinletDialog(owner, false);
		this.init();
	}

	public NeuronGraphic(Frame owner, String title) {
		this.dialog = new ThinletDialog(owner, title, false);
		this.init();
	}

	private void init() {
		this.thinlet = new AntiAliasedThinlet();
		Object panel = null;

		try {
			panel = thinlet.parse("/visao/gui/telas/graficos/NeuronGraphic.xml",
					this);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		thinlet.add(panel);
		dialog.setContent(thinlet);
		createChart();
		int fim = 0;
		int fim2 = 0;

		try {
			double[][] numeroCarac = App.getMonitorFAN().getNeuroniosAtuais()
					.get(0).getMatrizNeural();
			fim = App.getMonitorFAN().getNeuroniosAtuais().size();
			fim2 = numeroCarac.length;

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Object choice;
		GerenciadorPadroes gp = App.getGerenciadorPadroes();
		for (int i = 0; i < fim; i++) {
			choice = thinlet.create("choice");
			int classMapeada = RecalculadorClasses.getClassesMapIndexReal().get(i);
			String classe = gp.getMapaClasses().get(classMapeada);
			if (classe == null) {
				classe = Integer.toString(classMapeada);
			}
			thinlet.setString(choice, "text", classe);
			thinlet.add(cmbNeuronio, choice, i);
			thinlet.setInteger(cmbNeuronio, "selected", 0);
		}
		for (int i = 0; i < fim2; i++) {
			choice = thinlet.create("choice");
			thinlet.setString(choice, "text", Integer.toString(i + 1));
			thinlet.add(cmbCaracteristica, choice, i);
		}
		choice = thinlet.create("choice");
		thinlet.setString(choice, "text", "Todas");
		thinlet.add(cmbCaracteristica, choice, fim2);
		thinlet.setInteger(cmbCaracteristica, "selected", 0);
        
		updateTipoNeuronio();
		dialog.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				createImage();
			}
		});
	}

	private void createImage() {
		int height = dialog.getHeight() - 60;
		if (height <= 0)
			height = 1;
		thinlet.setIcon(lblGrafico, "icon", (Image) freeChart
				.createBufferedImage(thinlet.getWidth() - 10, height));
	}

	public void show() {
		dialog.pack();
		dialog.setSize(1000, 400);
		//dialog.setAlwaysOnTop(true);
		dialog.setLocationRelativeTo(dialog.getOwner());
		dialog.setVisible(true);
		createImage();
	}

	public void initParameters(Object labelImagem, Object cmbNeuronio,
			Object cmbTipoNeuronio, Object cmbCaracteristica) {
		this.cmbNeuronio = cmbNeuronio;
		this.lblGrafico = labelImagem;
		this.cmbTipoNeuronio = cmbTipoNeuronio;
		this.cmbCaracteristica = cmbCaracteristica;

	}

	private void createChart() {

		// create the chart...
		freeChart = ChartFactory.createLineChart("Grfico de Neurnio", // chart
				// title
				"Range", // domain axis label
				"Valor", // range axis label
				dataset, // data
				PlotOrientation.VERTICAL, // orientation
				true, // include legend
				true, // tooltips
				false // urls
				);

	}

	public void updateCaracteristica() {
		indiceCaracteristica = thinlet.getSelectedIndex(cmbCaracteristica);
		updateTipoNeuronio();
	}

	public void updateNeuronio() {
		indiceNeuronio = thinlet.getSelectedIndex(cmbNeuronio);
		updateTipoNeuronio();
	}

	public void updateTipoNeuronio() {
		int indiceTipo = thinlet.getSelectedIndex(cmbTipoNeuronio);
		((DefaultCategoryDataset) dataset).clear();
		if (indiceTipo == 0) {
			array = App.getMonitorFAN().getNeuroniosAtuais();
		}
		if (indiceTipo == 1) {
			array = App.getMonitorFAN().getNeuroniosMelhorAritmetica();
		}
		if (indiceTipo == 2) {
			array = App.getMonitorFAN().getNeuroniosMelhorHarmonica();
		}
		if (indiceTipo == 3) {
			array = App.getMonitorFAN().getNeuroniosMelhorMaximoMinimo();
		}
		double[][] array2 = array.get(indiceNeuronio).getMatrizNeural();
		
		caracteristica = new String[array2.length];
		if (indiceCaracteristica == array2.length) {
			for (int i = 0; i < array2.length; i++) {
				double somatorio = array.get(indiceNeuronio).getSomatorio(i+1);
				caracteristica[i] = "Caracterstica" + Integer.toString(i + 1);
				for (int ii = 0; ii < array2[0].length; ii++) {
					String columns = Integer.toString(ii);
					((DefaultCategoryDataset) dataset).addValue(array2[i][ii]/somatorio,
							caracteristica[i], columns);
				}
			}
		} else {
			double somatorio = array.get(indiceNeuronio).getSomatorio(indiceCaracteristica+1);
			for (int i = 0; i < array2[0].length; i++) {
				String columns = Integer.toString(i);
				((DefaultCategoryDataset) dataset).addValue(
						array2[indiceCaracteristica][i]/somatorio, "Caracterstica",
						columns);
			}
		}
		createImage();
	}

	public static void main(String[] args) {
		final NeuronGraphic ng = new NeuronGraphic(new Frame(),
				"Gr�fico de Neur�nio");
		ng.show();
		ng.dialog.addWindowListener(new WindowListener() {

			public void windowActivated(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowClosed(WindowEvent e) {
				System.exit(0);

			}

			public void windowClosing(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowDeactivated(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowDeiconified(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowIconified(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowOpened(WindowEvent e) {
				// TODO Auto-generated method stub

			}

		});
	}
}