package visao.gui.telas.graficos.classes;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Image;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import thinlet.Thinlet;
import thinletcommons.AntiAliasedThinlet;
import thinletcommons.ThinletDialog;
import controle.main.App;

public class StatisticGraphic {

	private Thinlet thinlet;

	private ThinletDialog dialog;

	private Object lblGrafico;

	private JFreeChart freeChart = null;

	private CategoryDataset dataset = new DefaultCategoryDataset();

	private Timer timer = new Timer(true);

	private TimerTask task = new AtualizaGrafico();

	public StatisticGraphic(Dialog owner) {
		this.dialog = new ThinletDialog(owner, false);
		this.init();
	}

	public StatisticGraphic(Dialog owner, String title) {
		this.dialog = new ThinletDialog(owner, title, false);
		this.init();
	}

	public StatisticGraphic(Frame owner) {
		this.dialog = new ThinletDialog(owner, false);
		this.init();
	}

	public StatisticGraphic(Frame owner, String title) {
		this.dialog = new ThinletDialog(owner, title, false);
		this.init();
	}

	private void init() {
		this.thinlet = new AntiAliasedThinlet();
		Object panel = null;

		try {
			panel = thinlet.parse(
					"/visao/gui/telas/graficos/StatisticGraphic.xml", this);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		thinlet.add(panel);
		dialog.setContent(thinlet);
		createChart();
		timer.scheduleAtFixedRate(task, 1000, 1000);
	}

	private void createImage() {
		
		int height = dialog.getHeight() - 30;
		if (height <= 0)
			height = 1;
		synchronized (App.getLockGraphicStatistic()) {
			
		thinlet.setIcon(lblGrafico, "icon", (Image) freeChart
				.createBufferedImage(thinlet.getWidth() - 10, height));
	}
	}
	public void show() {
		dialog.pack();
		dialog.setSize(800, 400);
		dialog.setAlwaysOnTop(true);
		dialog.setLocationRelativeTo(dialog.getOwner());
		dialog.setVisible(true);
		createImage();
	}

	public void initParameters(Object labelImagem) {
		this.lblGrafico = labelImagem;
	}

	private void createChart() {
		dataset = App.getDatasetpool().getDatasetEstatistica();
        synchronized (App.getLockGraphicStatistic()) {
			
		// create the chart...
		freeChart = ChartFactory.createLineChart("Evoluo de Erros", // chart
				// title
				"poca", // domain axis label
				"Porcentagem de Acerto", // range axis label
				dataset, // data
				PlotOrientation.VERTICAL, // orientation
				true, // include legend
				true, // tooltips
				false // urls
				);
        }
		
	}

	public static void main(String[] args) {
		final StatisticGraphic sg = new StatisticGraphic(new Frame(),
				"Grfico de Estatsticas");
		sg.show();
		sg.dialog.addWindowListener(new WindowListener() {

			public void windowActivated(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowClosed(WindowEvent e) {
				System.exit(0);

			}

			public void windowClosing(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowDeactivated(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowDeiconified(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowIconified(WindowEvent e) {
				// TODO Auto-generated method stub

			}

			public void windowOpened(WindowEvent e) {
				// TODO Auto-generated method stub

			}

		});
	}

	private class AtualizaGrafico extends TimerTask {

		@Override
		public void run() {
			createChart();
			createImage();
		}

	}
}
