package visao.gui.telas.utils;

import java.awt.Dialog;
import java.awt.Frame;
import java.io.IOException;

import thinlet.Thinlet;
import thinletcommons.AntiAliasedThinlet;
import thinletcommons.ThinletDialog;

/**
 * Creates a <i>non-modal</i> dialog for show the data in
 * EasyFAN.
 * @author Filipe Pais Lenfers
 * @version 0.7
 */
public class DataView {

	private Thinlet thinlet;
	private ThinletDialog dialog;
	private Object table;
	private Object titleLabel;
	private Object numPatterns;
	private Object numClasses;
	private Object labelPage, labelMaxPage; 
	private int pageSize = 200;
	private int page = 1;
	private String[][] matrix;
	private int maxPages;
	
	/**
	 * Instanciate a <code>DataView</code> using the param
	 * <code>owner</code> as the <code>DataView</code> parent.
	 * Renember: DataViews are non-modal.
	 * @param owner the parent of the <code>DataView</code>.
	 */
	public DataView(Dialog owner) {
		this.dialog = new ThinletDialog(owner,false);
		this.init();
	}
	
	/**
	 * Instanciate a <code>DataView</code> using the param
	 * <code>owner</code> as the <code>DataView</code> parent.
	 * Renember: DataViews are non-modal.
	 * @param owner the parent of the <code>DataView</code>.
	 * @param title the dialog title
	 */
	public DataView(Dialog owner, String title) {
		this.dialog = new ThinletDialog(owner,title,false);
		this.init();
	}
	
	/**
	 * Instanciate a <code>DataView</code> using the param
	 * <code>owner</code> as the <code>DataView</code> parent.
	 * Renember: DataViews are non-modal.
	 * @param owner the parent of the <code>DataView</code>.
	 */
	public DataView(Frame owner) {
		this.dialog = new ThinletDialog(owner,false);
		this.init();
	}
	
	/**
	 * Instanciate a <code>DataView</code> using the param
	 * <code>owner</code> as the <code>DataView</code> parent.
	 * Renember: DataViews are non-modal.
	 * @param owner the parent of the <code>DataView</code>.
	 * @param title the dialog title
	 */
	public DataView(Frame owner, String title) {
		this.dialog = new ThinletDialog(owner,title,false);
		this.init();
	}
	
	/**
	 * Initiate the DataView doing the parse of the thinlet window,
	 * and adding the thinlet to a dialog.
	 */
	private void init() {
		this.thinlet = new AntiAliasedThinlet();
		Object panel = null;
		try {
			panel = thinlet.parse("/visao/gui/telas/utils/dataview.xml", this);
		} catch (IOException e) {
			// TODO Criar mensagem de erro na constura��o
			e.printStackTrace();
			
		}
        thinlet.add(panel);
        dialog.setContent(thinlet);
	}
	
	/**
	 * Shows the DataView making it visible.
	 */
	public void show() {
		dialog.pack();
		//dialog.setSize(600, 400);
		dialog.setAlwaysOnTop(true);
        dialog.setLocationRelativeTo(dialog.getOwner());
        dialog.setVisible(true);
        labelMaxPage = thinlet.find("pageMax");
        labelPage = thinlet.find("page");
        if (this.matrix != null) draw();
	}
	
	/**
	 * This method sets the widgets to the DataView handle.
	 * Use this method in the xml that specifies the thinlet windows.
	 * @param table the table where the data gonna be inserted
	 * @param titleLabel the title label, its diferent from the window title
	 * @param numPatterns the label where gonna be the number of registers in the table 
	 * @param numClasses the label where gonna be the number of classes in the table
	 */
	public void initParameters(Object table, Object titleLabel, Object numPatterns, Object numClasses) {
		this.table = table;
		this.titleLabel = titleLabel;
		this.numPatterns = numPatterns;
		this.numClasses = numClasses;
	}
	
	/**
	 * Defines the text in the title label.
	 * Remark: The title label, its diferent from the window title
	 * @param text the text to be in the title label
	 */
	public void setTitleLabel(String text) {
		thinlet.setString(this.titleLabel, "text", text);
	}
	
	/**
	 * Defines the number of classes that gonna be showed in the label.
	 * @param num the number of distinc classes.
	 */
	public void setNumClasses(int num) {
		thinlet.setString(this.numClasses, "text", Integer.toString(num));
	}
	
	/**
	 * Update the label whith the count of the rows. 
	 */
	public void updatePatternsCount() {
		thinlet.setString(this.numPatterns, "text", Integer.toString(thinlet.getCount(table)));
	}
	
	/**
	 * Add a row in the table
	 * @param strings the row, is a vararg
	 */
	public void addRow(final String... strings) {
		//Runnable rodar = new Runnable() {
         //public void run() {
           	Object header = Thinlet.create("header");
       	    int qtdClasses = 0;
			
        	//*************Colocando cabe�alho nas colunas*******************************************
		    for (int coluna = 0; coluna < strings.length -1 ; coluna++) {
		    	qtdClasses ++;
		    	Object column = Thinlet.create("column");
				String carac = "Caracteristica ";
				carac += coluna+1;
				thinlet.setString(column,"text", carac );
				thinlet.setInteger(column,"width",150);
				thinlet.add(header, column);
			}
		    thinlet.setString(thinlet.find("caracTrN"),"text",Integer.toString(qtdClasses)) ;    
		    Object column = Thinlet.create("column");
			thinlet.setString(column,"text", "Classe" );
			thinlet.add(header, column);
			thinlet.add(table, header);
		    //***************************************************************************************
		   Object row = Thinlet.create("row");
	       Object cell;
		   for (String s : strings ) {
			  cell = Thinlet.create("cell");
			  thinlet.setString(cell,"text", s);
			  thinlet.add(row, cell);
		   }
		   thinlet.add(table, row);
		
    		//}
    		//};
		//Thread thread = new Thread(rodar);
  	    //thread.start();
  	    
  	  	}
	
	/**
	 * Add all the rows from a table(double dimension array).
	 * @param table the double dimensioned array.
	 */
	public void addRows(final String[][] table) {
		this.matrix = table;
		this.page = 1;
		calculateMaxPages();
		draw();
	}
	
	/**
	 * Draws the itens in the table, based on the page and the 
	 * page size.
	 */
	public void draw() {
		thinlet.setString(labelMaxPage, "text", Integer.toString(this.maxPages));
        thinlet.setString(labelPage, "text", Integer.toString(this.page));
		int i = (this.page-1) * this.pageSize;
		int fim1 = (this.page) * this.pageSize;
		int fim2 = this.matrix.length;
		this.clearTable();
		int qtdPadroes = 0;
		for(; i < fim1 && i < fim2;i++) {
			addRow(this.matrix[i]);
			qtdPadroes++;
		}
		thinlet.setString(thinlet.find("padroesTrN"),"text",Integer.toString(qtdPadroes)) ;
	}

	/**
	 * Clear all the rows in the table.
	 * @param table
	 */
	public void clearTable() {
		thinlet.removeAll(table);
	}
		
	/**
	 * N�mero serial para serializa��o gerado pelo eclipse.
	 */
	private static final long serialVersionUID = 5195907069388087662L;
	
	/**
	 * PAra testes...
	 * @param args
	 */
	public static void main(String[] args) {
		DataView dw = new DataView(new Frame(),"Teste");
		//Adicinando linha via array
		dw.addRow(new String[]{"re","123","1we1123", "as"});
		//ADicionando linha passando qualquer numero de strings por virgula
		dw.addRow("123","1sadsad23","1we1123", "a333");
		//adionando um array bidimensional
		dw.addRows(new String[][]{{"re","123","1we1123", "as"},{"123","1sadsad23","1we1123", "a333"}});
		dw.setTitleLabel("Um Teste");
		dw.setNumClasses(3);
		dw.show();
		dw.addRows(new String[][]{{"re","123","1we1123", "as"},{"123","1sadsad23","1we1123", "a333"}});
		dw.updatePatternsCount();
		
		while (dw.dialog.isShowing()) {}
		dw.dialog.dispose();
		
		System.exit(0);
		
	}

	public int getPage() {
		return this.page;
	}

	public void setPage(int page) {
		this.page = page;
		draw();
	}

	public int getPageSize() {
		return this.pageSize;
	}

	public void setPageSize(int pageSize) {
		this.pageSize = pageSize;
		calculateMaxPages();
		draw();
	}
	
	public void nextPage() {
		if (page < maxPages)
			setPage(++this.page);
	}
	
	public void beforePage(){
		if (page > 1)
			setPage(--this.page);
	}
	
	private void calculateMaxPages() {
		this.maxPages = (this.matrix.length / this.pageSize);
		if ((this.matrix.length % this.pageSize) > 0) this.maxPages++;
	}

}
