package visao.gui.telas.utils;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.io.File;
import java.io.IOException;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

import thinletcommons.AntiAliasedThinlet;
import thinletcommons.ThinletDialog;

public class ImageSequencePlayer {

	final String pasta;
	
	final Toolkit toolkit = Toolkit.getDefaultToolkit();
	
	private int imageNumBuffer = 1;
	
	private int imageNumPlay = 1;
	
	final private String ext,prefix,sufix, inicioArq, fimArq; 
	
	final private long delay;
	
	final private int bufferTamanho = 40;
	
	MediaTracker md = new MediaTracker(null);
	
	private final LinkedBlockingQueue<Image> bufferImagens = new LinkedBlockingQueue<Image>(bufferTamanho);
	
	private AntiAliasedThinlet thinlet;
	private ThinletDialog dialog;
	private Object lblImagem;
	
	private Produtor produtor;
	private Thread threadProdutor;
	private Consumidor consumidor;
	private Thread threadConsumidor;
	
	public ImageSequencePlayer(Dialog owner,String title,String pasta, String ext, long delay, String prefix, String sufix) {
		dialog = new ThinletDialog(owner,title,false);
		init();
		this.pasta = pasta;
		this.ext = ext;
		inicioArq = pasta + prefix;
		this.delay = delay;
		this.prefix = prefix;
		this.sufix = sufix;
		fimArq = sufix + ext;
		
		thinlet.setString(lblImagem, "text", "Carregando video...");
		
		for (;imageNumBuffer < bufferTamanho; imageNumBuffer++ ) {
			carregarImagemBuffer(imageNumBuffer);
		}	
		
		/*try {
			md.waitForAll();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		md.checkAll(true);
		while (md.checkAll()) {
			Thread.yield();
		}*/
		
		desenharImagemAtual();
		
	}
	
	public ImageSequencePlayer(Frame owner,String title,String pasta, String ext, long delay, String prefix, String sufix) {
		this.dialog = new ThinletDialog(owner,title,false);
		init();
		this.pasta = pasta;
		this.ext = ext;
		inicioArq = pasta + prefix;
		this.delay = delay;
		this.prefix = prefix;
		this.sufix = sufix;
		fimArq = sufix + ext; 
		
		for (;imageNumBuffer < bufferTamanho; imageNumBuffer++ ) {
			carregarImagemBuffer(imageNumBuffer);
		}
		desenharImagemAtual();
		
		//thinlet.setInteger(lblImagem, "width", bufferImagens.peek().getWidth(null));
		//thinlet.setInteger(lblImagem, "height", bufferImagens.peek().getHeight(null));
	}
	
	private void init() {
		this.thinlet = new AntiAliasedThinlet();
		Object panel = null;
		try {
			panel = thinlet.parse("/visao/gui/telas/utils/player.xml", this);
		} catch (IOException e) {
			e.printStackTrace();
		}

        thinlet.add(panel);
        dialog.setContent(thinlet);
	}

	public void initParameters(Object lblImagem) {
		this.lblImagem = lblImagem;
	}
	
	public void play() {
		imageNumPlay++;
		if (threadProdutor == null) {
			produtor = new Produtor();
			threadProdutor = new Thread(produtor);
			consumidor = new Consumidor();
			threadConsumidor = new Thread(consumidor);
			threadProdutor.start();
			threadConsumidor.start();
		}
		else if (!threadProdutor.isAlive()) {
			produtor = new Produtor();
			threadProdutor = new Thread(produtor);
			consumidor = new Consumidor();
			threadConsumidor = new Thread(consumidor);
			threadProdutor.start();
			threadConsumidor.start();
		}
		//notifyAll();
	}
	
	public void stop() {
		
		produtor.stop();
		consumidor.stop();
		produtor = null;
		consumidor = null;
		threadConsumidor = null;
		threadProdutor = null;
		imageNumBuffer = 1; 
		imageNumPlay = 1;
		bufferImagens.clear();
		carregarImagemBuffer(1);
		desenharImagemAtual();
	}
	
	public void pause() {
		//try {
			//threadProdutor.wait();
		produtor.stop();
		consumidor.stop();
		produtor = null;
		consumidor = null;
		threadConsumidor = null;
		threadProdutor = null;
			//threadConsumidor.wait();
		//} catch (InterruptedException e) {
			//e.printStackTrace();
		//}
	}
	
	private void carregarImagemBuffer(int i) {
		try {
			String arquivo = inicioArq + i + fimArq;
			File file = new File(arquivo);
			//System.out.println(file.getAbsolutePath());
			//System.out.println(file.exists());
			
			if (file.exists()) {
				Image img = toolkit.createImage(arquivo);
				//img = bufferImage(img);
				//md.addImage(img, i);
				bufferImagens.put(img);
			}
			else {
				produtor.stop();
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	private void desenharImagemAtual() {
		//try {
			thinlet.setIcon(lblImagem, "icon", bufferImagens.poll());
			//dialog.repaint();
		//} catch (InterruptedException e) {
			//e.printStackTrace();
		//}
	}
	
	public void show() {
		//dialog.pack();
		dialog.setSize(1000, 600);
		//dialog.setAlwaysOnTop(true);
		dialog.setModal(true);
		dialog.setLocationRelativeTo(dialog.getOwner());
		dialog.setVisible(true);
	}

	
	//-----------------------------------------------------------------
	private class Produtor implements Runnable {

		private boolean continuar = true;
		
		public void run() {
			while(continuar) {
				carregarImagemBuffer(++imageNumBuffer);
			}
		}
			
		public void stop() {
			continuar = false;
		}
		
	}
	//------------------------------------------------------------------
	public class Consumidor implements Runnable {

		private boolean continuar = true;
		
		public void run() {
			while(continuar) {
				desenharImagemAtual();
				imageNumPlay++;
				try {
					Thread.sleep(delay);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}	
		}
		
		public void stop() {
			continuar = false;
		}
	}
	//------------------------------------------------------------------
	
	private static final int DEFAULT_IMAGE_TYPE = BufferedImage.TYPE_INT_RGB;

    public BufferedImage bufferImage(Image image) {
        return bufferImage(image, DEFAULT_IMAGE_TYPE);
    }

    public BufferedImage bufferImage(Image image, int type) {
        BufferedImage bufferedImage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
        Graphics2D g = bufferedImage.createGraphics();
        g.drawImage(image, null, null);
        waitForImage(bufferedImage);
        return bufferedImage;
    }

    private void waitForImage(BufferedImage bufferedImage) {
        final ImageLoadStatus imageLoadStatus = new ImageLoadStatus();
        bufferedImage.getHeight(new ImageObserver() {
            public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
                if (infoflags == ALLBITS) {
                    imageLoadStatus.heightDone = true;
                    return true;
                }
                return false;
            }
        });
        bufferedImage.getWidth(new ImageObserver() {
            public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
                if (infoflags == ALLBITS) {
                    imageLoadStatus.widthDone = true;
                    return true;
                }
                return false;
            }
        });
        while (!imageLoadStatus.widthDone && !imageLoadStatus.heightDone) {
            try {
                Thread.sleep(300);
            } catch (InterruptedException e) {

            }
        }
    }

    class ImageLoadStatus {

        public boolean widthDone = false;
        public boolean heightDone = false;
    }
	
	public static void main(String[] args) {
		
		ImageSequencePlayer player = new ImageSequencePlayer(new Frame(),"Video Teste","src/visao/gui/telas/utils/testeVideo/", ".jpeg", 150l, "", "");
		player.show();
		
	}
}
