package jfan.fan;

/**
 * Classe responsvel pelos testes da rede FAN, testa um
 * padro  emite a opinio da rede.
 */
import java.util.ArrayList;

import jfan.fan.padroes.IPadrao;


/**
 * Classe responsvel pelos testes da rede FAN, testa um
 * padro  emite a opinio da rede.
 * @author Filipe Pais Lenfers
 * @version 0.2.8
 */
public class TesteFAN {
	
	/**
	 * ArrayList que mantm os neurnios nos quais o teste
	 * se basear.
	 */
	private ArrayList<NeuronioFAN> neuronios = null;
	
	public TesteFAN() {}
	
	/**
	 * Instancia TesteFAN para usar usando os neurnios
	 * que sero usados como parmetros.
	 * @param neuronios Os neurnios que sero usados par ao teste.
	 */
	public TesteFAN(ArrayList<NeuronioFAN> neuronios) {
		this.neuronios = neuronios;
	}
	
	/**
	 * Classifica um padro baseado nos neurnios.
	 * @param p O padro para o qual ser calculada a classe.
	 * @return A classe que a rede classifica o padro.
	 */
	public NeuronioFAN classificar(IPadrao p) {
		NeuronioFAN melhorNeuronio = this.neuronios.get(0);
		double maiorForca = melhorNeuronio.determinarForca(p);
		double forcaAtual;
		for (int i= 1;i<this.neuronios.size();i++) {
			forcaAtual = this.neuronios.get(i).determinarForca(p);
			if (maiorForca < forcaAtual) {
				melhorNeuronio = this.neuronios.get(i);
				maiorForca = forcaAtual;
			}
		}
		return melhorNeuronio;
	}
	
	/**
	 * Troca os neurnios que sero usados pelo objeto de TesteFAN.
	 * @param neurons Os novos neurnios que este objeto ir usar.
	 */
	public void setNeuronios(ArrayList<NeuronioFAN> neurons) {
		this.neuronios = neurons;
	}
}
