package jfan.fan;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

import jfan.fan.padroes.IPadrao;
import jfan.fan.temperas.ITemperaSimulada;

public class TreinoQueue extends TreinoFAN implements ITreinador {

	LinkedBlockingQueue<Tarefa>[] filas;
	
	TreinaFila[] treinaFila;
	
	Thread[] threadsTreinaFila;
	
	public void treinar(ArrayList<IPadrao> padroes) {
		for (int i = 0; i < treinaFila.length; i++) {
			treinaFila[i] = new TreinaFila();
			treinaFila[i].fila = filas[i];
			threadsTreinaFila[i] = new Thread(treinaFila[i]);
			threadsTreinaFila[i].start();
		}
		
		for(IPadrao p : padroes ) {
			enfileirar(p);
			Thread.yield();
		}
		
			Thread.yield();
		
		for (TreinaFila t : treinaFila) {
			t.keepGoing = false;
		}
		
		while(temThreadViva()) {
			Thread.yield();
		}
	}
	
	private boolean temThreadViva(){
		for (Thread t : threadsTreinaFila) {
			if( t.isAlive() ) return true;
		}
		
		return false;
	}
	
	private void enfileirar(IPadrao p){
		
		NeuronioFAN n = testeFan.classificar(p);//determinaNeuronio(classeRede);
		if (n.getClasseAssociada() != p.getClasse()) {
			//define se haver penalizao ou no, quanto maior o peso menor a chance de penalizao.
			if (n.getPesoPenalizacao() >= 1000 || random.nextInt(1001) > n.getPesoPenalizacao()) {
				//penalizar(n, p);
				Tarefa task = new Tarefa();
				task.alvo = n;
				task.padrao = p;
				task.tipoTarefa = TipoTarefa.penalizar;
				try {
					filas[n.getClasseAssociada()].put(task);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		//Esse if abaixo tambem sorteia a chance de reforo, quanto maior o peso maior a chance de reforo
		//if ((f.getPesoPenalizacao() >= 1000) || (random.nextInt(1001) <= f.getPesoPenalizacao()))
		Tarefa task = new Tarefa();
		task.alvo = null;
		task.padrao = p;
		task.tipoTarefa = TipoTarefa.reforcar;
		try {
			filas[p.getClasse()].put(task);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//reforcar(p);
		
	}
	
	@SuppressWarnings("unchecked")
	public void setNeuronios(ArrayList<NeuronioFAN> neurons) {
		super.setNeuronios(neurons);
		filas = (LinkedBlockingQueue<Tarefa>[]) Array.newInstance(LinkedBlockingQueue.class, neurons.size());
		treinaFila = new TreinaFila[neurons.size()];
		threadsTreinaFila = new Thread[neurons.size()];
		for (int i = 0; i < filas.length; i++) {
			filas[i] = new LinkedBlockingQueue<Tarefa>();
		}
	}
	
	
	//----Classes/Enums de apoio------------
	
	private class TreinaFila implements Runnable {

		private LinkedBlockingQueue<Tarefa> fila;
		
		private boolean keepGoing = true; 
		
		public void run() {
			while (keepGoing || fila.size() > 0) {
				if (fila.size() > 0){
					try {
						Tarefa task = fila.poll(200, TimeUnit.MILLISECONDS);
						if (task.tipoTarefa == TipoTarefa.reforcar) {
							//Fazer o reforo
							reforcar(task.padrao);
						}
						else if (task.tipoTarefa == TipoTarefa.penalizar) {
							//Fazer a penalizao
							penalizar(task.alvo, task.padrao);
						}
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						System.out.println("Ocorreu um erro de interrupo inesperada.");
						e.printStackTrace();
					}
				}
				else {
					//try {
						Thread.yield();
					//} catch (InterruptedException e) {
						//TODO ver isso
					//	System.out.println("Ocorreu um erro de interrupo inesperada.");					
					//	e.printStackTrace();
					//}
				}
			}
		}
		
	}
	
	private enum TipoTarefa {
		reforcar, penalizar
	}
	
	private class Tarefa {
		private TipoTarefa tipoTarefa;
		private IPadrao padrao;
		private NeuronioFAN alvo;
	}
	//-----------------------
}
