package jfan.fan.normalizadores;

import java.util.ArrayList;

import jfan.fan.padroes.IPadrao;

public class NormalizatorMaxMean implements INormalizator, Runnable {
	
	private double[] max;
	private double[] mean;
	private int caracEscolhida;
	private ArrayList<IPadrao> conjuntoArmazenado;

	public NormalizatorMaxMean(NormalizatorConfiguration config) {
		max = config.getMaximos();
		mean = config.getMedias();
	}
	
	public NormalizatorMaxMean(){}

	public void normalizar(ArrayList<IPadrao> padroes) {
		for (IPadrao p : padroes) normalizar(p);
	}

	public void normalizar(ArrayList<IPadrao> padroes, int caracteristica) {
		for (IPadrao p : padroes) normalizar(p, caracteristica);
	}

	public void normalizar(IPadrao p) {
		int quantas = p.getQuantasCaracteristicas();
		for (int i = 1; i <= quantas; i++) normalizar(p,i);
	}

	public void normalizar(IPadrao p, int caracteristica) {
		double caracNorm = p.getCaracteristica(caracteristica) - mean[caracteristica-1];
		caracNorm /= (max[caracteristica-1] - mean[caracteristica-1]);
		caracNorm++;
		caracNorm /= 2;
		p.setCaracteristicaNormalizada(caracteristica, caracNorm);
	}

	public void normalizarThreaded(ArrayList<IPadrao> padroes) {
		if (padroes.size() <= 0) return;
		int quantCarac = padroes.get(0).getQuantasCaracteristicas();
		Thread[] threads = new Thread[quantCarac];
		NormalizatorMaxMean nm;
		for (int i = 1; i<= quantCarac; i++){
			nm = new NormalizatorMaxMean();
			nm.conjuntoArmazenado = padroes;
			nm.caracEscolhida = i;
			nm.setValoresMaximosCaracteristicas(max);
			nm.setValoresMediasCaracteristicas(mean);
			threads[i-1] = new Thread(nm);
			threads[i-1].start();
		}
		boolean loop;
		do {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			loop = false;
			for (Thread t : threads) {
				loop = loop || t.isAlive();
			}
		}
		while(loop);
	}

	public void setValorMaximoCaracteristica(int caracteristica, double valor) {
		setValorCaracteristica(caracteristica, valor, max);
	}

	public void setValorMinimoCaracteristica(int caracteristica, double valor) {
		// TODO Auto-generated method stub

	}

	public void setValoresMaximosCaracteristicas(double[] valores) {
		max = valores;
	}

	public void setValoresMinimosCaracteristicas(double[] valores) {
		// TODO Auto-generated method stub
	}

	public void run() {
		normalizar(conjuntoArmazenado, caracEscolhida);
	}
	
	private void setValorCaracteristica(int caracteristica, double valor, double[] array) {
		if(array == null) {
			array = new double[caracteristica];
		}
		else if (array.length < caracteristica) {
			ArrayList<Double> dtemp = new ArrayList<Double>(caracteristica);
			for(Double d : array) dtemp.add(d);
			for (int i = dtemp.size(); i < caracteristica-1; i++) dtemp.add(0.0);
			dtemp.add(caracteristica-1, valor);
			array = new double[dtemp.size()];
			for (int i = 0; i < array.length; i++) {
				array[i] = dtemp.get(i);
			}
		}
		else {
			ArrayList<Double> dtemp = new ArrayList<Double>(caracteristica);
			for(Double d : array) dtemp.add(d);
			if (dtemp.get(caracteristica-1)==null) {
				dtemp.add(caracteristica-1, valor);
			}
			else {
				dtemp.set(caracteristica-1, valor);
			}
			array = new double[dtemp.size()];
			for (int i = 0; i < array.length; i++) {
				array[i] = dtemp.get(i);
			}
		}
	}

	public void setValorMedioCaracteristica(int caracteristica, double valor) {
		setValorCaracteristica(caracteristica, valor, mean);
	}

	public void setValoresMediasCaracteristicas(double[] valores) {
		mean = valores;
	}

}
