package jfan.fan.utilitarias;

import java.util.ArrayList;

import jfan.fan.padroes.IPadrao;


/**
 * Classe que calcula as caractersticas FAN de um conjunto de padres,
 * a classe foi feita para permitir a execuo paralela.
 * @author Filipe Pais Lenfers
 * @version 0.7.9
 */
public class CalcularCaracteristicasFANThread implements Runnable {
	/**
	 * A caractersca selecionada para ser calculada.
	 */
	private int caracteristicaSelecionada;

	/*
	 * Outra opo  converter o Vector/ArrayList para um array para tentar
	 * ficar mais rpido mais aumentando o uso de memria
	 */
	// private Vector<Padrao> padroes = null;
	/**
	 * O conjunto de padres que ter a a caracterstica calculada.
	 */
	private ArrayList<IPadrao> padroes = null;

	/**
	 * O raio difuso da rede.
	 */
	private int raioDifuso;

	/**
	 * O parmetro de suporte a conjuntos difusos da rede.
	 */
	private int J;

	/**
	 * Instancia a classe para pronta execuo do mtodo run.
	 * @param padroes o conjunto de padres que tero as caracteristicasFAN calculadas
	 * @param numCaracteristica a caracteristica que ser calculada, que deve ser maior que 0
	 * @param raioDifuso o raio difuso da rede
	 * @param suporteConjuntoDifuso o nmero de suporte a conjunto difuso da rede
	 */	
	public CalcularCaracteristicasFANThread(ArrayList<IPadrao> padroes, int numCaracteristica, int raioDifuso, int suporteConjuntoDifuso) {
		this.caracteristicaSelecionada = numCaracteristica;
		this.padroes = padroes;
		this.raioDifuso = raioDifuso;
		this.J = suporteConjuntoDifuso;
	}

	/**
	 * Retorna a caracterstica que foi selecionada.
	 * @return a caracterstica que foi selecionada.
	 */
	public int getCaracteristica() {
		return this.caracteristicaSelecionada;
	}

	/**
	 * Retorna o conjunto de padres que foi ou ser utilizado. 
	 * @return o conjunto de padres que foi ou ser utilizado.
	 */
	public ArrayList<IPadrao> getPadroes() {
		return this.padroes;
	}

	/**
	 * Executa o calculo das caracteristicasFAN de forma paralela.
	 */
	public void run() {
		CalcularCaracteristicasFANBatch calc = new CalcularCaracteristicasFANBatch(this.raioDifuso,this.J);;
		for (IPadrao p : this.padroes)
				calc.calcularCaracteristicaFAN(p,this.caracteristicaSelecionada);
	}
}
