package jfan.io.arquivos;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Vector;

import jfan.fan.padroes.IPadrao;
import jfan.fan.padroes.PadraoNormalizado;
import jfan.io.ILeitura;

/**
 * @author Fabiano
 *
 */
public class CarregaDat implements ILeitura {

	/**
	 * Contm os padres normalizados
	 */
	public ArrayList<IPadrao> padroes = null;

	/**
	 * Define o tipo de separador de colunas no arquivo de padres
	 */
	private String separadorColuna = "  ";
	
	/**
	 * Define o nome de arquivo de padres
	 */
	private String nomeArquivo = null;
	
	
	/**
	 * Construtor da classe CarregaDat
	 * @param umNomeDeArquivo
	 *	Um arquivo com padres
	 *
	 * @param umSeparadorDeColuna
	 *	Um separador de coluna
	 *
	 */
	public CarregaDat(String umNomeDeArquivo, String umSeparadorDeColuna) {
		this.nomeArquivo = umNomeDeArquivo;
		this.separadorColuna = umSeparadorDeColuna;
	}

	public int getNumClasses() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(nomeArquivo));
		ArrayList<Double> asClasses = new ArrayList<Double>();
		String linha = "";
		int contadorLinhas = 0;
		while ((linha = arqDados.readLine()) != null) {
			StringTokenizer t = new StringTokenizer(linha, separadorColuna);
			int contadorTokens = 0;
			while (t.hasMoreTokens()) {
				if (t.nextToken().length() != 0
						&& contadorTokens == t.countTokens())
					asClasses.add(Double.parseDouble(t.nextToken()));
				else
					t.nextToken();
				contadorTokens += 1;
			}
			contadorLinhas += 1;
		}
		ArrayList<Double> achados = new ArrayList<Double>();
		for (int i = 0; i < asClasses.size(); i++)
			if (!achados.contains(asClasses.get(i)))
				achados.add(asClasses.get(i));
		arqDados.close();
		return achados.size();
	}

	public int[] getClasses() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(nomeArquivo));
		ArrayList<Double> asClasses = new ArrayList<Double>();
		String linha = "";
		int contadorLinhas = 0;
		while ((linha = arqDados.readLine()) != null) {
			StringTokenizer t = new StringTokenizer(linha, separadorColuna);
			int contadorTokens = 0;
			while (t.hasMoreTokens()) {
				if (t.nextToken().length() != 0
						&& contadorTokens == t.countTokens())
					asClasses.add(Double.parseDouble(t.nextToken()));
				else
					t.nextToken();
				contadorTokens += 1;
			}
			contadorLinhas += 1;
		}
		ArrayList<Double> achados = new ArrayList<Double>();
		for (int i = 0; i < asClasses.size(); i++)
			if (!achados.contains(asClasses.get(i)))
				achados.add(asClasses.get(i));
		arqDados.close();
        int[] classes = new int[achados.size()];
        for (int i=0;i<achados.size();i++)
        	classes[i]=achados.get(i).intValue();
		return classes;
	}

	public int getNumCaracteristicas() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(nomeArquivo));
		String linha = "";
		linha = arqDados.readLine();
		StringTokenizer t = new StringTokenizer(linha, separadorColuna);
		arqDados.close();
		return t.countTokens() - 1;
	}

	public int getNumPadroes() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(nomeArquivo));
		int c = 0;
		while ((arqDados.readLine()) != null)
			c += 1;
		arqDados.close();
		return c;
	}
	

	public ArrayList<IPadrao> getPadroes() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(nomeArquivo));
		String linha = "";
		Vector<Double> asCaracteristicas = new Vector<Double>();
		int classe = 0;
		padroes = new ArrayList<IPadrao>();
		padroes.clear();
		while ((linha = arqDados.readLine()) != null) {
			StringTokenizer t = new StringTokenizer(linha, separadorColuna);
			while (t.hasMoreTokens())
				asCaracteristicas.add(Double.parseDouble(t.nextToken()));
			classe = asCaracteristicas.lastElement().intValue();
			asCaracteristicas.removeElementAt(asCaracteristicas.size() - 1);
			asCaracteristicas.trimToSize();

			double[] copiaDasCaracteristicas = new double[asCaracteristicas.size()];
			for (int j = 0; j < asCaracteristicas.size(); j++)
				copiaDasCaracteristicas[j] = asCaracteristicas.elementAt(j);

			padroes.add(new PadraoNormalizado(copiaDasCaracteristicas, classe));
			asCaracteristicas.removeAllElements();
			padroes.trimToSize();
		}
		arqDados.close();
		return padroes;
	}
	
	public void setArquivo(String umArquivo) throws Exception {
		this.nomeArquivo = umArquivo;
	}

	public void setSeparadorColuna(String umSeparadorColuna) throws Exception {
		this.separadorColuna = umSeparadorColuna;
	}

	public int getNumLinhasArquivo() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(nomeArquivo));
		int contadorLinhas = 0;
        while ((arqDados.readLine())!=null)
        		contadorLinhas+=1;
        arqDados.close();
        return contadorLinhas;
	}

	public int getNumColunasArquivo() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(nomeArquivo));
		int contadorColunas = 0;
		String linha = "";
        while ((linha = arqDados.readLine())!=null){
        	StringTokenizer t = new StringTokenizer(linha,separadorColuna);
        	contadorColunas=t.countTokens();
        	break;
        }
        arqDados.close();
        return contadorColunas;
	}

	public String[][] getPadroesOriginais() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(nomeArquivo));
		String linha = "";
		Vector<String> umPadrao = new Vector<String>();
		String[][] arrayPadroes = new String[getNumLinhasArquivo()][getNumColunasArquivo()];
		int contadorLinhas = 0;
		while ((linha = arqDados.readLine())!=null){
			contadorLinhas+=1;
			StringTokenizer t = new StringTokenizer(linha,separadorColuna);
			while (t.hasMoreTokens())
				umPadrao.add(t.nextToken());
			for (int i=0;i<umPadrao.size();i++)
				arrayPadroes[contadorLinhas-1][i]=umPadrao.elementAt(i);
			umPadrao.removeAllElements();
		}
		arqDados.close();
		return arrayPadroes;
	}

}
