package jfan.io.arquivos;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Vector;

import jfan.fan.NeuronioFAN;

/**
 * Esta classe tem por finalidade carregar redes j� treinadas pelo LabFan
 * afim de manter compatibilidade com o mesmo
 * @version 0.0.1
 * @author Fabiano
 *
 */
public class CarregaNeuronioFAN {
	
	/**
	 * Define o nome de arquivo de Neur�nios 
	 */
	private String nomeArquivo = null;

	/**
	 * Define o separador de colunas do arquivo de Neur�nios 
	 */
	private String separadorColuna = null;
	
	int numSuporteConjuntoDifuso;
	
	int raioDifuso;
	
	
	/**
	 * Construtor da classe CarregaNeuronioFAN
	 * @param umArquivoDeNeuronios
	 *	Um arquivo com padr�es
	 *
	 * @param umSeparadorDeColunao
	 *	Um separador de coluna
	 *
	 */
	public CarregaNeuronioFAN(String umArquivoDeNeuronios, String umSeparadorDeColuna, int raioDifuso, int suporteConjuntosDifusos) throws Exception{
		setNomeArquivo(umArquivoDeNeuronios);
		setSeparadorColuna(umSeparadorDeColuna);
		this.raioDifuso = raioDifuso;
		this.numSuporteConjuntoDifuso = suporteConjuntosDifusos;
	}
	
	/**
	 * 
	 * @param umArquivo Um arquivo com padr�es
	 */
	public void setNomeArquivo(String umArquivo) throws Exception {
		this.nomeArquivo = umArquivo;
	}

	/**
	 * @return Retorna uma string com o nome do arquivo
	 *	
	 */
	public String getNomeArquivo() throws Exception {
		return this.nomeArquivo;
	}

	/**
	 * @param Define qual o caracter usado como separador dos dados 
	 *	
	 */
	public void setSeparadorColuna(String umSeparadorDeColuna) throws Exception {
		this.separadorColuna = umSeparadorDeColuna;
	}

	/**
	 * @return Retorna qual o caracter usado como separador dos dados 
	 *	
	 */
	public String getSeparadorColuna(){
		return this.separadorColuna;
	}
	

	/**
	 * @return Retorna um array com a matriz neuronal 
	 *	
	 */
	public Double[][] getDadosArquivoNeuronio() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(getNomeArquivo()));
		String linha = "";
		Vector<Double> umaTupla = new Vector<Double>();
		Double[][] arrayNeuronios = new Double[getNumLinhasArquivo()][getNumColunasArquivo()];
		int contadorLinhas = 0;
		while ((linha = arqDados.readLine())!=null){
			contadorLinhas+=1;
			StringTokenizer t = new StringTokenizer(linha,getSeparadorColuna());
			while (t.hasMoreTokens()) {
				@SuppressWarnings("unused")
				final String s = t.nextToken();
				umaTupla.add(Double.parseDouble(t.nextToken()));
			}
						
			for (int i=0;i<umaTupla.size();i++)
				arrayNeuronios[contadorLinhas-1][i]=umaTupla.elementAt(i);
			umaTupla.removeAllElements();
		}
		arqDados.close();
		return arrayNeuronios;
	}
	
	/**
	 * @return Retorna o numero de linhas que o arquivo possui 
	 *	
	 */
	public int getNumLinhasArquivo() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(getNomeArquivo()));
		int contadorLinhas = 0;
        while ((arqDados.readLine())!=null)
        		contadorLinhas+=1;
        arqDados.close();
        return contadorLinhas;
	}

	/**
	 * @return Retorna o numero de colunas que o arquivo possui 
	 *	
	 */
	public int getNumColunasArquivo() throws Exception {
		BufferedReader arqDados = new BufferedReader(new FileReader(getNomeArquivo()));
		int contadorColunas = 0;
		String linha = "";
        while ((linha = arqDados.readLine())!=null){
        	StringTokenizer t = new StringTokenizer(linha,getSeparadorColuna());
        	contadorColunas=t.countTokens();
        	break;
        }
        arqDados.close();
        return contadorColunas;
	}

	/**
	 * @return Retorna um ArrayList contendo os objetos neuronio 
	 *	
	 */
	public ArrayList<NeuronioFAN> getNeuroniosFAN() throws Exception {
		Double[][] arrayNeuronios = getDadosArquivoNeuronio();
		int numDeCaracteristicas = arrayNeuronios[0].length;
		//int numSuporteConjuntoDifuso = 99;
		//int raioDifuso = 6;
		int numDeClasses = (arrayNeuronios.length+1)/100;//perigoso
		ArrayList<NeuronioFAN> listaComNeuronios  = new ArrayList<NeuronioFAN>();
		listaComNeuronios.clear();
		
		int aLinhaNoArray = 0;
		for (int classe=1;classe<=numDeClasses;classe++){
			NeuronioFAN neu = new NeuronioFAN(numDeCaracteristicas,numSuporteConjuntoDifuso,raioDifuso,classe);
			neu.inicializarZerado();
			for (int caracteristica=0;caracteristica<numDeCaracteristicas;caracteristica++){
				int i = 0;
				int fim = aLinhaNoArray+numSuporteConjuntoDifuso;
				for (int posicao = aLinhaNoArray; posicao < fim; posicao++){
					neu.setValor(caracteristica+1, i , arrayNeuronios[posicao][caracteristica]);
					i+=1;
				}
				neu.setSomatorio(caracteristica, arrayNeuronios[aLinhaNoArray+numSuporteConjuntoDifuso][caracteristica]);
			}
			listaComNeuronios.add(neu);
			aLinhaNoArray+=100;			
		}
		return listaComNeuronios;
	}
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		//Tests
		try {
			/*
			CarregaNeuronioFAN cnf = new CarregaNeuronioFAN("c:\\patterns\\neuro.fan"," ");
			System.out.println(cnf.getNumColunasArquivo());
			System.out.println(cnf.getNumLinhasArquivo());
			System.out.println(cnf.getNeuroniosFAN());
			ArrayList<NeuronioFAN> listaComNeuronios  = new ArrayList<NeuronioFAN>();
			listaComNeuronios = cnf.getNeuroniosFAN();
			System.out.println(listaComNeuronios.size());
			PersistorDados pd = new PersistorDados();
			pd.gravaNeuroniosFanDat(cnf.getNeuroniosFAN(), "c:\\testeNeuFAN.dat");
			*/
			
		} catch (Exception e) {
			e.printStackTrace();
			// TODO: handle exception
		}
		
	}

}
