package jfan.io.arquivos;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Vector;
import jfan.fan.padroes.IPadrao;
import jfan.fan.padroes.Padrao;
import jfan.io.ILeitura;
import jxl.Cell;
import jxl.Sheet;
import jxl.Workbook;

/**
 * Esta classe tem por finalidade carregar arquivos do Excel desde a verso 97 at XP
 * @version 0.0.2
 * @author Fabiano
*/

public class CarregaXls implements ILeitura {

	/**
	 * Contm os padres desnormalizados  
	 */
	public ArrayList<IPadrao> padroes = null;

	/**
	 * Define o nome de arquivo de padres 
	 */
	private String nomeArquivo = "";

	/**
	 * Define uma coluna inicial inclusive
	 */
	private int colunaInicial = 0;
	
	/**
	 * Define uma coluna final inclusive
	 */
	private int colunaFinal = 1;

	/**
	 * Define uma linha inicial inclusive 
	 */
	private int linhaInicial = 1;

	/**
	 * Define uma linha final inclusive 
	 */
	private int linhaFinal = 0;

	/**
	 * Define a coluna que contm a classe 
	 */
	private int colunaClasse = 0;
	
	/**
	 * Construtor da classe CarregaXls
	 * @param umNomeDeArquivo
	 *	Um arquivo com padres
	 *
	 * @param aColunaDaClasse
	 *	A Coluna que contm a classe
	 *
	 * @param aColunaInicial
	 *	Define a coluna inicial inclusive
	 *
	 * @param aColunaFinal
	 *	Define a coluna final inclusive
	 *
	 * @param aLinhaInicial
	 *	Define a linha inicial inclusive
	 *
	 * @param aLinhaFinal
	 *	Define a linha final inclusive
	 */

	public CarregaXls(String umNomeDeArquivo, int aColunaDaClasse, int aColunaInicial, int aColunaFinal, int aLinhaInicial, int aLinhaFinal){
		this.nomeArquivo = umNomeDeArquivo;
		this.colunaClasse = aColunaDaClasse;
		this.colunaInicial = aColunaInicial;
		this.colunaFinal = aColunaFinal;
		this.linhaInicial = aLinhaInicial;
		this.linhaFinal = aLinhaFinal;
	}
	
	/**
	 *  @return retorna um array primitivo de inteiros contendo as classes do arquivo
	 *   
	 */
	public int[] getClasses() throws Exception {
		ArrayList<String> asClasses = new ArrayList<String>();
		asClasses.clear();
		String[][] osPadroes = getPadroesOriginais();
		for (int i=0;i<osPadroes.length;i++)
	    	if (!asClasses.contains(osPadroes[i][osPadroes[0].length-1]))
	    		asClasses.add(osPadroes[i][osPadroes[0].length-1]);
		
		int[] arrayClasses = new int[asClasses.size()];
		int intClasse =0;
		Double doubleClasse;
		for (int i=0;i<asClasses.size();i++){
			doubleClasse = Double.parseDouble(asClasses.get(i));
			intClasse = doubleClasse.intValue();
			arrayClasses[i]=intClasse;
		}
		Arrays.sort(arrayClasses);
		return arrayClasses;
	}

	/**
	 *  @return retorna um inteiro primitivo contendo o numero de caracteristicas do arquivo
	 *   
	 */
	public int getNumCaracteristicas() throws Exception {
		Workbook workbook = Workbook.getWorkbook(new File(getArquivo()));  
	    Sheet sheet = workbook.getSheet(0);
	    int numColunas = sheet.getColumns();
	    workbook.close();
		return numColunas-1;
	}

	/**
	 *  @return retorna um inteiro primitivo contendo o numero de classes do arquivo
	 *   
	 */
	public int getNumClasses() throws Exception {
		ArrayList<String> asClasses = new ArrayList<String>();
		asClasses.clear();
		String[][] osPadroes = getPadroesOriginais();
		for (int i=0;i<osPadroes.length;i++)
	    	if (!asClasses.contains(osPadroes[i][osPadroes[0].length-1]))
	    		asClasses.add(osPadroes[i][osPadroes[0].length-1]);
		return asClasses.size();
	}

	/**
	 *  @return retorna um inteiro primitivo contendo o numero de padres do arquivo
	 *   
	 */
	public int getNumPadroes() throws Exception {
	    Workbook workbook = Workbook.getWorkbook(new File(getArquivo()));  
	    Sheet sheet = workbook.getSheet(0);
	    int numLinhas = sheet.getRows();
	    workbook.close();
		return numLinhas;
	}

	/**
	 *  @return retorna um ArrayList com uma lista de objetos Padrao
	 *   
	 */
	public ArrayList<IPadrao> getPadroes() throws Exception {
		padroes = new ArrayList<IPadrao>();
		padroes.clear();
		String[][] osPadroes = getPadroesOriginais();
		Vector<Double> umPadrao = new Vector<Double>(); 
		double[] arrayCaracteristicas = new double[osPadroes[0].length-1];
		int classe = 0;
		for (int i=0;i<osPadroes.length;i++){
			for (int j=0;j<osPadroes[0].length;j++)
				umPadrao.add(Double.parseDouble(osPadroes[i][j]));
			
			for(int k=0;k<umPadrao.size()-1;k++)
				arrayCaracteristicas[k]=umPadrao.get(k);
			classe=umPadrao.lastElement().intValue();
			padroes.add(new Padrao(arrayCaracteristicas,classe));
			umPadrao.clear();
		}
		return padroes;
	}

	/**
	 *  @return retorna um array de Strings contendo uma matriz com os dados lidos do arquivo,
	 *   com todas as linhas e todas as colunas
	 *   
	 */
	public String[][] getPadroesOriginais() throws Exception {
	    Workbook workbook = Workbook.getWorkbook(new File(getArquivo()));  
	    Sheet sheet = workbook.getSheet(0);
	    int numLinhas = sheet.getRows();
	    int numColunas = sheet.getColumns();
	    //obtem todos os dados da planilha excel
	    String[][] arrayPadroes = new String[numLinhas][numColunas]; 
	    for (int i=0;i<numLinhas;i++)
	    	for (int j=0;j<numColunas;j++){
	    		Cell valorCelula = sheet.getCell(j,i);
	    		if (valorCelula.getContents().length()==0)
	    			arrayPadroes[i][j]="0.0";
			    else
			    	arrayPadroes[i][j] = valorCelula.getContents().replaceAll(",",".");
	    	}
	    workbook.close();
	    
	    //faz verificao se nmero de linhas especificado  maior do que contm no arquivo
	    if (this.linhaFinal>numLinhas)
	    	this.linhaFinal=numLinhas;
	    
	    //faz verificao se nmero de colunas especificado  maior do que contm no arquivo
	    if (this.colunaFinal>numColunas)
	    	this.colunaFinal=numColunas;
	    
	    //monta array delimitando dados
	    numLinhas = (this.linhaFinal-this.linhaInicial)+1;
	    numColunas = (this.colunaFinal-this.colunaInicial)+2;
 	    String[][] arrayPadroesDelimitados = new String[numLinhas][numColunas];
	    int l = 0;
	    int c = 0;
 	    for (int i=linhaInicial-1;i<linhaFinal;i++){
 		    c = 0;
 	    	for (int j=colunaInicial-1;j<colunaFinal;j++){
	    		arrayPadroesDelimitados[l][c]=arrayPadroes[i][j];
	    		c++;
 	    	}
	    	arrayPadroesDelimitados[l][arrayPadroesDelimitados[0].length-1]=arrayPadroes[i][colunaClasse-1];
		    l++;
	    }
	    
 	    return arrayPadroesDelimitados;
	}
	
	/**
	 *  @return retorna o nome do arquivo
	 *   
	 */
	public String getArquivo() throws Exception {
		return this.nomeArquivo;
	}


	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			CarregaXls xls = new CarregaXls("c:\\fan\\arquivo.xls",1,1,3,4000,20000);
			System.out.println(xls.getPadroesOriginais());
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
		}
		// TODO Auto-generated method stub

	}

}
