/*
 * ORFBO.java
 *
 * Created on 31 de Julho de 2007, 16:32
 *
 * By: Oslei Taborda Ribas
 */

package jorfinder.business;

import java.util.ArrayList;
import jorfinder.information.ClasseVO;
import jorfinder.information.CodonVO;
import jorfinder.information.ORFCodonVO;
import jorfinder.information.ORFVO;
import jorfinder.log2java.Logger;
import jorfinder.persistence.ClasseDAO;
import jorfinder.persistence.ORFCodonDAO;
import jorfinder.persistence.ORFDAO;
import jorfinder.util.Mensagem;
import jorfinder.util.UniqueException;
import jorfinder.util.VioletionFKException;


public class ORFBO {
    
    public static Logger log = new Logger(ORFBO.class.getName());
    private Mensagem msg = new Mensagem();
    /** Creates a new instance of ORFBO */
    public ORFBO() {}
    
    public Mensagem getMensagem(){
        return this.msg;
    }
    
    
    public void adiciona(ORFVO objeto) {
            try {                
                //verifica se o registro j existe no banco
                if(ORFDAO.getObjetoByUnique(objeto).getCod() > 0){
                    this.msg.setMsg("Erro! J existe uma ORF deste tipo com este nome no banco!");
                    this.msg.setSucesso(false);                     
                } else {
                    ORFDAO.adiciona(objeto);
                    ArrayList<ORFVO> arrayORF = ORFDAO.getListaObjeto();
                    //pega o cdigo do registro q acabou de ser inserido no banco
                    int codORF = arrayORF.get(arrayORF.size() - 1).getIntCodORF();

                    //necessrio para pegar o ndice dos registros dos cdons
                    CodonBO codonBO = new CodonBO();
                    ArrayList<CodonVO> seq = codonBO.getListaObjeto();                    
                    String codon = "";
                    //a cada 3 caracteres cria um objeto ORFCodonVO
                    for(int cont = 0; cont < objeto.getConteudo().length()/3; cont++){		
                        //extrai os 3 caracteres
                        codon = objeto.getConteudo().substring(cont*3, (cont + 1)*3);
                        ORFCodonVO orfcodonVO = new ORFCodonVO();
                        //seta o cdigo do ORF q foi cadastrado. Chave estrangeira da tabela ORF
                        orfcodonVO.setCodORF(codORF);
                        //extrai o indice da sequencia, ex: AAA = 0, TTT = 63. 
                        //chave estrangeira da tabela codon3
                        boolean conferido = false;
                        //confere a substring com cada codon da sequencia at o achar. Quando acha, atribui o seu respencivo codigo e para o loop
                        for(int c=0; c<seq.size() && !conferido; c++){

                            if(codon.equals(seq.get(c).getCodon())){
                                orfcodonVO.setCodCodon(seq.get(c).getCod());
                                conferido = true;
                            }
                        }
                        orfcodonVO.setCodonSequencia(cont);
                        ORFCodonDAO.adiciona(orfcodonVO);
                    }
                    //coloca o objeto alterado de volta no seu lugar                    
                    this.msg.setMsg("Cadastro realizado com sucesso!");
                    this.msg.setSucesso(true);
                }
            } catch (jorfinder.util.UniqueException ex) {
                this.msg.setMsg("Houve falha ao inserir o registro!");
                this.msg.setSucesso(false);
             }               		
    }
    public ArrayList <ORFVO> getListaObjeto(){
            return ORFDAO.getListaObjeto();
    }
    public ORFVO getUltimoCadastrado(){
        ArrayList <ORFVO> lista = getListaObjeto();
        return lista.get(lista.size() - 1);
    } 
    
    public ArrayList<ORFVO> getListaObjetoORFBase(){
        return ORFDAO.getListaObjetoORFBase();
    }
    
    public ArrayList<ORFVO> getListaObjetoORFTreino(){
        return ORFDAO.getListaObjetoORFTreino();
    }   
    
    public static int getTotalCodonORF(ORFVO orf){
        return ORFCodonDAO.getQtdeObjetoByUnique(orf);
    }
    
    public ORFVO getORFVO(ORFVO orf){
        
        return ORFDAO.getObjetoByUnique(orf);
    }

    public void atualiza(ORFVO objeto) {
		
            try {
                    
                    ORFDAO.atualiza(objeto);
                    try {
                        //exclui todos os registros da tabela orfcodon referentes ao codon que est sendo atualizado
                        ORFCodonDAO.exclui(objeto);
                        //abaixo comea a inserir todos os codons novamente
                        ArrayList<ORFVO> arrayORF = ORFDAO.getListaObjeto();

                        //necessrio para pegar o ndice dos registros dos cdons
                        CodonBO codonBO = new CodonBO();
                        ArrayList<CodonVO> seq = codonBO.getListaObjeto();
                        String codon = "";
                        //a cada 3 caracteres cria um objeto ORFCodonVO
                       for(int cont = 0; cont < objeto.getConteudo().length()/3; cont++){		
                            //extrai os 3 caracteres
                            codon = objeto.getConteudo().substring(cont*3, (cont + 1)*3);
                            ORFCodonVO orfcodonVO = new ORFCodonVO();
                            //seta o cdigo do ORF q foi cadastrado. Chave estrangeira da tabela ORF
                            orfcodonVO.setCodORF(objeto.getCod());
                            //extrai o indice da sequencia, ex: AAA = 0, TTT = 63. 
                            //chave estrangeira da tabela codon3
                            boolean conferido = false;
                            //confere a substring com cada codon da sequencia at o achar. Quando acha, atribui o seu respencivo codigo e para o loop
                            for(int c=0; c<seq.size() && !conferido; c++){

                                if(codon.equals(seq.get(c).getCodon())){
                                    orfcodonVO.setCodCodon(seq.get(c).getCod());
                                    conferido = true;
                                }
                            }
                            orfcodonVO.setCodonSequencia(cont);
                            ORFCodonDAO.adiciona(orfcodonVO);
                        }
                        //adiciona o objeto que acabou de ser cadastrado para atualizar a jtable
                        this.msg.setMsg("Alterao realizada com sucesso!");
                        this.msg.setSucesso(true);
                    } catch (VioletionFKException ex) {
                        log.info("Erro ao atualizar registros. O que o java disse a respeito:"+ex.getMessage());
                        this.msg.setMsg("No foi possvel atualizar o contedo do ORF!");
                        this.msg.setSucesso(false);
                        
                    }

            } catch (UniqueException e) {
                    this.msg.setMsg("No foi possvel atualizar o cadastro desse ORF porque j existe um ORF cadastrado com esses Dados!");
                    this.msg.setSucesso(false);
            }
    }
    
    public void exclui(ORFVO objeto) {
		
            try {                      
                    ORFDAO.exclui(objeto);
                    this.msg.setMsg("ORF Base Excluda com sucesso!");
                    this.msg.setSucesso(true);
            } catch (VioletionFKException e) {
                    this.msg.setMsg("No foi possvel deletar esse registro porque existem outros registros que dependem dele!");
                    this.msg.setSucesso(false);
            }
    }
}
