/*
 * ClasseDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class ClasseDAO {
    
    public static Logger log = new Logger(ClasseDAO.class.getName());       
    public static final String TABLE = "Classe";
    // evita que seja instnciado.
    private ClasseDAO() {
    }
    
    public static void adiciona(ClasseVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (nome) values (?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 
                 stmt.setString(1, objeto.getNome());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar ClasseVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
    public static void atualiza(ClasseVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "update " + TABLE + " set nome = ? where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 
                 stmt.setString(1, objeto.getNome());
                 stmt.setInt(2,objeto.getCod());                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                      throw new UniqueException();
                  } else {
                      log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static void exclui(ClasseVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCod());                 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static ClasseVO getObjetoByUnique(ClasseVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            ClasseVO obj = objeto;
            ResultSet rs = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 if (!((obj.getCod() ) == 0)){
                    comando = "select cod, nome from " + TABLE + " where cod = ? " ;
                    stmt = con.prepareStatement(comando);
                    stmt.setInt(1,objeto.getCod());
                    
                 }else {
                     comando = "select cod, nome from " + TABLE + " where nome = ? " ;
                     stmt = con.prepareStatement(comando);
                     stmt.setString(1,objeto.getNome());
                 }
                                  
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if (rs.next()){
                        // popula o objeto
                        obj = new ClasseVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));                           
                 }
                      
         
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);                      
              }finally {
                  ConnectionFactory.closeAll(stmt, rs);
              } 
              return obj;
       }
    
    public static ArrayList<ClasseVO> getListaObjeto(){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<ClasseVO> array = new ArrayList<ClasseVO>();
            ResultSet rs = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select cod, nome from " + TABLE + " order by cod";
                 stmt = con.prepareStatement(comando);
                                      
                 //executa
                 rs = stmt.executeQuery();
                 ClasseVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new ClasseVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));                           
                        array.add(obj);
                 }
                  
                                 
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
       }
    }
    


