/*
 * CodonDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class CodonDAO {
    
    public static Logger log = new Logger(CodonDAO.class.getName());       
    public static final String TABLE = "Codon";
    // evita que seja instnciado.
    private CodonDAO() {
    }
    
    public static void adiciona(CodonVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (cod, codon) values (?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCod());
                 stmt.setString(2, objeto.getCodon());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar CodonVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
    public static void atualiza(CodonVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "update " + TABLE + " set codon = ? where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 
                 stmt.setString(1, objeto.getCodon());
                 stmt.setInt(2,objeto.getCod());                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                      throw new UniqueException();
                  } else {
                      log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static void exclui(CodonVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCod());                 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static CodonVO getObjetoByUnique(CodonVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            CodonVO obj = objeto;
            ResultSet rs = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 if (!((obj.getCod() ) == 0)){
                    comando = "select cod, codon from " + TABLE + " where cod = ? " ;
                    stmt = con.prepareStatement(comando);
                    stmt.setInt(1,objeto.getCod());
                    
                 }else {
                     comando = "select cod, codon from " + TABLE + " where codon = ? " ;
                     stmt = con.prepareStatement(comando);
                     stmt.setString(1,objeto.getCodon());
                 }
                                  
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if (rs.next()){
                        // popula o objeto
                        obj = new CodonVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setCodon(rs.getString("codon"));                           
                 }
                      
         
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return obj;
       }
    
    public static ArrayList<CodonVO> getListaObjeto(){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<CodonVO> array = new ArrayList<CodonVO>();
            ResultSet rs = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select cod, codon from " + TABLE + " order by cod";
                 stmt = con.prepareStatement(comando);
                                      
                 //executa
                 rs = stmt.executeQuery();
                 CodonVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new CodonVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setCodon(rs.getString("codon"));                           
                        array.add(obj);
                 }
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
       }
    }
    


