/*
 * ConnectionFactory.java
 *
 * Created on 18 de Julho de 2007, 22:46
 *
 * By: Oslei Taborda Ribas
 */

package jorfinder.persistence;


import java.sql.*;
import jorfinder.log2java.Logger;

public class ConnectionFactory {
  
        public static Logger log = new Logger(ConnectionFactory.class.getName());       
        // guarda a minha instncia
        private static ConnectionFactory instancia;
        // guarda o bd que esta conectado
        private RDB rdb = null;


        // evita que seja instanciado - para que possamos criar um singleton
        private ConnectionFactory() {  
            this.setDefaultRDB();
        }
        
        // pega a instancia
        public static ConnectionFactory getInstancia(){
            return instancia;
        }
         // retorna a conexao ativa
         public static synchronized Connection getConnection() {
             if (instancia == null) {
                log.info("Criando instncia da connectionFactory");
                instancia = new ConnectionFactory();
             }
             return instancia.rdb.getConnection();
        }
         
         // usado no final do programa, quando o usurio clickar em sair do programa
         // vai ter que chamar esse metodo para fechar a conexo
         public static synchronized void fecharConexao(){
                log.info("Chamando o metodo para fechar a conexo do RDB");
                instancia.rdb.fecharConexao();
          }
    
        public void setRDB(RDB rdb){
            log.info("Setando o seguinte RDB como padro " + rdb.getClass().getName());
            this.rdb = rdb;
        }
        
        public void setDefaultRDB(){
            log.info("Setando o JavaDB como banco padro");
            rdb = new JavadbRDB();
            rdb.conectar();
        }
        
        public static String getCodUniqueException(){
            return instancia.rdb.getCodUniqueException();
        }
        
        public static String getCodVioletionFKException(){
            return instancia.rdb.getCodVioletionFKException();
        }
        
        public static void closeStmt(PreparedStatement stmt){
                  if( stmt != null ) {
                     try { 
                         stmt.close(); 
                     } catch(Exception ex) { 
                        log.exception(ex);
                     }
                  }
        }
        
        public static void closeAll(PreparedStatement stmt, ResultSet rs){
                  // fechando o statement
                  if( stmt != null ) {
                             try { 
                                 stmt.close(); 
                             } catch(Exception ex) { 
                                log.exception(ex);
                             }
                  }
                  // fechando o resultset
                  if( rs != null ) {
                         try { 
                             rs.close(); 
                         } catch(Exception ex) { 
                            log.exception(ex);
                         }
                      }
        }
}
    

