/*
 * GabaritoDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.Timestamp;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class GabaritoDAO {
    
    public static Logger log = new Logger(GabaritoDAO.class.getName());       
    public static final String TABLE = "Gabarito";
    // evita que seja instnciado.
    private GabaritoDAO() {
    }
    
    public static void adiciona(GabaritoVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (nome, descricao, geracoes, mutacao, fitness, tempoProcessamento, populacao, elitismo) values (?,?,?,?,?,?,?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setString(1, objeto.getNome());
                 stmt.setString(2, objeto.getDescricao());
                 stmt.setInt(3, objeto.getGeracoes());
                 stmt.setInt(4, objeto.getMutacao());
                 stmt.setDouble(5, objeto.getFitness());        
                 stmt.setTimestamp(6, new Timestamp(objeto.getTempoProcessamento().getTime()));        
                 stmt.setInt(7, objeto.getPopulacao());
                 stmt.setInt(8, objeto.getElitismo());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar GabaritoVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
    public static void atualiza(GabaritoVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "update " + TABLE + " set nome = ? , descricao = ? , geracoes = ? , mutacao = ? , fitness = ? , tempoProcessamento = ?, populacao = ? , elitismo = ?  where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 
                 stmt.setString(1, objeto.getNome());
                 stmt.setString(2, objeto.getDescricao());
                 stmt.setInt(3, objeto.getGeracoes());
                 stmt.setInt(4, objeto.getMutacao());
                 stmt.setDouble(5, objeto.getFitness());
                 stmt.setTimestamp(6, new Timestamp(objeto.getTempoProcessamento().getTime()) );
                 stmt.setInt(7, objeto.getPopulacao());
                 stmt.setInt(8, objeto.getElitismo());        
                 stmt.setInt(9, objeto.getCod());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                      throw new UniqueException();
                  } else {
                      log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static void exclui(GabaritoVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCod());                 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static GabaritoVO getObjetoByUnique(GabaritoVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            GabaritoVO obj = objeto;
            ResultSet rs = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 if (!((obj.getCod() ) == 0)){
                    comando = "select cod, nome, descricao, geracoes, mutacao, fitness, tempoProcessamento, populacao, elitismo from " + TABLE + " where cod = ? " ;
                    stmt = con.prepareStatement(comando);
                    stmt.setInt(1,objeto.getCod());
                    
                 }else {
                     comando = "select cod, nome, descricao, geracoes, mutacao, fitness, tempoProcessamento, populacao, elitismo from " + TABLE + " where nome = ? " ;
                     stmt = con.prepareStatement(comando);
                     stmt.setString(1,objeto.getNome());
                 }
                                  
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if (rs.next()){
                        // popula o objeto
                        obj = new GabaritoVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));                           
                        obj.setDescricao(rs.getString("descricao"));
                        obj.setGeracoes(rs.getInt("geracoes"));
                        obj.setMutacao(rs.getInt("mutacao"));
                        obj.setFitness(rs.getDouble("fitness")); 
                        obj.setTempoProcessamento(rs.getTimestamp("tempoProcessamento"));
                        obj.setPopulacao(rs.getInt("populacao"));
                        obj.setElitismo(rs.getInt("elitismo"));
                        
                 }
                      
         
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return obj;
       }
    
    public static GabaritoVO getUltimoObjetoInserido(){
            Connection con = null;
            PreparedStatement stmt = null;
            ResultSet rs = null;
            GabaritoVO obj = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 
                 comando = "select cod, nome, descricao, geracoes, mutacao, fitness, tempoProcessamento, populacao, elitismo from " + TABLE + " where cod = (select max(cod) from Gabarito) " ;
                 stmt = con.prepareStatement(comando);
                                  
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if (rs.next()){
                        // popula o objeto
                        obj = new GabaritoVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));                           
                        obj.setDescricao(rs.getString("descricao"));
                        obj.setGeracoes(rs.getInt("geracoes"));
                        obj.setMutacao(rs.getInt("mutacao"));
                        obj.setFitness(rs.getDouble("fitness"));
                        obj.setTempoProcessamento(rs.getTimestamp("tempoProcessamento"));
                        obj.setPopulacao(rs.getInt("populacao"));
                        obj.setElitismo(rs.getInt("elitismo"));
                        
                 }
                      
         
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return obj;
       }
    
    public static ArrayList<GabaritoVO> getListaObjeto(){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<GabaritoVO> array = new ArrayList<GabaritoVO>();
            ResultSet rs = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select cod, nome, descricao, geracoes, mutacao, fitness, tempoProcessamento, populacao, elitismo from " + TABLE + " order by cod";
                 stmt = con.prepareStatement(comando);
                                      
                 //executa
                 rs = stmt.executeQuery();
                 GabaritoVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new GabaritoVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));                           
                        obj.setDescricao(rs.getString("descricao"));
                        obj.setGeracoes(rs.getInt("geracoes"));
                        obj.setMutacao(rs.getInt("mutacao"));
                        obj.setFitness(rs.getDouble("fitness"));
                        obj.setTempoProcessamento(rs.getTimestamp("tempoProcessamento"));
                        obj.setPopulacao(rs.getInt("populacao"));
                        obj.setElitismo(rs.getInt("elitismo"));                           
                        array.add(obj);                        
                 }
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
       }
    }
    


