/*
 * TreinoORFDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class GabaritoORFDAO {
    
    public static Logger log = new Logger(GabaritoORFDAO.class.getName());       
    public static final String TABLE = "GabaritoOrf";
    // evita que seja instnciado.
    private GabaritoORFDAO() {
    }
    
    public static void adiciona(GabaritoVO objeto1, ORFVO objeto2)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (codGabarito, codOrf) values (?,?)";
                 stmt = con.prepareStatement(comando);
                 
                // seta os valores
                 stmt.setInt(1,objeto1.getCod());
                 stmt.setInt(2, objeto2.getCod());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar GabaritoORFVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
     public static void exclui(GabaritoVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where codGabarito = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCod());                 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
     public static void exclui(ORFVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where codOrf = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 
                 stmt.setInt(1,objeto.getCod());                 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
        
     public static void exclui(GabaritoVO objeto1, ORFVO objeto2)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where codGabarito = ? and codOrf = ? ";
                 stmt = con.prepareStatement(comando);
                 
                // seta os valores
                 stmt.setInt(1,objeto1.getCod());                 
                 stmt.setInt(2,objeto2.getCod());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }   
    
     public static ArrayList<GabaritoVO> getListaObjetoByUnique(ORFVO objeto){
            
            Connection con = null;
            PreparedStatement stmt = null;
            PreparedStatement stmt2 = null;
            ArrayList<GabaritoVO> array = new ArrayList<GabaritoVO>();
            GabaritoVO obj2 = null;
            ResultSet rs = null;
            ResultSet rs2 = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select codGabarito from " + TABLE + " where codOrf = ? ";
                 String comando2 =  "select cod, nome, descricao, geracoes, mutacao, fitness, tempoProcessamento, populacao, elitismo from Gabarito where cod = ? " ;
                 
                 stmt = con.prepareStatement(comando);                 
                 stmt.setInt(1,objeto.getCod()); 
                                      
                 //executa
                 rs = stmt.executeQuery();
                 
                 int indice;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while (rs.next()){
                     
                        // pega o codigo do objeto                                            
                        indice = rs.getInt("codGabarito");
                        stmt2 = con.prepareStatement(comando2);
                 
                        stmt2.setInt(1,indice);
                        rs2 = stmt2.executeQuery();
                        // posiciona o ponteiro na 1 posio e ve se no  nulo
                         if (rs2.next()){
                                // popula o objeto
                                obj2 = new GabaritoVO();
                                obj2.setCod(rs2.getInt("cod"));
                                obj2.setNome(rs2.getString("nome"));                           
                                obj2.setDescricao(rs2.getString("descricao"));
                                obj2.setGeracoes(rs2.getInt("geracoes"));
                                obj2.setMutacao(rs2.getInt("mutacao"));
                                obj2.setFitness(rs2.getDouble("fitness"));
                                obj2.setTempoProcessamento(rs2.getTimestamp("tempoProcessamento"));
                                obj2.setPopulacao(rs2.getInt("populacao"));
                                obj2.setElitismo(rs2.getInt("elitismo"));                           
                                array.add(obj2);
                         }   
                        
                 }
                  
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                    ConnectionFactory.closeAll(stmt2,  rs2);
                 }
              return array;
       }
     
     public static ArrayList<ORFVO> getListaObjetoByUnique(GabaritoVO objeto){
            
            Connection con = null;
            PreparedStatement stmt = null;
            PreparedStatement stmt2 = null;
            ArrayList<ORFVO> array = new ArrayList<ORFVO>();
            ORFVO obj2 = null;
            ResultSet rs = null;
            ResultSet rs2 = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select codOrf from " + TABLE + " where codGabarito = ? ";
                 String comando2 =  "select cod, codclasse, nome, descricao, tipo from Orf where cod = ? " ;
                 
                 stmt = con.prepareStatement(comando);                 
                 
                 stmt.setInt(1,objeto.getCod()); 
                                      
                 //executa
                 rs = stmt.executeQuery();
                 
                 int indice;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while (rs.next()){
                     
                        // pega o codigo do objeto                                            
                        indice = rs.getInt("codOrf");
                        stmt2 = con.prepareStatement(comando2);
                        
                        stmt2.setInt(1,indice);
                        
                        rs2 = stmt2.executeQuery();
                        // posiciona o ponteiro na 1 posio e ve se no  nulo
                         if (rs2.next()){
                                // popula o objeto
                                obj2 = new ORFVO();
                                ClasseVO classe2 = new ClasseVO();
                                classe2.setCod(rs2.getInt("codclasse"));
                                
                                obj2.setCod(rs2.getInt("cod"));
                                obj2.setClasse(classe2);                           
                                obj2.setNome(rs2.getString("nome"));
                                obj2.setDescricao(rs2.getString("descricao"));
                                obj2.setTipo(rs2.getString("tipo"));
                                array.add(obj2);
                         }   
                        ConnectionFactory.closeAll(stmt2,  rs2);
                        
                 }
                  
              } catch (SQLException sqle) {
                    log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                    ConnectionFactory.closeAll(stmt2,  rs2);
                 }
              return array;
       }

          public static void adiciona(GabaritoORFVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try 
            {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " values (?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getGabaritoCodigo());
                 stmt.setInt(2,objeto.getOrfCodigo());
                 // executa
                 stmt.execute();
                 
             } 
             catch (SQLException sqle){
                 
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar GabaritoORFVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              
             }
       }
    
     public static void exclui(GabaritoORFVO objeto)throws VioletionFKException
    {
            Connection con = null;
            PreparedStatement stmt = null;
            try 
            {
                 con = ConnectionFactory.getConnection();              
                 String comando;
                 
                 if (objeto.getOrfCodigo() == 0)
                 {
                    comando= "delete from " + TABLE + " where  codGabarito = ?";
                    stmt = con.prepareStatement(comando); 
                    stmt.setInt(1,objeto.getGabaritoCodigo()); 
                 }
                 else if(objeto.getGabaritoCodigo()==0)
                 {
                   
                      comando= "delete from " + TABLE + " where  codOrf = ?";
                      stmt = con.prepareStatement(comando); 
                      stmt.setInt(1,objeto.getOrfCodigo()); 
                 }
                                
                 // executa
                 stmt.execute();
                 
              }
              catch (SQLException sqle){
                  
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
              
       }
    
     
      public static ArrayList<GabaritoORFVO> getListaObjeto(GabaritoORFVO objeto)
     {
            Connection con = null;
            PreparedStatement stmt = null;
            ResultSet rs = null;
            ArrayList<GabaritoORFVO> array = new ArrayList<GabaritoORFVO>();
                        
            try 
            {
                 con = ConnectionFactory.getConnection();
                 String comando = "select codGabarito,codOrf from " + TABLE + " where codGabarito =?  order by codGabarito";
                 stmt = con.prepareStatement(comando);
                 stmt.setInt(1,objeto.getGabaritoCodigo());
                                      
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula objetos
                        GabaritoORFVO gabaritoORFVO = new GabaritoORFVO();
                        gabaritoORFVO.setGabaritoCodigo(rs.getInt("codGabarito"));
                        gabaritoORFVO.setOrfCodigo(rs.getInt("codOrf"));
                                                 
                        array.add(gabaritoORFVO);
                 }
              }
              catch (SQLException sqle){
                    log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                    
                 }
                  
              return array;
       }
  
    
}    


