/*
 * JavadbRDB.java
 *
 * Created on 23 de Julho de 2007, 23:30
 *
 * By: Oslei Taborda Ribas
 */

package jorfinder.persistence;
import java.sql.*;
import jorfinder.log2java.Logger;


public class JavadbRDB implements RDB{
    
        public static Logger log = new Logger(JavadbRDB.class.getName());
            // define o driver de conexo
        private String driver = "org.apache.derby.jdbc.EmbeddedDriver";
        // define o nome do banco
        private String dbName="dbjorfinder";
        // define a url de conexo com bd - se o banco no existir ele vai criar um na raiz do projeto
        private String connectionURL = "jdbc:derby:" + dbName + ";create=true";
        // guarda a conexo
        private Connection conn = null;

        /** Creates a new instance of JavadbRDB */
        public JavadbRDB() {
        }
   
        
        public void conectar() {

          //   tenta conectar ao banco
          try  {
              log.info("Localizando o driver no caminho informado...");
              // procura o driver no caminho definido
              Class.forName(driver); 

            // como podemos ver no  feito nenhum tratamento do erro 
            // pois isso no deve ocorrer, somente ocorre se vc gerar um .jar errado  
          } catch(java.lang.ClassNotFoundException ex)     {
              log.exception("No foi possvel conectar, possvel problema no classpath, ou localizar do .jar do derby", ex);
          }

          //   Tenta pegar a conexo
         try {
                // Tenta conectar se no existir o bd ele cria
                
                log.info("Conectando com o JavaDB...");
                conn = DriverManager.getConnection(connectionURL);

                // verifica se todas as tabelas esto ok
                if (!this.isBancoIntegro(conn)){
                    log.info("O banco no est integro ser apagado");
                    // As tabelas no est ok! ento temos que apagar tudo!
                    this.deleteTables(conn);
                    log.info("Recriando as tabelas.");
                    // agora temos que criar tudo de novo!
                    this.criarTabelas(conn);
                }


                } catch (SQLException ex){
                    log.exception("codigo de erro = " + ex.getErrorCode());
                    log.exception(ex);
                 }
             }

         // retorna a conexao ativa
         public Connection getConnection() {
             
             return this.conn;
        }
         
         // usado no final do programa, quando o usurio clickar em sair do programa
         // vai ter que chamar esse metodo para fechar a conexo
         public void fecharConexao(){
                try {
                    log.info("tentando fechar a conexo com o javadb");
                    conn.close();
                } catch (SQLException ex) {
                    log.exception("Ocorreu algum erro ao fechar a conexo com o javadb", ex);
                }

               boolean gotSQLExc = false;
               try {
                  log.info("executando um shutdown no derby."); 
                  DriverManager.getConnection("jdbc:derby:;shutdown=true");
               } catch (SQLException ex)  {	
                  // se o Derby consegui derrubar o banco ele retorna esse codigo
                   // qualquer outro codigo significa erro
                  if ( ex.getSQLState().equals("XJ015") ) {		
                     gotSQLExc = true;
                  }
               }
               if (!gotSQLExc) {
               	  log.exception("O Banco de Dados no foi fechado normalmente");
               }  else  {
                  log.info("O Banco de Dados foi fechado corretamente... ");	
               }  
            
            
        
      }
    
        // cria todas as tabelas no bd
         private void criarTabelas(Connection conn){
                try {

                    Statement statement = conn.createStatement();
                    String[] comandos = this.getSqlCriaTabelas();
                    for (int i = 0; i < comandos.length ; i++){
                        statement.execute(comandos[i]);
                        
                    }
                } catch (SQLException ex) {
                     log.exception("Ocorreu o seguinte erro ao criar as tabelas " , ex);
                     
                }

         }
         
         // verifica se existem todas as tabelas e todos os campos
         private boolean isBancoIntegro(Connection conn){
                                    
                  try {
                        Statement statement = conn.createStatement();
                        String[] comandos = this.getSqlVerificaTabelas();
                        
                        for (int i = 0; i < comandos.length ; i++){
                             log.info(" executando o seguinte comando = " + comandos[i]);
                             statement.execute(comandos[i]);
                        }
                  }  catch (SQLException sqle) {
                      
                      log.exception("O banco no est ok", sqle);
                      return false;
                     
                  }
                  
                  return true;
         }
         
         // tenta apagar todas as tabelas! E deixar o banco limpo!
         private void deleteTables(Connection conn){
                        
                        try{
                        Statement statement = conn.createStatement();                        
                        String[] comandos = this.getSqlDeleteTables();             
                        
                        for (int i = 0; i < comandos.length ; i++){
                            try {                                         
                                statement.execute(comandos[i]);
                                }  catch (SQLException sqle) {
                                    log.exception("erro ao executar o seguinte comando = " + comandos[i]);
                                    log.exception("Ocorreu um erro ao deletar a tabela " , sqle);
                                  // no faz nada se der erro no delete, porque o erro deve ser porque a tabela no existe
                                  }
                             }
                         }catch(SQLException sqle) {
                                  log.exception("no foi possvel pegar a conexo para iniciar a deletar as tabelas!");  
                                  log.exception("erro ao conectar = " , sqle);                                  
                       }
         }
         // comandos para verificar se existem todas as tabelas necessarias e com os campos certos!
         private String[] getSqlVerificaTabelas(){
             String retorno[] = new String[10];
             
             retorno[0] = " select cod , nome, descricao from Treino where cod = (select max(cod) from Treino)";
             retorno[1] = " select cod, geracoes, mutacao, fitness, tempoProcessamento, populacao, elitismo, nome, descricao from Gabarito where cod = (select max(cod) from Gabarito)";
             retorno[2] = " select cod, nome from Classe where cod = (select max(cod) from Classe)";
             retorno[3] = " select cod, codon from Codon where cod = (select max(cod) from Codon)";
             retorno[4] = " select cod, codTreino, nome, descricao,rede from Rede where cod = (select max(cod) from Rede)";
             retorno[5] = " select cod, codClasse, nome, descricao, tipo from Orf where cod = (select max(cod) from Orf)";
             retorno[6] = " select codGabarito, codOrf, codTreino, caracteristica from TreinoOrfGabarito where codGabarito = (select max(codGabarito) from TreinoOrfGabarito) ";
             retorno[7] = " select codGabarito, codOrf from GabaritoOrf where codGabarito = (select max(codGabarito) from GabaritoOrf)";            
             retorno[8] = " select codOrf, codCodon, codonSequencia from OrfCodon where codOrf = (select max(codOrf) from OrfCodon)";
             retorno[9] = " select codGabarito, codCodon, nota from  Nota where codGabarito = (select max(codGabarito) from Nota)";
         
            return retorno;       
             
         }
         
        // retorna todos os comando de create das tabelas
         private String[] getSqlCriaTabelas(){
             
            String retorno[] = new String[12];
                 
            retorno[0] = " create table Treino ("
                   +"   cod int not null primary key generated by default as identity (start with 1, increment by 1),"
                   +"   nome varchar(20) not null," 
                   +"   descricao varchar(80),"
                   +"   unique(nome)" 
                   +")";
            
            retorno[1] = " create table Gabarito ("
                   +"   cod int not null primary key generated by default as identity (start with 1, increment by 1),"
                   +"   nome varchar(20) not null,"
                   +"   descricao varchar(50)," 
                   +"   geracoes int,"
                   +"   mutacao int,"
                   +"   fitness double,"
                   +"   tempoProcessamento timestamp,"
                   +"   populacao int,"
                   +"   elitismo int,"
                   +"   unique(nome)" 
                   +" )";
            
            retorno[2] = " create table Classe ("
                   +"   cod int not null primary key generated by default as identity (start with 1, increment by 1),"
                   +"   nome varchar(15) not null,"
                   +"   unique(nome)"
                   +" )";
            
            retorno[3] = " create table Codon("
                   +"   cod int not null primary key generated by default as identity (start with 1, increment by 1),"
                   +"   codon char(3) not null,"
                   +"   unique (codon)" 
                   +" )";
            
            retorno[4] = " create table Rede ("
                   +"   cod  int not null primary key generated by default as identity (start with 1, increment by 1),"
                   +"   codtreino  int not null,"
                   +"   nome varchar(20) not null,"
                   +"   descricao varchar(80) ,"
                   +"   rede clob(1M) ," 
                   +"   unique (nome)," 
                   +"   foreign key(codtreino) references Treino(cod)"
                   +"       on delete no action"
                   +"       on update no action"
                   +" )";
            
            retorno[5] = " create table Orf ("
                   +"   cod  int not null primary key generated by default as identity (start with 1, increment by 1),"
                   +"   codclasse  int not null,"
                   +"   nome varchar(250) not null,"
                   +"   descricao varchar(80) ,"
                   +"   tipo char(1) not null,"
                   +"   unique (nome, tipo)," 
                   +"   foreign key(codclasse) references Classe(cod)"
                   +"       on delete no action"
                   +"       on update no action"
                   +" )";
          
            retorno[6] = " create table TreinoOrfGabarito ("
                   +"   codGabarito  int not null,"
                   +"   codOrf  int not null,"
                   +"   codTreino  int not null,"
                   +"   caracteristica double,"
                   +"   primary key(codGabarito, codOrf, codTreino),"
                   +"   foreign key(codGabarito) references Gabarito(cod)"
                   +"       on delete no action"
                   +"       on update no action,"
                   +"   foreign key(codTreino)references Treino(cod)"
                   +"       on delete no action"
                   +"       on update no action,"
                   +"   foreign key(codOrf) references Orf(cod)"
                   +"       on delete no action"
                   +"       on update no action "    
                   +" )";
            
            retorno[7] = " create table GabaritoOrf ("
                   +"   codGabarito int not null,"
                   +"   codOrf int not null,"
                   +"   primary key(codGabarito, codOrf),"
                   +"   foreign key(codGabarito) references Gabarito(cod)"
                   +"       on delete no action"
                   +"       on update no action,"
                   +"   foreign key(codOrf) references Orf(cod)"
                   +"       on delete no action"
                   +"       on update no action"
                   +" )";
            
            retorno[8] = " create table OrfCodon ("
                   +"   codOrf int not null,"
                   +"   codCodon int not null,"
                   +"   codonSequencia int not null,"
                   +"   primary key(codOrf, codCodon, codonSequencia),"
                   +"   foreign key(codOrf) references Orf(cod)"
                   +"       on delete cascade"
                   +"       on update no action,"
                   +"   foreign key(codCodon) references Codon(cod)"
                   +"       on delete no action"
                   +"       on update no action"
                   +" )";
            
            retorno[9] = " create table  Nota ("
                   +"   codGabarito int not null,"
                   +"   codCodon int not null,"
                   +"   nota int not null,"
                   +"   primary key(codGabarito, codCodon, nota),"
                   +"   foreign key(codCodon) references Codon(cod)"
                   +"       on delete no action"
                   +"       on update no action,"
                   +"   foreign key(codGabarito) references Gabarito(cod)"
                   +"       on delete no action"
                   +"       on update no action"
                   +" )";
            
                         //preenche a tabela codon com os 64 cdons possveis
             retorno[10] = "insert into codon values " +
                     "(0, 'AAA'),  (1, 'GAA'),  (2, 'CAA'),  (3, 'TAA'),  (4, 'AGA'),  (5, 'GGA'),  (6, 'CGA'),  (7, 'TGA')," +
                     "(8, 'ACA'),  (9, 'GCA'),  (10, 'CCA'), (11, 'TCA'), (12, 'ATA'), (13, 'GTA'), (14, 'CTA'), (15, 'TTA')," +
                     "(16, 'AAG'), (17, 'GAG'), (18, 'CAG'), (19, 'TAG'), (20, 'AGG'), (21, 'GGG'), (22, 'CGG'), (23, 'TGG')," +
                     "(24, 'ACG'), (25, 'GCG'), (26, 'CCG'), (27, 'TCG'), (28, 'ATG'), (29, 'GTG'), (30, 'CTG'), (31, 'TTG')," +
                     "(32, 'AAC'), (33, 'GAC'), (34, 'CAC'), (35, 'TAC'), (36, 'AGC'), (37, 'GGC'), (38, 'CGC'), (39, 'TGC')," +
                     "(40, 'ACC'), (41, 'GCC'), (42, 'CCC'), (43, 'TCC'), (44, 'ATC'), (45, 'GTC'), (46, 'CTC'), (47, 'TTC')," +
                     "(48, 'AAT'), (49, 'GAT'), (50, 'CAT'), (51, 'TAT'), (52, 'AGT'), (53, 'GGT'), (54, 'CGT'), (55, 'TGT')," +
                     "(56, 'ACT'), (57, 'GCT'), (58, 'CCT'), (59, 'TCT'), (60, 'ATT'), (61, 'GTT'), (62, 'CTT'), (63, 'TTT')";
             
             //j insere as classes protena e no protena como default
             retorno[11] = "insert into classe values(1, 'Protena'), (2, 'No Protena')";

         
            return retorno;       
         }          
         
         // comandos para apagar todas as tabelas se elas existirem para poder criar tudo de novo
         // detalhe a ordem de delete  inversa a ordem de create se no pode ocorrer problemas com restrio de FK
         private String[] getSqlDeleteTables(){
                    String retorno[] = new String[10];
             
             retorno[9] = " drop table Treino ";
             retorno[8] = " drop table Gabarito";
             retorno[7] = " drop table Classe";
             retorno[6] = " drop table Codon";
             retorno[5] = " drop table Rede";
             retorno[4] = " drop table Orf ";
             retorno[3] = " drop table TreinoOrfGabarito ";
             retorno[2] = " drop table GabaritoOrf ";            
             retorno[1] = " drop table OrfCodon ";
             retorno[0] = " drop table Nota ";
         
             return retorno;  
         }    
         
         // get and set

        public String getDataBaseName() {
            
            return dbName;
        }

        public void setDataBaseName(String dataNaseName) {
            log.info("Setando nome do banco para = " + dataNaseName);
            dbName = dataNaseName;
        }

        public String getCodUniqueException() {
            return "23505";
        }

        public String getCodVioletionFKException() {
            return "23503";
        }
         
         
}
    



