/*
 * CodonDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class NotaDAO{
    
    public static Logger log = new Logger(NotaDAO.class.getName());       
    public static final String TABLE = "Nota";
    // evita que seja instnciado.
    private NotaDAO() {
    }
    
    public static void adiciona(NotaVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (codGabarito, codCodon, nota ) values (?,?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCodGabarito());
                 stmt.setInt(2,objeto.getCodCodon());
                 stmt.setInt(3,objeto.getNota());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar NotaVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
    public static void exclui(NotaVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where codGabarito = ? and codCodon = ? and nota = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCodGabarito());
                 stmt.setInt(2,objeto.getCodCodon());
                 stmt.setInt(3,objeto.getNota());              
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static void exclui(CodonVO objeto)throws VioletionFKException{
        Connection con = null;
        PreparedStatement stmt = null;
        try {
             con = ConnectionFactory.getConnection();
             String comando = "delete from " + TABLE + " where codCodon = ? ";
             stmt = con.prepareStatement(comando);
             // seta os valores
             stmt.setInt(1,objeto.getCod());                           
             
             // executa
             stmt.execute();
             
          } catch (SQLException sqle) {
              String theError = (sqle).getSQLState();
              // verifica se  um erro de violao de Foreign Key
              if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                  throw new VioletionFKException();
              } else {
                   log.exception(sqle);
              }    
          } finally {
                ConnectionFactory.closeStmt(stmt);
          }
   }
    
    public static void exclui(GabaritoVO objeto)throws VioletionFKException{
        Connection con = null;
        PreparedStatement stmt = null;
        try {
             con = ConnectionFactory.getConnection();
             String comando = "delete from " + TABLE + " where codGabarito = ? ";
             stmt = con.prepareStatement(comando);
             // seta os valores
             stmt.setInt(1,objeto.getCod());
             
             // executa
             stmt.execute();
             
          } catch (SQLException sqle) {
              String theError = (sqle).getSQLState();
              // verifica se  um erro de violao de Foreign Key
              if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                  throw new VioletionFKException();
              } else {
                   log.exception(sqle);
              }    
          } finally {
                ConnectionFactory.closeStmt(stmt);
          }
   }
    
    public static ArrayList<NotaVO> getListaObjetoByUnique(GabaritoVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<NotaVO> array = new ArrayList<NotaVO>();
            ResultSet rs = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 
                 comando = "select codGabarito, codCodon, nota from " + TABLE + " where codGabarito = ? ";
                 stmt = con.prepareStatement(comando);
                 stmt.setInt(1,objeto.getCod());
                                                   
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 NotaVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new NotaVO();
                        obj.setCodGabarito(rs.getInt("codGabarito"));
                        obj.setCodCodon(rs.getInt("codCodon"));                           
                        obj.setNota(rs.getInt("nota"));
                        array.add(obj);
                 }
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return array;
       }
    
    public static ArrayList<NotaVO> getListaObjetoByUnique(CodonVO objeto){
        Connection con = null;
        PreparedStatement stmt = null;
        ArrayList<NotaVO> array = new ArrayList<NotaVO>();
        ResultSet rs = null;
        
        try {
             con = ConnectionFactory.getConnection();
             String comando = null;
             // verifica qual unique sera a busca
             
             comando = "select codGabarito, codCodon, nota from " + TABLE + " where codCodon = ? ";
             stmt = con.prepareStatement(comando);
             stmt.setInt(1,objeto.getCod());
                                               
             //executa
             rs = stmt.executeQuery();
             // posiciona o ponteiro na 1 posio e ve se no  nulo
             NotaVO obj;
             // posiciona o ponteiro na 1 posio e ve se no  nulo
             while(rs.next()){
                    // Cria e popula o objeto
                    obj = new NotaVO();
                    obj.setCodGabarito(rs.getInt("codGabarito"));
                    obj.setCodCodon(rs.getInt("codCodon"));                           
                    obj.setNota(rs.getInt("nota"));
                    array.add(obj);
             }
             
          } catch (SQLException sqle) {
               log.exception(sqle);
                  
          } finally {
                ConnectionFactory.closeAll(stmt,  rs);
             }
          return array;
   }

    
}
    


