/*
 * CodonDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class ORFCodonDAO {
    
    public static Logger log = new Logger(ORFCodonDAO.class.getName());       
    public static final String TABLE = "OrfCodon";
    // evita que seja instnciado.
    private ORFCodonDAO() {
    }
    
    public static void adiciona(ORFCodonVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (codOrf, codCodon, codonSequencia ) values (?,?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCodORF());
                 stmt.setInt(2,objeto.getCodCodon());
                 stmt.setInt(3,objeto.getCodonSequencia());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar ORFCodonVO", sqle);
                  }    
              }finally {
                    ConnectionFactory.closeStmt(stmt);
              } 
       }
    
    
    public static void exclui(ORFVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "DELETE FROM " + TABLE + " WHERE codORF = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getIntCodORF());             
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static ArrayList<ORFCodonVO> getListaObjetoByUnique(ORFVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<ORFCodonVO> array = new ArrayList<ORFCodonVO>();
            ResultSet rs = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 comando = "select codOrf, codCodon, codonSequencia from " + TABLE + " where codOrf = ? order by codonSequencia";
                 stmt = con.prepareStatement(comando);
                 stmt.setInt(1,objeto.getCod());
                                                   
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 ORFCodonVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new ORFCodonVO();
                        obj.setCodORF(rs.getInt("codOrf"));
                        obj.setCodCodon(rs.getInt("codCodon"));                           
                        obj.setCodonSequencia(rs.getInt("codonSequencia"));
                        array.add(obj);
                 }
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);       
              }finally {
                    ConnectionFactory.closeAll(stmt,rs);
              }
            
              return array;
       }
    
    public static ArrayList<ORFCodonVO> getListaObjetoByUnique(CodonVO objeto){
        Connection con = null;
        PreparedStatement stmt = null;
        ArrayList<ORFCodonVO> array = new ArrayList<ORFCodonVO>();
        ResultSet rs = null;
        
        try {
             con = ConnectionFactory.getConnection();
             String comando = null;
             // verifica qual unique sera a busca
             
             comando = "select codOrf, codCodon, codonSequencia from " + TABLE + " where codCodon = ? ";
             stmt = con.prepareStatement(comando);
             stmt.setInt(1,objeto.getCod());
                                               
             //executa
             rs = stmt.executeQuery();
             // posiciona o ponteiro na 1 posio e ve se no  nulo
             ORFCodonVO obj;
             // posiciona o ponteiro na 1 posio e ve se no  nulo
             while(rs.next()){
                    // Cria e popula o objeto
                    obj = new ORFCodonVO();
                    obj.setCodORF(rs.getInt("codOrf"));
                    obj.setCodCodon(rs.getInt("codCodon"));                           
                    obj.setCodonSequencia(rs.getInt("codonSequencia"));
                    array.add(obj);
             }
             
          } catch (SQLException sqle) {
               log.exception(sqle);
                  
          } finally {
                ConnectionFactory.closeAll(stmt,  rs);
             }
          return array;
   }

    public static int getQtdeObjetoByUnique(ORFVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<ORFCodonVO> array = new ArrayList<ORFCodonVO>();
            ResultSet rs = null;
            int resposta = 0; 
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 
                 comando = "select count (*) from " + TABLE + " where codOrf = ? ";
                 stmt = con.prepareStatement(comando);
                 stmt.setInt(1,objeto.getCod());
                                                   
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 ORFCodonVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if(rs.next()){
                        // Cria e popula o objeto
                        resposta = rs.getInt(1);
                 }
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return resposta;
       }

    
}
    


