/*
 * ORFDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class ORFDAO {
    
    public static Logger log = new Logger(ORFDAO.class.getName());       
    public static final String TABLE = "Orf";
    // evita que seja instnciado.
    private ORFDAO() {
    }
    
    public static void adiciona(ORFVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (codclasse, nome, descricao, tipo ) values (?,?,?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getClasse().getCod());
                 stmt.setString(2, objeto.getNome());
                 stmt.setString(3, objeto.getDescricao());
                 stmt.setString(4, objeto.getTipo());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar ORFVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
    public static void atualiza(ORFVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "update " + TABLE + " set codclasse = ? , nome = ? , descricao = ? , tipo = ? where cod = ?";
                 stmt = con.prepareStatement(comando);
               
                 // seta os valores
                 stmt.setInt(1, objeto.getClasse().getCod());
                 stmt.setString(2,objeto.getNome());                 
                 stmt.setString(3,objeto.getDescricao());                 
                 stmt.setString(4,objeto.getTipo());   
                 stmt.setInt(5, objeto.getCod());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                      throw new UniqueException();
                  } else {
                      log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static void exclui(ORFVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where cod = ?";
                 stmt = con.prepareStatement(comando);
                 
                // seta os valores
                 stmt.setInt(1,objeto.getCod());                 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static ORFVO getObjetoByUnique(ORFVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            ORFVO obj = objeto;
            ResultSet rs = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 if (!((obj.getCod() ) == 0)){
                    comando = "select cod, codclasse, nome, descricao, tipo from " + TABLE + " where cod = ? " ;
                    stmt = con.prepareStatement(comando);
                    stmt.setInt(1,objeto.getCod());
                    
                 }else {
                     comando = "select cod, codclasse, nome, descricao, tipo  from " + TABLE + " where nome = ? and tipo = ?" ;
                     stmt = con.prepareStatement(comando);
                     stmt.setString(1,objeto.getNome());
                     stmt.setString(2,objeto.getTipo());
                 }
                                  
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if (rs.next()){
                        // popula o objeto
                        obj = new ORFVO();
                        
                        ClasseVO classeVO = new ClasseVO();
                        classeVO.setCod(rs.getInt("codclasse"));
                        classeVO = ClasseDAO.getObjetoByUnique(classeVO);
                        obj.setClasse(classeVO);   
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));
                        obj.setDescricao(rs.getString("descricao"));
                        obj.setTipo(rs.getString("tipo"));
                        //pegando o contedo da orf
                        String comando2 = "select codon.codon from orfcodon " +
                                " inner join codon on codon.cod = orfcodon.codcodon" +
                                " where orfcodon.codorf = ? " +
                                " order by orfcodon.codonsequencia";
                      
                        stmt = con.prepareStatement(comando2);
                        stmt.setInt(1, rs.getInt("cod"));
                        ResultSet rs2 = stmt.executeQuery();
                        StringBuffer conteudo = new StringBuffer();
                        while(rs2.next()){
                            conteudo.append(rs2.getString("codon"));
                        }
                        obj.setConteudo(conteudo);                        
                 }
                      
         
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return obj;
       }
    
    public static ArrayList<ORFVO> getListaObjetoORFBase(){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<ORFVO> array = new ArrayList<ORFVO>();
            ResultSet rs = null;
            ResultSet rs2 = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select cod, codclasse, nome, descricao, tipo from " + TABLE + " where tipo = 'b' order by cod";
                 stmt = con.prepareStatement(comando);
                                      
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        ORFVO orfVO = new ORFVO();
                        //instancia classe ClasseVO e seta a penas o cod para que a 
                        //ClasseDAO retorne o objeto ClasseVO do cod passado
                        ClasseVO classeVO = new ClasseVO();
                        classeVO.setCod(rs.getInt("codclasse"));
                        classeVO = ClasseDAO.getObjetoByUnique(classeVO);
                        orfVO.setClasse(classeVO);                        
                        orfVO.setCod(rs.getInt("cod"));
                        orfVO.setNome(rs.getString("nome"));
                        orfVO.setDescricao(rs.getString("descricao"));
                        orfVO.setTipo(rs.getString("tipo"));
                        
                        String comando2 = "select codon.codon from orfcodon " +
                                " inner join codon on codon.cod = orfcodon.codcodon" +
                                " where orfcodon.codorf = ? " +
                                " order by orfcodon.codonsequencia";
                      
                        stmt = con.prepareStatement(comando2);
                        stmt.setInt(1, rs.getInt("cod"));
                        rs2 = stmt.executeQuery();
                        StringBuffer conteudo = new StringBuffer();
                        while(rs2.next()){
                            conteudo.append(rs2.getString("codon"));
                        }
                        orfVO.setConteudo(conteudo);
                        array.add(orfVO);
                 }
                  
                                 
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
                    ConnectionFactory.closeAll(stmt, rs2);
              }
              return array;
       }
    
       public static ArrayList<ORFVO> getListaObjetoORFTreino() {
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<ORFVO> array = new ArrayList<ORFVO>();
            ResultSet rs = null;
            ResultSet rs2 = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select cod, codclasse, nome, descricao, tipo from " + TABLE + " where tipo = 't' order by cod";
                 stmt = con.prepareStatement(comando);
                                      
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        ORFVO obj = new ORFVO();
                        //instancia classe ClasseVO e seta a penas o cod para que a 
                        //ClasseDAO retorne o objeto ClasseVO do cod passado
                        ClasseVO classeVO = new ClasseVO();
                        classeVO.setCod(rs.getInt("codclasse"));
                        classeVO = ClasseDAO.getObjetoByUnique(classeVO);
                        obj.setClasse(classeVO);                    
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));
                        obj.setDescricao(rs.getString("descricao"));
                        obj.setTipo(rs.getString("tipo"));
                        
                        String comando2 = "select codon.codon from orfcodon " +
                                " inner join codon on codon.cod = orfcodon.codcodon" +
                                " where orfcodon.codorf = ? " +
                                " order by orfcodon.codonsequencia";
                      
                        stmt = con.prepareStatement(comando2);
                        stmt.setInt(1, rs.getInt("cod"));
                        rs2 = stmt.executeQuery();
                        StringBuffer conteudo = new StringBuffer();
                        while(rs2.next()){
                            conteudo.append(rs2.getString("codon"));
                        }
                        obj.setConteudo(conteudo);
                        array.add(obj);
                 }      
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
                    ConnectionFactory.closeAll(stmt, rs2);
              }
              return array;
        }

        public static ArrayList getListaObjeto() {
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<ORFVO> array = new ArrayList<ORFVO>();
            ResultSet rs = null;
            ResultSet rs2 = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select cod, codclasse, nome, descricao, tipo from " + TABLE + " order by cod";
                 stmt = con.prepareStatement(comando);
                                      
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        ORFVO obj = new ORFVO();
                        //instancia classe ClasseVO e seta a penas o cod para que a 
                        //ClasseDAO retorne o objeto ClasseVO do cod passado
                        ClasseVO classeVO = new ClasseVO();
                        classeVO.setCod(rs.getInt("codclasse"));
                        classeVO = ClasseDAO.getObjetoByUnique(classeVO);
                        obj.setClasse(classeVO);                    
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));
                        obj.setDescricao(rs.getString("descricao"));
                        obj.setTipo(rs.getString("tipo"));
                        
                        String comando2 = "select codon.codon from orfcodon " +
                                " inner join codon on codon.cod = orfcodon.codcodon" +
                                " where orfcodon.codorf = ? " +
                                " order by orfcodon.codonsequencia";
                      
                        stmt = con.prepareStatement(comando2);
                        stmt.setInt(1, rs.getInt("cod"));
                        rs2 = stmt.executeQuery();
                        StringBuffer conteudo = new StringBuffer();
                        while(rs2.next()){
                            conteudo.append(rs2.getString("codon"));
                        }
                        obj.setConteudo(conteudo);
                        array.add(obj);
                 }      
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
                    ConnectionFactory.closeAll(stmt, rs2);
              }
              return array;
        }
    }
    


