/*
 * RedeDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import javax.swing.JOptionPane;

import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class RedeDAO {
    
    public static Logger log = new Logger(RedeDAO.class.getName());       
    public static final String TABLE = "Rede";
    // evita que seja instnciado.
    private RedeDAO() {
    }
    
    public static void adiciona(RedeVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (codtreino, nome, descricao,rede) values (?,?,?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 
                 stmt.setInt(1,objeto.getCodTreino());
                 stmt.setString(2, objeto.getNome());
                 stmt.setString(3, objeto.getDescricao());
                 stmt.setString(4,objeto.getRedeTXT());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar RedeVO  1 ", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
    public static void atualiza(RedeVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "update " + TABLE + " set codtreino = ? , nome = ?, descricao = ?, rede = ? where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 
                 stmt.setInt(1,objeto.getCodTreino());
                 stmt.setString(2, objeto.getNome());
                 stmt.setString(3, objeto.getDescricao());
                 stmt.setString(4,objeto.getRedeTXT());
                 stmt.setInt(5,objeto.getCod());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                      throw new UniqueException();
                  } else {
                      log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static void exclui(RedeVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCod());                 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static RedeVO getObjetoByUnique(RedeVO objeto)throws IOException{
            Connection con = null;
            PreparedStatement stmt = null;
            RedeVO obj = objeto;
            ResultSet rs = null;
            String aux;
            int c;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 if (!((obj.getCod() ) == 0)){
                    comando = "select cod, codtreino, nome, descricao, rede from " + TABLE + " where cod = ? " ;
                    stmt = con.prepareStatement(comando);
                    stmt.setInt(1,objeto.getCod());
                    
                 }else {
                     comando = "select cod, codtreino, nome, descricao, rede from " + TABLE + " where nome = ? " ;
                     stmt = con.prepareStatement(comando);
                     stmt.setString(1,objeto.getNome());
                 }
                                  
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if (rs.next()){
                        // popula o objeto
                        obj = new RedeVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setCodTreino(rs.getInt("codtreino"));                           
                        obj.setNome(rs.getString("nome"));
                        obj.setDescricao(rs.getString("descricao"));
                        java.sql.Clob aclob = rs.getClob("rede");
                        java.io.InputStream ip = rs.getAsciiStream("rede");
                        c = ip.read();
                        aux="";
                        while (c > 0)
                        {
                           aux=aux+((char)c);
                           c = ip.read();
                        }
                        obj.setRedeTXT(aux);
                 }
                      
         
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return obj;
       }
    
    public static ArrayList<RedeVO> getListaObjeto() throws IOException{
            Connection con = null;
            PreparedStatement stmt = null;
            String aux;
            ArrayList<RedeVO> array = new ArrayList<RedeVO>();
            ResultSet rs = null;
            int c;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select cod, codtreino, nome, descricao, rede from " + TABLE + " order by cod";
                 stmt = con.prepareStatement(comando);
                                      
                 //executa
                 rs = stmt.executeQuery();
                 RedeVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new RedeVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setCodTreino(rs.getInt("codtreino"));                           
                        obj.setNome(rs.getString("nome"));
                        obj.setDescricao(rs.getString("descricao")); 
                        array.add(obj);
                 }
                  
                                 
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
       }
    }
    


