/*
 * TreinoDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class TreinoDAO {
    
    public static Logger log = new Logger(TreinoDAO.class.getName());       
    public static final String TABLE = "Treino";
    // evita que seja instnciado.
    private TreinoDAO() {
    }
    
    public static void adiciona(TreinoVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (nome, descricao) values (?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setString(1, objeto.getNome());
                 stmt.setString(2, objeto.getDescricao());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar TreinoVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
    public static void atualiza(TreinoVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "update " + TABLE + " set nome = ?, descricao = ?  where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                                  
                 stmt.setString(1, objeto.getNome());
                 stmt.setString(2, objeto.getDescricao());
                 stmt.setInt(3,objeto.getCod());
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                      throw new UniqueException();
                  } else {
                      log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static void exclui(TreinoVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where cod = ?";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCod());                 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static TreinoVO getObjetoByUnique(TreinoVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            TreinoVO obj = objeto;
            ResultSet rs = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 // verifica qual unique sera a busca
                 if (!((obj.getCod() ) == 0)){
                    comando = "select cod, nome, descricao from " + TABLE + " where cod = ? " ;
                    stmt = con.prepareStatement(comando);
                    stmt.setInt(1,objeto.getCod());
                    
                 }else {
                     comando = "select cod, nome, descricao from " + TABLE + " where nome = ? " ;
                     stmt = con.prepareStatement(comando);
                     stmt.setString(1,objeto.getNome());
                 }
                                  
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if (rs.next()){
                        // popula o objeto
                        obj = new TreinoVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));                           
                        obj.setDescricao(rs.getString("descricao"));                           
                 }    
         
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return obj;
       }
    
    public static ArrayList<TreinoVO> getListaObjeto(){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<TreinoVO> array = new ArrayList<TreinoVO>();
            ResultSet rs = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select cod, nome, descricao from " + TABLE + " order by cod";
                 stmt = con.prepareStatement(comando);
                                      
                 //executa
                 rs = stmt.executeQuery();
                 
                 TreinoVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new TreinoVO();
                        obj.setCod(rs.getInt("cod"));
                        obj.setNome(rs.getString("nome"));                           
                        obj.setDescricao(rs.getString("descricao"));
                        array.add(obj);
                 }
                                 
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
      }

}
    


