/*
 * CodonDAO.java
 *
 * Created on 18 de Julho de 2007, 22:47
 *
 * By: Oslei Taborda Ribas-
 */

package jorfinder.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import jorfinder.information.*;
import jorfinder.util.*;
import jorfinder.log2java.Logger;


/**
 *
 * @author oeslei
 */
public class TreinoORFGabaritoDAO {
    
    public static Logger log = new Logger(TreinoORFGabaritoDAO.class.getName());       
    public static final String TABLE = "TreinoOrfGabarito";
    // evita que seja instnciado.
    private TreinoORFGabaritoDAO() {
    }
    
    public static void adiciona(TreinoORFGabaritoVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "insert into " + TABLE + " (codGabarito, codOrf, codTreino, caracteristica) values (?,?,?,?)";
                 stmt = con.prepareStatement(comando);
                 // seta os valores
                 stmt.setInt(1,objeto.getCodGabarito());
                 stmt.setInt(2,objeto.getCodORF());
                 stmt.setInt(3,objeto.getCodTreino());
                 stmt.setDouble(4,objeto.getCaracteristica());		 
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                       throw new UniqueException();
                  } else {
                        log.exception("Erro ao cadastrar TreinoORFGabaritoVO", sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    
    public static void atualiza(TreinoORFGabaritoVO objeto)throws UniqueException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "update " + TABLE + " set caracteristica = ? where codGabarito = ? and codOrf = ? and codTreino = ?";
                 stmt = con.prepareStatement(comando);
                 
                // seta os valores
                 stmt.setDouble(1,objeto.getCaracteristica());
                 stmt.setInt(2,objeto.getCodGabarito());
                 stmt.setInt(3,objeto.getCodORF());
                 stmt.setInt(4,objeto.getCodTreino());
                               
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de chave unique
                  if (theError.equals(ConnectionFactory.getCodUniqueException())){
                      throw new UniqueException();
                  } else {
                      log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static void exclui(TreinoORFGabaritoVO objeto)throws VioletionFKException{
            Connection con = null;
            PreparedStatement stmt = null;
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "delete from " + TABLE + " where codTreino = ?";
                 stmt = con.prepareStatement(comando);
                
                 // seta os valores
                 stmt.setInt(1,objeto.getCodTreino());               
                 
                 // executa
                 stmt.execute();
                 
              } catch (SQLException sqle) {
                  String theError = (sqle).getSQLState();
                  // verifica se  um erro de violao de Foreign Key
                  if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                      throw new VioletionFKException();
                  } else {
                       log.exception(sqle);
                  }    
              } finally {
                    ConnectionFactory.closeStmt(stmt);
              }
       }
    
    public static TreinoORFGabaritoVO getObjetoByUnique(TreinoORFGabaritoVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            TreinoORFGabaritoVO obj = objeto;
            ResultSet rs = null;
            
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = null;
                 
                // verifica qual unique sera a busca
                 comando = "select codGabarito, codOrf, codTreino, caracteristica from " + TABLE + " where codGabarito = ? and codOrf = ? and codTreino = ?";
                 stmt = con.prepareStatement(comando);
                 stmt.setInt(1,objeto.getCodGabarito());
                 stmt.setInt(2,objeto.getCodORF());
                 stmt.setInt(3,objeto.getCodTreino());    
                    
                                                   
                 //executa
                 rs = stmt.executeQuery();
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 if (rs.next()){
                        // popula o objeto
                        obj = new TreinoORFGabaritoVO();
                        obj.setCodGabarito(rs.getInt("codGabarito"));
                        obj.setCodORF(rs.getInt("codOrf"));                           
                        obj.setCodTreino(rs.getInt("codTreino"));
                        obj.setCaracteristica(rs.getDouble("caracteristica"));
                 }
                      
         
                 
              } catch (SQLException sqle) {
                   log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt,  rs);
                 }
              return obj;
       }
   
    
    public static ArrayList<TreinoORFGabaritoVO> getListaObjeto(){
            Connection con = null;
            PreparedStatement stmt = null;
            ArrayList<TreinoORFGabaritoVO> array = new ArrayList<TreinoORFGabaritoVO>();
            ResultSet rs = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String comando = "select codGabarito, codOrf, codTreino, caracteristica from " + TABLE + " order by codGabarito";
                 stmt = con.prepareStatement(comando);
                 log.info("Buscando no BD uma lista de todos os TreinoORFGabaritoVO");
                                      
                 //executa
                 rs = stmt.executeQuery();
                 TreinoORFGabaritoVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new TreinoORFGabaritoVO();
                        obj.setCodGabarito(rs.getInt("codGabarito"));
                        obj.setCodORF(rs.getInt("codOrf"));                           
                        obj.setCodTreino(rs.getInt("codTreino"));
                        obj.setCaracteristica(rs.getDouble("caracteristica"));                          
                        array.add(obj);
                 }
                  
                                 
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
       }
    
 
        public static ArrayList<TreinoORFGabaritoVO> getTreinoUsaCodTreino(TreinoVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            TreinoVO obj1 = objeto;
            ArrayList<TreinoORFGabaritoVO> array = new ArrayList<TreinoORFGabaritoVO>();
            ResultSet rs = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String  comando = "select distinct (codGabarito) from " + TABLE + " where codTreino = ?  order by codGabarito ";
                 stmt = con.prepareStatement(comando);
                 stmt.setInt(1,obj1.getCod());  
                 log.info("Buscando no BD uma lista de todos os TreinoORFGabaritoVO");
                                      
                 //executa
                 rs = stmt.executeQuery();
                 TreinoORFGabaritoVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new TreinoORFGabaritoVO();
                        obj.setCodGabarito(rs.getInt("codGabarito"));
                                               
                        array.add(obj);
                 }
                  
                                 
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
       }
        
      
         public static ArrayList<TreinoORFGabaritoVO> getTreinoUsaCodTreino1(TreinoVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            TreinoVO obj1 = objeto;
            ArrayList<TreinoORFGabaritoVO> array = new ArrayList<TreinoORFGabaritoVO>();
            ResultSet rs = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String  comando = "select distinct (codOrf) from " + TABLE + " where codTreino = ?  order by codOrf ";
                 stmt = con.prepareStatement(comando);
                 stmt.setInt(1,obj1.getCod());  
                 log.info("Buscando no BD uma lista de todos os TreinoORFGabaritoVO");
                                      
                 //executa
                 rs = stmt.executeQuery();
                 TreinoORFGabaritoVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new TreinoORFGabaritoVO();
                        obj.setCodORF(rs.getInt("codOrf"));
                                               
                        array.add(obj);
                 }
                  
                                 
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
       }
       
       public static ArrayList<TreinoORFGabaritoVO> getTreinoUsaCodTreino2(TreinoVO objeto){
            Connection con = null;
            PreparedStatement stmt = null;
            TreinoVO obj1 = objeto;
            ArrayList<TreinoORFGabaritoVO> array = new ArrayList<TreinoORFGabaritoVO>();
            ResultSet rs = null;
                        
            try {
                 con = ConnectionFactory.getConnection();
                 String  comando = "select codGabarito, codOrf, codTreino, caracteristica from " + TABLE + " where codTreino = ?  order by codOrf, codGabarito ";
                 stmt = con.prepareStatement(comando);
                 stmt.setInt(1,obj1.getCod());  
                 log.info("Buscando no BD uma lista de todos os TreinoORFGabaritoVO");
                                      
                 //executa
                 rs = stmt.executeQuery();
                 TreinoORFGabaritoVO obj;
                 // posiciona o ponteiro na 1 posio e ve se no  nulo
                 while(rs.next()){
                        // Cria e popula o objeto
                        obj = new TreinoORFGabaritoVO();
                        obj.setCodGabarito(rs.getInt("codGabarito"));
                        obj.setCodORF(rs.getInt("codOrf"));                           
                        obj.setCodTreino(rs.getInt("codTreino"));
                        obj.setCaracteristica(rs.getDouble("caracteristica")); 
                        array.add(obj);
                 }
                  
                                 
                 
              } catch (SQLException sqle) {
                  log.exception(sqle);
                      
              } finally {
                    ConnectionFactory.closeAll(stmt, rs);
              }
              return array;
       }
       
        
    }
    


