/*
 *
 */
  
package jorfinder.view;

import java.awt.Font;
import java.awt.Frame;
import java.io.File;
import java.io.IOException;
import javax.swing.JDialog;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;
  
/** 
 * Usada pra plotar o grfico com o RNA ordenado
 */
public class GraficoFORM extends JDialog  {

	private static final long serialVersionUID = 1L;
    
     /**
     * @param tituloFrame  Ttulo do Frame.
     * @param tituloGraf  Ttulo do Grfico.
     * @param rna Contedo do grfico.
     */
    public GraficoFORM(JDialog telaPai, final String tituloFrame, String tituloGraf, byte[] dadosGraf) {
        
        super(telaPai, tituloFrame);       
    	
        final JFreeChart chart = createCombinedChart(tituloGraf, dadosGraf);
        final ChartPanel panel = new ChartPanel(chart, true, true, true, false, true);
        panel.setPreferredSize(new java.awt.Dimension(500, 500));
        add(panel);
        
        pack();
        RefineryUtilities.centerFrameOnScreen(this);
        setVisible(true);
    }
    
     /**
     * Grava um arquivo JPG do grfico no diretrio padro
     * @param grafico
     * @param titulo
     * @throws IOException
     */
    public void gravaJPEG(JFreeChart grafico, String titulo) throws IOException{
        ChartUtilities.saveChartAsJPEG(new File(titulo.replace("RNA", "JPG")), grafico, 500, 500);
    }
    
    /**
     * Seta caracteristicas do Grfico
     * @param titulo
     * @return Uma instancia da classe JFreeChart.
     */
    private JFreeChart createCombinedChart(String titulo, byte[] dadosGraf) {

        // create subplot 1...
        final XYDataset data1 = createDataset1(dadosGraf);
        final XYItemRenderer renderer1 = new StandardXYItemRenderer();
        final NumberAxis rangeAxis1 = new NumberAxis("Notas dos Cdons");
        final XYPlot subplot1 = new XYPlot(data1, null, rangeAxis1, renderer1);
        subplot1.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);
        
        final XYTextAnnotation annotation = new XYTextAnnotation("Hello!", 50.0, 10000.0);
        annotation.setFont(new Font("SansSerif", Font.PLAIN, 9));
        annotation.setRotationAngle(Math.PI / 4.0);
        subplot1.addAnnotation(annotation);
        

        // parent plot...
        final CombinedDomainXYPlot plot = new CombinedDomainXYPlot(new NumberAxis("Sequncia"));
        plot.setGap(10.0);
        
        // add the subplots...
        plot.add(subplot1, 1);
        plot.setOrientation(PlotOrientation.VERTICAL);

        // return a new chart containing the overlaid plot...
        return new JFreeChart(titulo,
                              JFreeChart.DEFAULT_TITLE_FONT, plot, true);

    }

    /**
     * Insere no Grfico o int[] rna passado.
     * @return Srie de valores a serem plotados no grfico.
     */
    private XYDataset createDataset1(byte[] dadosGraf) {
        final XYSeries series1 = new XYSeries("Curva de Padro");
        for (int cont = 1; cont < dadosGraf.length; cont++){        	
        	series1.add(cont, Double.valueOf(dadosGraf[cont]));       	
        }       

        final XYSeriesCollection collection = new XYSeriesCollection();
        collection.addSeries(series1);
        return collection;
    }
}
