/*
 * AGBO.java
 *
 * Created on 28 de Novembro de 2007, 10:00
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package jorfinder.business;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import jorfinder.information.CodonVO;
import jorfinder.information.GabaritoVO;
import jorfinder.information.NotaVO;
import jorfinder.information.RedeVO;
import jorfinder.log2java.Logger;
import jorfinder.persistence.CodonDAO;
import jorfinder.persistence.GabaritoDAO;
import jorfinder.persistence.NotaDAO;
import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

/**
 *
 * @author matheus
 */
public class AGBO {
    
    private ArrayList<CodonVO> codons;
    private ArrayList<ArrayList<NotaVO>> conjGabaritos = new ArrayList<ArrayList<NotaVO>>();
    public static Logger log = new Logger(AGBO.class.getName());     
    /** Creates a new instance of AGBO */
    public AGBO(RedeVO redevo) {
        //taca os cdons na memria
        CodonDAO codondao = new CodonDAO();
        this.codons = codondao.getListaObjeto();
        codondao.commitAndCloseConn(); 
        
        //tava os gabaritos na memria
        GabaritoDAO gabaritodao = new GabaritoDAO();
        NotaDAO notasdao = new NotaDAO();
        for (GabaritoVO gabvo : gabaritodao.getListaObjeto(redevo)) {
            this.conjGabaritos.add(notasdao.getObjetoByUnique(gabvo));    
        }
        gabaritodao.commitAndCloseConn();
        notasdao.commitAndCloseConn();
    }  
    
    public static void gravaArquivoRede(String path, String conteudo){       
        File arquivo = new File(path);
        if(!arquivo.exists()){
        BufferedWriter out;
            try {
                out = new BufferedWriter(new FileWriter(arquivo));
                out.write(conteudo);
                out.close();
            } catch (IOException ex) {
                log.exception("erro ao executar gravaArquivoRede", ex);
            }    
        }
    }  
    
    public byte[] getTraducaoCodOrf(String orf){
        String codon;
        byte[] traducao = new byte[orf.length()/3];
        for(int cont = 0; cont < orf.length()/3; cont++){		
            codon = orf.substring(cont*3, (cont + 1)*3);
            traducao[cont] = this.getCodCodon(codon);
        }
        return traducao;
    }   
    
    public byte getCodCodon(String codon){
        byte cod = 0;
        boolean conferido = false;
           
        for(int c=0; c < this.codons.size() && !conferido; c++){
            if(this.codons.get(c).getCodon().equals(codon)){
                cod = (byte) this.codons.get(c).getCod();
                conferido = true;                
            }
        }
        return cod;
    }
        
    public byte[][] getTraducaoGabarito(byte[] arrayCodonOrf){
        
        byte[][] traducao = new byte[conjGabaritos.size()][arrayCodonOrf.length];
        for(int c =0; c < conjGabaritos.size(); c++){
            for(int cont =0; cont < arrayCodonOrf.length; cont++){
                traducao[c][cont] = (byte) conjGabaritos.get(c).get(arrayCodonOrf[cont]).getNota();
                
            }
        }
        
        return traducao;
    }    
    
    public double[] getFitness(byte[][] traducaoDaOrf){
        double[] notas = new double[traducaoDaOrf.length];
        for(int cont=0; cont<traducaoDaOrf.length; cont++){
            notas[cont] = getFitnessComUmaOrf(traducaoDaOrf[cont]);
            
        }
        return notas;
    }
    

    
    /* metodo usado para calcular o fitNess  um dos metodos principais do AG
	*/
	 public static Double getFitnessComUmaOrf(byte[] traducaoDaOrf1){   
                // classe para calculos estatisticos
                DescriptiveStatistics stats = DescriptiveStatistics.newInstance();
                double somaDiferenca = 0.0;
                double difMedia;
                // adiciona os dados do array  e soma a diferena
                for( int i = 1; i < traducaoDaOrf1.length; i++) {
                        stats.addValue(traducaoDaOrf1[i]);
                        somaDiferenca += Math.abs(traducaoDaOrf1[i] - traducaoDaOrf1[i-1]);
                }
                
                difMedia  = somaDiferenca/traducaoDaOrf1.length;
                
                float d_alta = triang((float) difMedia, 64);
                float d_baixa = triang((float) difMedia,1);
                double fitness = (double) d_baixa/d_alta;  
               
                return fitness  ;                
	}
        
        public static float triang ( float x, float centro ) {
                float y, d1, d;
                d = 64;
                d1 = d + 5;
                if (x<centro)
                        y = (float) (1./d1)*(x+d1-centro);
                else
                        y = (float) (1./d1)*(-x+d1+centro);
                
                return (y);
        }
}
