package jorfinder.persistence;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import jorfinder.util.UniqueException;
import jorfinder.util.VioletionFKException;


public abstract class AbstractDAO<ValueObject> {
	
	protected Connection conn;
	
	// contrutor principal
	protected AbstractDAO(){
		conn = jorfinder.persistence.ConnectionFactory.getConnection();
		try {
			conn.setAutoCommit(false);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	
	}
	
	// esse contrutor vai ser usado quando a mesma conexo for compartilhada entre as DAO's
	// em situaes em que ocorra a mesma transao usando vrias DAO  necessrio isso.
	protected AbstractDAO(Connection conn){
		this.conn = conn;
		try {
			conn.setAutoCommit(false);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	// apenas d o commit
	public void commit(){
		try {
			conn.commit();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	// apenas d o rollback
	public void rollback(){
		try {
			conn.rollback();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	// aqui  executa um commit e fechado a conexo
	public void commitAndCloseConn(){
		try {
			conn.commit();
			jorfinder.persistence.ConnectionFactory.closeConnection(conn);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	// aqui  executado um rollback e fechado a conexo
	public void rollbackAndCloseConn(){
		try {
			conn.rollback();
			jorfinder.persistence.ConnectionFactory.closeConnection(conn);	
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	// esse metodo deve ser usado para pegar a conexo e compartilhar as conexo entre vrias DAO's
	// no caso de uma nica transao.
	public java.sql.Connection getConnection(){
		return conn;
	}
	
	// fecha um resultset e preparedStatement
	protected void closePstmtAndResultSet(PreparedStatement pstmt, ResultSet rs){
	      try {
	    	  if (rs != null) 
	    		  rs.close();} 
	      catch (SQLException e) {}
	      
	      try {
	    	  if (pstmt != null) 
	    		  pstmt.close();} 
	      catch (SQLException e) {}
	}
	
	// fecha um preparedStatement
	protected void closePstmt(PreparedStatement pstmt){
	      
	      try {
	    	  if (pstmt != null){ 
	    		  pstmt.close();
	    	  }
	    	  }catch (SQLException e) {}
	}
	
	abstract public void adiciona(ValueObject obj)throws UniqueException;

	abstract public void atualiza(ValueObject obj)throws UniqueException;
	
	abstract public void exclui(ValueObject obj)throws VioletionFKException;
	
	abstract public ValueObject getObjetoByUnique(ValueObject objeto);
	
	abstract public List<ValueObject> getListaObjeto();
	
	
	
	
}
