package jorfinder.persistence;


import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import jorfinder.information.GabaritoVO;
import jorfinder.information.RedeVO;
import jorfinder.log2java.Logger;
import jorfinder.util.UniqueException;
import jorfinder.util.VioletionFKException;

public class GabaritoDAO extends AbstractDAO<GabaritoVO>{

	public static Logger log = new Logger(GabaritoDAO.class.getName());
	
	public GabaritoDAO(){

	}
	
	public GabaritoDAO(java.sql.Connection conn){
		super(conn);
		
	}
	
	
	@Override
	public void adiciona(GabaritoVO obj) throws UniqueException {
		PreparedStatement ps = null;
		try{
                    ps = conn.prepareStatement("insert into gabarito(codRede, nome) values(?,?)");
                    ps.setInt(1, obj.getCodRede());
                    ps.setString(2, obj.getNome());
                    
                    ps.execute(); 
                }catch (SQLException e) {
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de chave unique
                    if (theError.equals(ConnectionFactory.getCodUniqueException())){
                         log.exception("Lanando uma excesso de violao de unique ao adicionar GabaritoVO");
                         throw new UniqueException();
                    }
                    log.exception("Erro ao cadastrar GabaritoVO", e);
                  
                }finally{
                        this.closePstmt(ps);
		}	
	}

	@Override
	public void atualiza(GabaritoVO obj) throws UniqueException {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void exclui(GabaritoVO obj) throws VioletionFKException {
		// TODO Auto-generated method stub
		
	}
        
        @Override
		public ArrayList<GabaritoVO> getListaObjeto() {
            PreparedStatement ps = null;
            ResultSet rs = null;
            GabaritoVO obj = null;
            ArrayList<GabaritoVO> array = new ArrayList<GabaritoVO>();

            try {
                ps = conn.prepareStatement("select * from Gabarito order by cod");
                rs = ps.executeQuery();                
                while(rs.next()){
                    // Cria e popula o objeto
                    obj = new GabaritoVO();
                    obj.setCod(rs.getInt("cod"));
                    obj.setNome(rs.getString("nome"));             
                    array.add(obj);
                }                  
            }catch (SQLException e) {
                    log.exception("erro ao executar getListaObjeto", e);
            }finally{
                    this.closePstmtAndResultSet(ps, rs);
            }
            return array;
	}
        
        public ArrayList<GabaritoVO> getListaObjeto(RedeVO redevo) {
            PreparedStatement ps = null;
            ResultSet rs = null;
            GabaritoVO obj = null;
            ArrayList<GabaritoVO> array = new ArrayList<GabaritoVO>();

            try {
                ps = conn.prepareStatement("select * from Gabarito where codRede = ?");
                ps.setInt(1, redevo.getCod());
                rs = ps.executeQuery();                
                while(rs.next()){
                    // Cria e popula o objeto
                    obj = new GabaritoVO();
                    obj.setCod(rs.getInt("cod"));
                    obj.setCodRede(rs.getInt("codRede"));
                    obj.setNome(rs.getString("nome"));          
                    array.add(obj);
                }                  
            }catch (SQLException e) {
                    log.exception("erro ao executar getListaObjeto", e);
            }finally{
                    this.closePstmtAndResultSet(ps, rs);
            }
            return array;
	}

        @Override
		public GabaritoVO getObjetoByUnique(GabaritoVO objeto) {
		PreparedStatement ps = null;
		ResultSet rs = null;
                GabaritoVO gabaritovo = null;
		try{
                    ps = conn.prepareStatement("select * from Gabarito where codRede = ? and nome = ?");
                    ps.setInt(1, objeto.getCodRede());
                    ps.setString(2, objeto.getNome());
                    rs = ps.executeQuery();
                    if (rs.next()){
                             gabaritovo = new GabaritoVO();
                             gabaritovo.setCod(rs.getInt("cod"));
                             gabaritovo.setCodRede(rs.getInt("codRede"));
                             gabaritovo.setNome(rs.getString("nome"));
                    }
                }catch (SQLException e) {
                	log.exception("erro ao executar getListaObjeto", e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
		return gabaritovo;
	}       

}
