package jorfinder.persistence;


import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import jorfinder.information.ORFVO;
import jorfinder.log2java.Logger;
import jorfinder.util.UniqueException;
import jorfinder.util.VioletionFKException;

public class ORFDAO extends AbstractDAO<ORFVO>{
        
        public static Logger log = new Logger(UsuarioDAO.class.getName());
        
	public ORFDAO(){}	
        
	public ORFDAO(java.sql.Connection conn){
		super(conn);
		
	}
	
	@Override
	public void adiciona(ORFVO obj) throws UniqueException {
		PreparedStatement ps = null;
		try{
                    ps = conn.prepareStatement("insert into ORF(codPesquisa, posCodonStart, posCodonStop, linha, codTipoORF, observacao) values(?,?,?,?,?,?)");
                    ps.setInt(1, obj.getCodPesquisa());
                    ps.setInt(2, obj.getPosCodonStart());
                    ps.setInt(3, obj.getPosCodonStop());
                    ps.setInt(4, obj.getLinha());
                    ps.setInt(5, obj.getCodTipoORF());
                    ps.setString(6, obj.getObservacao());
                    
                    ps.execute(); 
                }catch (SQLException e) {
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de chave unique
                    if (theError.equals(ConnectionFactory.getCodUniqueException())){
                         throw new UniqueException();
                    }
                    log.exception("Erro ao cadastrar ORFVO", e);
                  
                }finally{
                        this.closePstmt(ps);
		}	
	}

	@Override
	public void atualiza(ORFVO obj) throws UniqueException {
		PreparedStatement ps = null;
		try{
					
                    ps = conn.prepareStatement("update orf set codTipoORF = ? , observacao = ?" +
                            " where codPesquisa = ? and posCodonStart = ? and posCodonStop = ? and linha = ?");

                    ps.setInt(1, obj.getCodTipoORF());
                    ps.setString(2, obj.getObservacao());
                    ps.setInt(3, obj.getCodPesquisa());
                    ps.setInt(4, obj.getPosCodonStart());
                    ps.setInt(5, obj.getPosCodonStop());
                    ps.setInt(6, obj.getLinha());
                    
                    ps.execute();
                }catch (SQLException e) {
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de chave unique
                    if (theError.equals(ConnectionFactory.getCodUniqueException())){
                         throw new UniqueException();
                    }
                    log.exception("Erro ao atualizar o orf", e);
                    
                }finally{
                        this.closePstmt(ps);
		}	
		
	}
        

	@Override
	public void exclui(ORFVO obj) throws VioletionFKException {
		// TODO Auto-generated method stub
		
	}
        
        public List<ORFVO> getListaObjeto(ORFVO fragvo) {
            PreparedStatement ps = null;
            ResultSet rs = null;
            ORFVO obj = null;
            ArrayList<ORFVO> array = new ArrayList<ORFVO>();

            try {
                ps = conn.prepareStatement("select * from ORF where codPesquisa = ? and linha = ? ");
                ps.setInt(1, fragvo.getCodPesquisa());
                ps.setInt(2, fragvo.getLinha());
                rs = ps.executeQuery();                
                while(rs.next()){
                    // Cria e popula o objeto
                    obj = new ORFVO();
                    obj.setCodPesquisa(rs.getInt("codPesquisa"));
                    obj.setCodTipoORF(rs.getInt("codTipoORF"));
                    obj.setLinha(rs.getInt("linha"));
                    obj.setObservacao(rs.getString("observacao"));
                    obj.setPosCodonStart(rs.getInt("posCodonStart"));
                    obj.setPosCodonStop(rs.getInt("posCodonStop"));                           
                    array.add(obj);
                }                  
            }catch (SQLException e) {
                    log.exception("erro ao executar getListObjeto", e);
            }finally{
                    this.closePstmtAndResultSet(ps, rs);
            }
            return array;
	}
        
        
	@Override
	public List<ORFVO> getListaObjeto() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public ORFVO getObjetoByUnique(ORFVO objeto) {
		PreparedStatement ps = null;
		ResultSet rs = null;
                ORFVO orfvo = null;
		try{
                    ps = conn.prepareStatement("select * from ORF where codPesquisa = ? and linha = ? and posCodonStart = ? and posCodonStop = ?");
                    ps.setInt(1, objeto.getCodPesquisa());
                    ps.setInt(2, objeto.getLinha());
                    ps.setInt(3, objeto.getPosCodonStart());
                    ps.setInt(4, objeto.getPosCodonStop());
                    rs = ps.executeQuery();
                    if (rs.next()){
                             orfvo = new ORFVO();
                             orfvo.setCodPesquisa(rs.getInt("codPesquisa"));
                             orfvo.setCodTipoORF(rs.getInt("codTipoORF"));
                             orfvo.setLinha(rs.getInt("linha"));
                             orfvo.setObservacao(rs.getString("observacao"));
                             orfvo.setPosCodonStart(rs.getInt("posCodonStart"));
                             orfvo.setPosCodonStop(rs.getInt("posCodonStop"));
                    }
                }catch (SQLException e) {
                	log.exception("erro ao executar getObjetoByUnique", e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
		return orfvo;
	}	
}
