package jorfinder.persistence; 

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import jorfinder.information.PesquisaVO;
import jorfinder.information.ProjetoVO;
import jorfinder.log2java.Logger;
import jorfinder.util.UniqueException;
import jorfinder.util.VioletionFKException;

public class PesquisaDAO extends AbstractDAO<PesquisaVO>{

	public static Logger log = new Logger(PesquisaDAO.class.getName());
	
	public PesquisaDAO(){
		
	}
	
	public PesquisaDAO(java.sql.Connection conn){
		super(conn);
		
	}
	
	
	@Override
	public void adiciona(PesquisaVO obj) throws UniqueException {
		PreparedStatement ps = null;
		try{
                    ps = conn.prepareStatement("insert into Pesquisa(nome, descricao, dtcriacao, codProjeto, codRede, dna) values(?,?, current_timestamp,?,?,?)");
                    ps.setString(1, obj.getNome());
                    ps.setString(2, obj.getDescricao());
                    ps.setInt(3, obj.getCodProjeto());      
                    ps.setInt(4, obj.getCodRede());
                    ps.setString(5, obj.getDna());
                    ps.execute();

                }catch (SQLException e) {
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de chave unique
                    if (theError.equals(ConnectionFactory.getCodUniqueException())){
                         log.exception("Lanando uma excesso de violao de unique ao adicionar PesquisaVO");
                         throw new UniqueException();
                    }
                    log.exception("Erro ao cadastrar PesquisaVO", e);
                }finally{
                        this.closePstmt(ps);
		}	
	}

	@Override
	public void atualiza(PesquisaVO obj) throws UniqueException {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void exclui(PesquisaVO obj) throws VioletionFKException {
		PreparedStatement ps = null;
		try{
					
					ps = conn.prepareStatement("delete from pesquisa where cod = ?");
		            ps.setInt(1, obj.getCod());                    
		            ps.execute();
		            
                    
                }catch (SQLException e) {
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de FK
                    if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                         log.exception("Lanando uma excesso de violao de FK ao excluir PesquisaVO");
                         throw new VioletionFKException();
                    }
                    	log.exception("Erro ao excluir pesquisaVO", e);
                    
                    
                }finally{
                        this.closePstmt(ps);
		}
		
	}
        
	public List<PesquisaVO> getListaObjeto(ProjetoVO projvo) {
            PreparedStatement ps = null;
            ResultSet rs = null;
            PesquisaVO obj = null;
            ArrayList<PesquisaVO> array = new ArrayList<PesquisaVO>();

            try {
                ps = conn.prepareStatement("select * from Pesquisa where codProjeto = ? ");
                ps.setInt(1, projvo.getCod());
                rs = ps.executeQuery();                
                while(rs.next()){
                    // Cria e popula o objeto
                    obj = new PesquisaVO();
                    obj.setCod(rs.getInt("cod"));
                    obj.setCodProjeto(rs.getInt("codProjeto"));
                    obj.setCodRede(rs.getInt("codRede"));
                    obj.setDescricao(rs.getString("descricao"));
                    obj.setDna(rs.getString("dna"));
                    obj.setNome(rs.getString("nome"));                           
                    array.add(obj);
                }                  
            }catch (SQLException e) {
                    log.exception("erro ao executar getListaObjeto", e);
            }finally{
                    this.closePstmtAndResultSet(ps, rs);
            }
            return array;
	}

	@Override
	public PesquisaVO getObjetoByUnique(PesquisaVO objeto) {
		PreparedStatement ps = null;
		ResultSet rs = null;
        PesquisaVO pesqvo = null;
		try{
                    ps = conn.prepareStatement("select * from Pesquisa where cod = ? ");
                    ps.setInt(1, objeto.getCod());
                    rs = ps.executeQuery();
                    if (rs.next()){
                             pesqvo = new PesquisaVO();
                             pesqvo.setCod(rs.getInt("cod"));
                             pesqvo.setNome(rs.getString("nome"));
                             pesqvo.setDescricao(rs.getString("descricao"));
                             pesqvo.setCodProjeto(rs.getInt("codProjeto"));
                             pesqvo.setCodRede(rs.getInt("codRede"));
                             pesqvo.setDna(rs.getString("dna"));
                    }
                }catch (SQLException e) {
                        log.exception("erro ao executar getObjetoByUnique", e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
		return pesqvo;
	}	

        public PesquisaVO getUltimoCadastrado() {
            PreparedStatement ps = null;
            ResultSet rs = null;
            PesquisaVO pesqvo = null;
            try {
                 ps = conn.prepareStatement("select  * from Pesquisa order by cod desc limit 1");
                 rs = ps.executeQuery();
                 if (rs.next()){
                     pesqvo = new PesquisaVO();
                     pesqvo.setCod(rs.getInt("cod"));
                     pesqvo.setNome(rs.getString("nome"));
                     pesqvo.setDescricao(rs.getString("descricao"));
                     pesqvo.setCodProjeto(rs.getInt("codProjeto"));
                     pesqvo.setCodRede(rs.getInt("codRede"));
                     pesqvo.setDna(rs.getString("dna"));
                 }

            } catch (SQLException ex) {
                log.exception("erro ao executar getUltimoCadastrado", ex);
            }finally{
                this.closePstmtAndResultSet(ps, rs);
    		}	                      
            return pesqvo;
        }

        @Override
		public List<PesquisaVO> getListaObjeto() {
                return null;
        }
}
