package jorfinder.persistence; 

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import jorfinder.log2java.Logger;
import jorfinder.util.RedeDefault;

public class PostgresRDBMS implements RDBMS{

	public static Logger log = new Logger(PostgresRDBMS.class.getName());
	
	public String getCodUniqueException() {
		return "23505";		
	}


	public String getCodVioletionFKException() {
		return "23503";
	}


	public void verificarRDBMS(Connection conn) {
				
              log.info("executando o verificarRDBMS");
			  
              // verifica se todas as tabelas esto ok
              if (!this.isBancoIntegro(conn)){
                  log.info("O banco no est integro ser apagado");
                  // As tabelas no est ok! ento temos que apagar tudo!
                  this.deleteTables(conn);
                  log.info("Recriando as tabelas.");
                  // agora temos que criar tudo de novo!
                  this.criarTabelas(conn);
                  
                  // grava a rede default no banco
                  RedeDefault.CadastrarRede();
                  
              }	 	

       }   


  
      // cria todas as tabelas no bd
       private void criarTabelas(Connection conn){
           
            String comando = "";
              try {
            	  
                  Statement statement = conn.createStatement();
                  String[] comandos = this.getSqlCriaTabelas();
                  for (int i = 0; i < comandos.length ; i++){
                	  comando = comandos[i];
                	  log.info("executando = " + comando);
                      statement.execute(comandos[i]);
                      
                  }
              } catch (SQLException ex) {
                   log.exception("Ocorreu o seguinte erro ao executar = " + comando , ex);
                   
              }

       }
       
       // verifica se existem todas as tabelas e todos os campos
       private boolean isBancoIntegro(Connection conn){
                                  
                try {
                      Statement statement = conn.createStatement();
                      String[] comandos = this.getSqlVerificaTabelas();
                      
                      for (int i = 0; i < comandos.length ; i++){
                           log.info(" executando o seguinte comando = " + comandos[i]);
                           statement.execute(comandos[i]);
                      }
                }  catch (SQLException sqle) {                    
                    log.exception("O banco no est ok", sqle);
                    return false;
                   
                }
                
                return true;
       }
       
       // tenta apagar todas as tabelas! E deixar o banco limpo!
       private void deleteTables(Connection conn){
                      
                      try{
                      Statement statement = conn.createStatement();                        
                      String[] comandos = this.getSqlDeleteTables();             
                      
                      for (int i = 0; i < comandos.length ; i++){
                          try {                                         
                              statement.execute(comandos[i]);
                              }  catch (SQLException sqle) {
                                  log.exception("erro ao executar o seguinte comando = " + comandos[i]);
                                  log.exception("Ocorreu um erro ao deletar a tabela " , sqle);
                                // no faz nada se der erro no delete, porque o erro deve ser porque a tabela no existe
                                }
                           }
                       }catch(SQLException sqle) {
                                log.exception("erro ao conectar = " , sqle);                                  
                     }
       }
       // comandos para verificar se existem todas as tabelas necessarias e com os campos certos!
       private String[] getSqlVerificaTabelas(){
           String retorno[] = new String[10];
           
           retorno[0] = " select cod , tipo from tipoORF where cod = (select max(cod) from tipoORF)";
           retorno[1] = " select cod , nome, descricao, rede from rede where cod = (select max(cod) from rede)";
           retorno[2] = " select cod , nome, instituicao, email, login, senha from usuario where cod = (select max(cod) from usuario)";
           retorno[3] = " select cod , owner, nome, descricao from projeto where cod = (select max(cod) from projeto)";
           retorno[4] = " select cod , codon from codon where cod = (select max(cod) from codon)";           
           retorno[5] = " select cod , codRede, nome from gabarito where cod = (select max(cod) from gabarito)";
           retorno[6] = " select cod , nome, descricao, dtcriacao, codProjeto, DNA from pesquisa where cod = (select max(cod) from pesquisa)";
           retorno[7] = " select codProjeto, codUsuario from usuarioProjeto where codProjeto = (select max(codProjeto) from usuarioProjeto)";            
           retorno[8] = " select codPesquisa, posCodonStart, posCodonStop, linha, codTipoORF, observacao from ORF where codPesquisa= (select max(codPesquisa) from ORF)";
           retorno[9] = " select codGabarito, nota, codCodon from nota where codGabarito = (select max(codGabarito) from nota)"; 
          return retorno;       
           
       }
       
      // retorna todos os comando de create das tabelas
       private String[] getSqlCriaTabelas(){
           
          String retorno[] = new String[13];
               
          retorno[0] = " CREATE TABLE tipoORF (" +
                  "  cod SERIAL," +
                  "  tipo VARCHAR(40) NOT NULL," +
                  "  PRIMARY KEY(cod) " +
                  ")";
          
          retorno[1] = " CREATE TABLE rede (" +
                  "  cod SERIAL," +
                  "  nome VARCHAR(50) NOT NULL," +
                  "  descricao VARCHAR(200)," +
                  "  rede TEXT NOT NULL," +
                  "  unique (nome), " +
                  "  PRIMARY KEY(cod)" +
                  ")";
          retorno[2] = " CREATE TABLE usuario (" +
                  "  cod SERIAL," +
                  "  nome VARCHAR(50) NOT NULL," +
                  "  instituicao VARCHAR(50) NOT NULL," +
                  "  email VARCHAR(50) NOT NULL," +
                  "  login VARCHAR(15) NOT NULL," +
                  "  senha char(32) NOT NULL," +
                  "  unique(email)," +
                  "  unique(login)," +
                  "  PRIMARY KEY(cod)" +
                  ")";
          
          retorno[3] = " CREATE TABLE projeto (" +
                  "  cod SERIAL," +
                  "  owner INTEGER NOT NULL," +
                  "  nome VARCHAR(200) NOT NULL," +
                  "  descricao VARCHAR(200)," +
                  "  unique (nome), " +
                  "  PRIMARY KEY(cod)," +
                  "  FOREIGN KEY(owner)" +
                  "  REFERENCES usuario(cod)" +
                  "  ON DELETE NO ACTION" +
                  "  ON UPDATE NO ACTION" +
                  ")";
          
          retorno[4] = " CREATE TABLE codon (" +
                  "  cod SERIAL," +
                  "  codon CHAR(3) NOT NULL," +
                  "  PRIMARY KEY(cod)" +
                  ")";
          
          
          retorno[5] = " CREATE TABLE gabarito (" +
                  "  cod SERIAL," +
                  "  codRede INTEGER NOT NULL," +
                  "  nome VARCHAR(50) NOT NULL," +
                  "  PRIMARY KEY(cod)," +
                  "  FOREIGN KEY(codRede)" +
                  "    REFERENCES rede(cod)" +
                  "      ON DELETE CASCADE" +
                  "      ON UPDATE CASCADE" +
                  ")";
          
          retorno[6] = " CREATE TABLE pesquisa (" +
                  "  cod SERIAL," +
                  "  nome VARCHAR(200) NOT NULL," +
                  "  descricao VARCHAR(200)," +
                  "  dtcriacao DATE," +
                  "  codProjeto INTEGER NOT NULL," +
                  "  codRede INTEGER NOT NULL," +
                  "  DNA TEXT NOT NULL," +
                  "  PRIMARY KEY(cod)," +
                  "  FOREIGN KEY(codProjeto)" +
                  "    REFERENCES projeto(cod)" +
                  "      ON DELETE CASCADE" +
                  "      ON UPDATE CASCADE," +
                  "  FOREIGN KEY(codRede)" +
                  "    REFERENCES rede(cod)" +
                  "      ON DELETE NO ACTION" +
                  "      ON UPDATE NO ACTION" +
                  ")";
          
          retorno[7] = " CREATE TABLE usuarioProjeto (" +
                  "  codProjeto INTEGER  NOT NULL," +
                  "  codUsuario INTEGER  NOT NULL," +
                  "  PRIMARY KEY(codProjeto, codUsuario)," +
                  "  FOREIGN KEY(codUsuario)" +
                  "    REFERENCES usuario(cod)" +
                  "      ON DELETE NO ACTION" +
                  "      ON UPDATE NO ACTION," +
                  "  FOREIGN KEY(codProjeto)" +
                  "    REFERENCES projeto(cod)" +
                  "      ON DELETE CASCADE" +
                  "      ON UPDATE CASCADE" +
                  ")";
          
          retorno[8] = " CREATE TABLE ORF (" +
                  "  codPesquisa INTEGER  NOT NULL," +
                  "  posCodonStart INTEGER  NOT NULL," +
                  "  posCodonStop INTEGER  NOT NULL," +
                  "  linha INTEGER  NOT NULL," +
                  "  codTipoORF INTEGER  NOT NULL," +
                  "  observacao VARCHAR(300)," +
                  "  PRIMARY KEY(codPesquisa, posCodonStart, posCodonStop, linha)," +
                  "  FOREIGN KEY(codPesquisa)" +
                  "    REFERENCES pesquisa(cod)" +
                  "      ON DELETE CASCADE" +
                  "      ON UPDATE CASCADE," +
                  "  FOREIGN KEY(codTipoORF)" +
                  "    REFERENCES tipoORF(cod)" +
                  "      ON DELETE NO ACTION" +
                  "      ON UPDATE NO ACTION" +
                  ")";
          
          retorno[9] = " CREATE TABLE nota (" +
                  "  codGabarito INTEGER NOT NULL," +
                  "  codCodon INTEGER NOT NULL," +
                  "  nota INTEGER NOT NULL," +                  
                  "  PRIMARY KEY(codGabarito, codCodon, nota)," +
                  "  FOREIGN KEY(codCodon)" +
                  "    REFERENCES codon(cod)" +
                  "      ON DELETE NO ACTION" +
                  "      ON UPDATE NO ACTION," +
                  "  FOREIGN KEY(codGabarito)" +
                  "    REFERENCES gabarito(cod)" +
                  "      ON DELETE CASCADE" +
                  "      ON UPDATE CASCADE" +
                  ")";
          
           
           //j insere os tipos de ORF possveis
           retorno[10] =  
                   "insert into tipoORF(tipo) VALUES ('ORF aprovada pela rede'); " +
                   "insert into tipoORF(tipo) VALUES ('ORF NO aprovada pela rede'); " +
                   "insert into tipoORF(tipo) VALUES ('ORF aprovada pelo usurio'); " +
                   "insert into tipoORF(tipo) VALUES ('ORF NO aprovada pelo usurio');";
           
           retorno[11] = 
                   "insert into usuario (nome, instituicao, email, login, senha) values ('Administrador do J-ORFinder', 'UFPR', 'administrador@jorfinder.ufpr.br', 'administrador', '52a6a2a9943a2a2b1c838b591452de65');";
           // as duas duas instrues abaixo devem ser deletadas quando o cadastro de projeto estiver ok
           retorno[12] =                    
                    "insert into codon values (0,'AAA');" +
                    "insert into codon values (1,'GAA');" +
                    "insert into codon values (2,'CAA');" +
                    "insert into codon values (3,'TAA');" +
                    "insert into codon values (4,'AGA');" +
                    "insert into codon values (5,'GGA');" +
                    "insert into codon values (6,'CGA');" +
                    "insert into codon values (7,'TGA');" +
                    "insert into codon values (8,'ACA');" +
                    "insert into codon values (9,'GCA');" +
                    "insert into codon values (10,'CCA');" +
                    "insert into codon values (11,'TCA');" +
                    "insert into codon values (12,'ATA');" +
                    "insert into codon values (13,'GTA');" +
                    "insert into codon values (14,'CTA');" +
                    "insert into codon values (15,'TTA');" +
                    "insert into codon values (16,'AAG');" +
                    "insert into codon values (17,'GAG');" +
                    "insert into codon values (18,'CAG');" +
                    "insert into codon values (19,'TAG');" +
                    "insert into codon values (20,'AGG');" +
                    "insert into codon values (21,'GGG');" +
                    "insert into codon values (22,'CGG');" +
                    "insert into codon values (23,'TGG');" +
                    "insert into codon values (24,'ACG');" +
                    "insert into codon values (25,'GCG');" +
                    "insert into codon values (26,'CCG');" +
                    "insert into codon values (27,'TCG');" +
                    "insert into codon values (28,'ATG');" +
                    "insert into codon values (29,'GTG');" +
                    "insert into codon values (30,'CTG');" +
                    "insert into codon values (31,'TTG');" +
                    "insert into codon values (32,'AAC');" +
                    "insert into codon values (33,'GAC');" +
                    "insert into codon values (34,'CAC');" +
                    "insert into codon values (35,'TAC');" +
                    "insert into codon values (36,'AGC');" +
                    "insert into codon values (37,'GGC');" +
                    "insert into codon values (38,'CGC');" +
                    "insert into codon values (39,'TGC');" +
                    "insert into codon values (40,'ACC');" +
                    "insert into codon values (41,'GCC');" +
                    "insert into codon values (42,'CCC');" +
                    "insert into codon values (43,'TCC');" +
                    "insert into codon values (44,'ATC');" +
                    "insert into codon values (45,'GTC');" +
                    "insert into codon values (46,'CTC');" +
                    "insert into codon values (47,'TTC');" +
                    "insert into codon values (48,'AAT');" +
                    "insert into codon values (49,'GAT');" +
                    "insert into codon values (50,'CAT');" +
                    "insert into codon values (51,'TAT');" +
                    "insert into codon values (52,'AGT');" +
                    "insert into codon values (53,'GGT');" +
                    "insert into codon values (54,'CGT');" +
                    "insert into codon values (55,'TGT');" +
                    "insert into codon values (56,'ACT');" +
                    "insert into codon values (57,'GCT');" +
                    "insert into codon values (58,'CCT');" +
                    "insert into codon values (59,'TCT');" +
                    "insert into codon values (60,'ATT');" +
                    "insert into codon values (61,'GTT');" +
                    "insert into codon values (62,'CTT');" +
                    "insert into codon values (63,'TTT');";
           
           return retorno;       
       }          
       
       // comandos para apagar todas as tabelas se elas existirem para poder criar tudo de novo
       // detalhe a ordem de delete  inversa a ordem de create se no pode ocorrer problemas com restrio de FK
       private String[] getSqlDeleteTables(){
                  String retorno[] = new String[10];
           
           retorno[9] = " drop table tipoORF ";
           retorno[8] = " drop table rede";
           retorno[7] = " drop table usuario";
           retorno[6] = " drop table projeto";
           retorno[5] = " drop table codon";           
           retorno[4] = " drop table gabarito";
           retorno[3] = " drop table pesquisa";
           retorno[2] = " drop table usuarioProjeto";            
           retorno[1] = " drop table ORF";
           retorno[0] = " drop table nota"; 
       
           return retorno;  
       }    
	
	

}
