package jorfinder.persistence;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import jorfinder.information.ProjetoVO;
import jorfinder.information.UsuarioVO;
import jorfinder.log2java.Logger;
import jorfinder.util.UniqueException;
import jorfinder.util.VioletionFKException;

public class ProjetoDAO extends AbstractDAO<ProjetoVO>{

	public static Logger log = new Logger(ProjetoDAO.class.getName());
	
	
	public void adiciona(ProjetoVO obj, List<UsuarioVO> listUsuario) throws UniqueException {
		PreparedStatement ps = null;
		try{
			
                    ps = conn.prepareStatement("insert into projeto(nome, owner, descricao) values(?,?,?)");
                    ps.setString(1, obj.getNome());
                    ps.setInt(2, obj.getOwner().getCod());
                    ps.setString(3, obj.getDescricao());                    
                    ps.execute();
                    
                    int codProjeto = this.getObjetoByUnique(obj).getCod();
                    
                    for (UsuarioVO usuario: listUsuario){
                    	ps = conn.prepareStatement("insert into usuarioProjeto(codProjeto,codUsuario) values(?,?)");
                        ps.setInt(1, codProjeto);
                        ps.setInt(2, usuario.getCod());
                        ps.execute();
                    	
                    }
                    
                    
                }catch (SQLException e) {
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de chave unique
                    if (theError.equals(ConnectionFactory.getCodUniqueException())){
                         log.exception("Lanando uma excesso de violao de unique ao adicionar ProjetoVO");
                         throw new UniqueException();
                    }
                    log.exception("Erro ao cadastrar ProjetoVO", e);
                    
                }finally{
                        this.closePstmt(ps);
		}
		
	}

	@Override	
	public void atualiza(ProjetoVO obj) throws UniqueException {
		PreparedStatement ps = null;
		try{
					
                    ps = conn.prepareStatement("update projeto  set nome = ? , descricao = ? where cod = ? ");
                    ps.setString(1, obj.getNome());
                    ps.setString(2, obj.getDescricao());
                    ps.setInt(3, obj.getCod());
                    ps.execute();
                    
                    ps = conn.prepareStatement("delete from usuarioProjeto where codProjeto = ? ");
                    ps.setInt(1, obj.getCod());
                    ps.execute();
                    
                }catch (SQLException e) {
                	
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de chave unique
                    if (theError.equals(ConnectionFactory.getCodUniqueException())){
                         log.exception("Lanando uma excesso de violao de unique ao adicionar ProjetoVO");
                         throw new UniqueException();
                    }
                    
                    log.exception("Erro ao atualizar cadastro de projeto", e);
                    
                }finally{
                        this.closePstmt(ps);
		}	

		
	}
	
	public void atualizaListUsuarios(ProjetoVO obj, List<UsuarioVO> listUsuario){
		PreparedStatement ps = null;
		try{		
                    
                for(int i =0; i < listUsuario.size(); i++){
                	UsuarioVO usuario = listUsuario.get(i);
                    ps = conn.prepareStatement("insert into usuarioProjeto(codProjeto,codUsuario) values(?,?)");
                    ps.setInt(1, obj.getCod());
                    ps.setInt(2, usuario.getCod());
                    ps.execute();
                    
                }
                
                
            }catch (SQLException e) {
            	log.exception("Erro ao atualizar cadastro de projeto", e);
                
            }finally{
                    this.closePstmt(ps);
		}	

		
	}
	
	
	

	@Override
	public void exclui(ProjetoVO obj) throws VioletionFKException {
		PreparedStatement ps = null;
		try{
					
					ps = conn.prepareStatement("delete from usuarioProjeto where codProjeto = ?");
		            ps.setInt(1, obj.getCod());                    
		            ps.execute();
		            
                    ps = conn.prepareStatement("delete from projeto where cod = ?");
                    ps.setInt(1, obj.getCod());                    
                    ps.execute();
                    
                    
                }catch (SQLException e) {
                	
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de FK
                    if (theError.equals(ConnectionFactory.getCodVioletionFKException())){
                         log.exception("Lanando uma excesso de violao de FK ao excluir ProjetoVO");
                         throw new VioletionFKException();
                    }
                    log.exception("Erro ao excluir projeto", e);
                    
                	
                    
                    
                }finally{
                        this.closePstmt(ps);
		}
		
	}

	@Override
	public List<ProjetoVO> getListaObjeto() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public ProjetoVO getObjetoByUnique(ProjetoVO objeto) {
		PreparedStatement ps = null;
		ResultSet rs = null;
        ProjetoVO projeto = null;
		try{
					String sql = "";
					if (objeto.getCod() != 0 ){
						sql = "select * from projeto where cod = ? ";
						ps = conn.prepareStatement(sql);
	                    ps.setInt(1, objeto.getCod());
					}else if(objeto.getNome() != null){
						sql = "select * from projeto where nome = ? ";
						ps = conn.prepareStatement(sql);
	                    ps.setString(1, objeto.getNome());
					}
                    
                    rs = ps.executeQuery();
                    if (rs.next()){
                             projeto = new ProjetoVO();
                             projeto.setCod(rs.getInt("cod"));
                             projeto.setNome(rs.getString("nome"));
                             projeto.setDescricao(rs.getString("descricao"));
                             int codUsuario = rs.getInt("owner");
                             UsuarioVO usu = new UsuarioVO();
                             usu.setCod(codUsuario);
                             UsuarioDAO dao = new UsuarioDAO();
                             UsuarioVO usuario = dao.getObjetoByUnique(usu);
                             dao.commitAndCloseConn();
                             projeto.setOwner(usuario);
                    }
                }catch (SQLException e) {
                        log.exception(e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
                
		return projeto;
	}

	@Override
	public void adiciona(ProjetoVO obj) throws UniqueException {
		// TODO Auto-generated method stub
		
	}
	

	public List<ProjetoVO> getListaProjetoUsuario(UsuarioVO obj){
		
		List<ProjetoVO> listUsuario = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		
		try{
					listUsuario = new ArrayList<ProjetoVO>();
					
					ps = conn.prepareStatement("select * from usuarioProjeto where codUsuario = ? ");
	                ps.setInt(1, obj.getCod());
                    rs = ps.executeQuery();
                    
                    ProjetoVO proje = new ProjetoVO();
                    
                    while(rs.next()){
                		int codProjeto = rs.getInt("codProjeto");
                		proje.setCod(codProjeto);
                		listUsuario.add(this.getObjetoByUnique(proje));
                		 
                    }
                    
                }catch (SQLException e) {
                        log.exception(e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
		
		return listUsuario;
	}
	
public List<UsuarioVO> getListaUsuarioProjeto(ProjetoVO obj){
		
		List<UsuarioVO> listUsuario = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		
		try{
					listUsuario = new ArrayList<UsuarioVO>();
					
					ps = conn.prepareStatement("select * from usuarioProjeto where codProjeto = ? ");
	                ps.setInt(1, obj.getCod());
                    rs = ps.executeQuery();
                    
                    UsuarioDAO dao= new UsuarioDAO();
                    UsuarioVO usu = new UsuarioVO();
                    int codProjeto;
                    while(rs.next()){
                		codProjeto = rs.getInt("codUsuario");
                		usu.setCod(codProjeto);
                		listUsuario.add(dao.getObjetoByUnique(usu));
                		 
                    }
                    dao.commitAndCloseConn();
                    
                }catch (SQLException e) {
                        log.exception(e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
		
		return listUsuario;
	}

	public boolean usuarioTemPermissaoAcesso(ProjetoVO projeto, UsuarioVO obj){
	
		boolean retorno = false;
		PreparedStatement ps = null;
		ResultSet rs = null;
		
		try{
					
					ps = conn.prepareStatement("select * from usuarioProjeto where codUsuario = ? and codProjeto = ? ");
	                ps.setInt(1, obj.getCod());
	                ps.setInt(2, projeto.getCod());
	                rs = ps.executeQuery();
	                if(rs.next()){
	                	retorno = true;
	                }
	                
	                
	            }catch (SQLException e) {
	                    log.exception(e);
	            }finally{
	                    this.closePstmtAndResultSet(ps, rs);
		}
		
		return retorno;
	}
	
}
