package jorfinder.persistence; 

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import jorfinder.information.UsuarioVO;
import jorfinder.log2java.Logger;
import jorfinder.util.UniqueException;
import jorfinder.util.VioletionFKException;

public class UsuarioDAO extends AbstractDAO<UsuarioVO>{

	public static Logger log = new Logger(UsuarioDAO.class.getName());
        
	public UsuarioDAO(){}
	
	public UsuarioDAO(java.sql.Connection conn){
		super(conn);
		
	}
	
	
	@Override
	public void adiciona(UsuarioVO obj) throws UniqueException {
		PreparedStatement ps = null;
		try{
					
                    ps = conn.prepareStatement("insert into usuario(nome, instituicao, email, login, senha) values(?,?,?,?,?)");
                    ps.setString(1, obj.getNome());
                    ps.setString(2, obj.getInstituicao());
                    ps.setString(3, obj.getEmail());
                    ps.setString(4, obj.getLogin());
                    ps.setString(5, obj.getSenha());
                    
                    ps.execute();
                }catch (SQLException e) {
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de chave unique
                    if (theError.equals(ConnectionFactory.getCodUniqueException())){
                         log.exception("Lanando uma excesso de violao de unique ao adicionar UsuarioVO");
                         throw new UniqueException();
                    }
                    log.exception("Erro ao cadastrar UsuarioVO", e);
                    
                }finally{
                        this.closePstmt(ps);
		}	
	}

	
	@Override
	public void atualiza(UsuarioVO obj) throws UniqueException {
		PreparedStatement ps = null;
		try{
					
                    ps = conn.prepareStatement("update usuario set nome = ? , instituicao = ? , email = ? , login = ?, senha = ? where cod = ?");
                    ps.setString(1, obj.getNome());
                    ps.setString(2, obj.getInstituicao());
                    ps.setString(3, obj.getEmail());
                    ps.setString(4, obj.getLogin());
                    ps.setString(5, obj.getSenha());
                    ps.setInt(6, obj.getCod());
                    
                    ps.execute();
                }catch (SQLException e) {
                	String theError = (e).getSQLState();
                    // verifica se  um erro de violao de chave unique
                    if (theError.equals(ConnectionFactory.getCodUniqueException())){
                         log.exception("Lanando uma excesso de violao de unique ao atualizar UsuarioVO");
                         throw new UniqueException();
                    }
                    log.exception("Erro ao atualizar cadastro de usuario", e);
                    
                }finally{
                        this.closePstmt(ps);
		}	
		
	}

	
	@Override
	public void exclui(UsuarioVO obj) throws VioletionFKException {
		// No implementado pois o modulo web no possui modulo de adminstrao de usurios do sistema
		
	}

	
	@Override
	public List<UsuarioVO> getListaObjeto() {
		PreparedStatement ps = null;
		ResultSet rs = null;
		UsuarioVO usuvo = null;
		List<UsuarioVO> array = null;
		try{
                    ps = conn.prepareStatement("select * from usuario order by login");
                    rs = ps.executeQuery();
                    array = new ArrayList<UsuarioVO>();
                    while (rs.next()){
                    		 usuvo = new UsuarioVO();
                             usuvo.setCod(rs.getInt("cod"));
                             usuvo.setNome(rs.getString("nome"));
                             usuvo.setLogin(rs.getString("login"));
                             usuvo.setSenha(rs.getString("senha"));
                             usuvo.setInstituicao(rs.getString("instituicao"));
                             usuvo.setEmail(rs.getString("email"));
                             array.add(usuvo);
                    }
                }catch (SQLException e) {
                        log.exception(e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
		return array;
	}

	
	@Override
	public UsuarioVO getObjetoByUnique(UsuarioVO objeto) {
		PreparedStatement ps = null;
		ResultSet rs = null;
        UsuarioVO usuvo = null;
		try{
					String sql = "";
					if (objeto.getCod() != 0 ){
						sql = "select * from usuario where cod = ? ";
						ps = conn.prepareStatement(sql);
	                    ps.setInt(1, objeto.getCod());
					}else if(objeto.getLogin() != null){
						sql = "select * from usuario where login = ? ";
						ps = conn.prepareStatement(sql);
	                    ps.setString(1, objeto.getLogin());
					}else if(objeto.getEmail() != null){
						sql = "select * from usuario where email = ? ";
						ps = conn.prepareStatement(sql);
	                    ps.setString(1, objeto.getEmail());
					}
                    
                    rs = ps.executeQuery();
                    if (rs.next()){
                             usuvo = new UsuarioVO();
                             usuvo.setCod(rs.getInt("cod"));
                             usuvo.setNome(rs.getString("nome"));
                             usuvo.setLogin(rs.getString("login"));
                             usuvo.setSenha(rs.getString("senha"));
                             usuvo.setInstituicao(rs.getString("instituicao"));
                             usuvo.setEmail(rs.getString("email"));
                    }
                }catch (SQLException e) {
                        log.exception(e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
                
		return usuvo;
	}
	
	public UsuarioVO getUsuarioWithLoginEmail(UsuarioVO objeto) {
		PreparedStatement ps = null;
		ResultSet rs = null;
        UsuarioVO usuvo = null;
		try{
					String sql = "select * from usuario where login = ? and email = ?";
					ps = conn.prepareStatement(sql);
                    ps.setString(1, objeto.getLogin());
                    ps.setString(2, objeto.getEmail());
				                    
                    rs = ps.executeQuery();
                    if (rs.next()){
                             usuvo = new UsuarioVO();
                             usuvo.setCod(rs.getInt("cod"));
                             usuvo.setNome(rs.getString("nome"));
                             usuvo.setLogin(rs.getString("login"));
                             usuvo.setSenha(rs.getString("senha"));
                             usuvo.setInstituicao(rs.getString("instituicao"));
                             usuvo.setEmail(rs.getString("email"));
                             
                    }
                }catch (SQLException e) {
                        log.exception(e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
                
		return usuvo;
	}
	
	public boolean isUserPasswordOk(UsuarioVO objeto) {
		
		PreparedStatement ps = null;
		ResultSet rs = null;
		try{
					String sql = "select * from usuario where login = ? and senha = ? ";
					ps = conn.prepareStatement(sql);
                    ps.setString(1, objeto.getLogin());
                    ps.setString(2, objeto.getSenha());
				    rs = ps.executeQuery();
                    if (rs.next()){
                         return true;
                    } 
                }catch (SQLException e) {
                        log.exception(e);
                }finally{
                        this.closePstmtAndResultSet(ps, rs);
		}
                
		return false;
	}	
}
