package jorfinder.view; 

import javax.servlet.RequestDispatcher;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import jorfinder.business.UsuarioBO;
import jorfinder.information.UsuarioVO;
import jorfinder.util.Crypt;
import jorfinder.util.Mensagem;
import jorfinder.util.ValidaString;


/**
 * Responsvel pelo cadastro de usurios
 * @author Oeslei Taborda Ribas
 */
public class AlteraDadosPesquisador implements Action{

	
	private UsuarioVO usuario;
	private String mensagem = "";
	private boolean trocarsenha = false; // variavel de controle para saber se o usurio solictou mudana de senha
	
	public AlteraDadosPesquisador(){}	
	
	public void execute(HttpServletRequest req, HttpServletResponse res)
			throws Exception {
			    
                //pegar os parametros da requisio;
                setParameter(req);
                
                // verifica se est tudo ok
                boolean dadosUsuario =  checkDados();
                
                boolean retornoBanco = false;
                if (dadosUsuario){
                	// se o usurio mudou a senha criptografa ela se no apenas pega a propria senha
                	// que est na sesso
                	if(trocarsenha){
                		cryptPassword();
                	}else{
                		UsuarioVO usu = (UsuarioVO) req.getSession().getAttribute("usuario");
                		usuario.setSenha(usu.getSenha());
                	}
                	
                	UsuarioBO usubo = new UsuarioBO();
                    retornoBanco = usubo.atualiza(usuario);
                    if(!retornoBanco){
                    	mensagem += usubo.getMensagemErro();
                    }
                	
                }
                
                RequestDispatcher rd;
                
                if (dadosUsuario && retornoBanco){
                	// atualiza o objeto usuario que est na sesso com os novos dados.
                	req.getSession().removeAttribute("usuario");
                	req.getSession().setAttribute("usuario", usuario);
                	
                	// devolve para a pagina de origim com  o informao que ocoreu com sucesso a operao
                	req.setAttribute("alert", new Mensagem().getMensagem("usuario.alteracao.sucesso"));
                	rd = req.getRequestDispatcher("/WEB-INF/jsp/dadospessoais.jsp");
                }else{
                	
                	req.setAttribute("mensagem", mensagem);
                	rd = req.getRequestDispatcher("/WEB-INF/jsp/dadospessoais.jsp");
                }
                
                rd.forward(req, res);
	}
	
	private void setParameter(HttpServletRequest req){
                usuario = new UsuarioVO();
                String mudarsenha = req.getParameter("mudarsenha");
                if(mudarsenha.equals("sim")){
                        trocarsenha = true;
                        usuario.setSenha(req.getParameter("senha").toString().trim());
                }
                usuario.setInstituicao(req.getParameter("instituicao").toString().trim().toLowerCase());
                usuario.setNome(req.getParameter("nome").toString().trim().toLowerCase());
                usuario.setEmail(req.getParameter("email").toString().trim());
                UsuarioVO usu = (UsuarioVO) req.getSession().getAttribute("usuario");
                usuario.setLogin(usu.getLogin());
                usuario.setCod(usu.getCod());
		
	}
	
	private boolean checkDados(){
		
		boolean dadosUsuarioOk = true;
		if (!ValidaString.isStringOK(usuario.getNome(), 1, 50)){
			dadosUsuarioOk = false;
			mensagem +=  new Mensagem().getMensagem("cadastro.nome.erro") ;
		}
		
		if (!ValidaString.isStringOK(usuario.getEmail(), 1, 50)){
			dadosUsuarioOk = false;
			mensagem +=  new Mensagem().getMensagem("cadastro.email.erro") ;
		}
		if (!ValidaString.isEmailOK(usuario.getEmail())){
			dadosUsuarioOk = false;
			mensagem +=  new Mensagem().getMensagem("cadastro.email.invalido") ;
		}
		if(trocarsenha){
			if (!ValidaString.isStringOK(usuario.getSenha(), 1, 15)){
				dadosUsuarioOk = false;
				mensagem +=  new Mensagem().getMensagem("cadastro.senha.erro") ;
			}
		}
		
		if (!ValidaString.isStringOK(usuario.getInstituicao(), 1, 50)){
			dadosUsuarioOk = false;
			mensagem +=  new Mensagem().getMensagem("cadastro.instituicao.erro") ;
		}
			
		return dadosUsuarioOk;
	}

	private void cryptPassword(){
			usuario.setSenha(Crypt.getHash(usuario.getSenha()));
		
	}
}





