package jfan.fan.utilitarias.fuzzy;

/**
 * Classe que contm as funes de pertinncia.
 * @author Filipe Pais Lenfers
 * @version 0.2.0
 */
public class FuncaoPertinencia {
		
	//Classe para funo triangular
	/**
	 * Classe que mantm os mtodos para calcular a pertinncia
	 * usando uma funo triangular.
	 */
	public static class Triangular {
		/**
		 * Calcula a pertinncia de um valor.
		 * @param valor o valor ao qual ser dada a pertinncia
		 * @param inicio o nicio do tringulo (a)
		 * @param centro o centro do tringulo (b), o nico valor que ter pertinencia 1 
		 * @param fim o final do tringulo (c)
		 * @return um grau de pertinncia para o parmetro valor
		 */
		public static double  calcularPertinencia(double valor, double inicio, double centro, double fim) {
			if (valor< inicio || fim <= valor)
				return 0;
			else {
				if (inicio <= valor && valor < centro) {
					return ((valor-inicio)/(centro-inicio));
				}
				else {
					return ((fim-valor)/(fim-centro));
				}
			}
			/*int d0 = (int) ((int) fim-inicio);
			int d1 = d0; //+ 1;
			if (valor<centro)
				return (float) (1./d1)*(valor+d1-centro);
			else
				return (float) (1./d1)*(-valor+d1+centro);*/


			
		}
		
		/**
		 * Calcula a pertinncia de um valor.
		 * @param valor o valor ao qual ser dada a pertinncia
		 * @param centro o centro do tringulo (b), o nico valor que ter pertinencia 1
		 * @param raioDecomposicao quantos valores o tringulo adotar a esquerda e a direita
		 * do centro para definir seu incio e fim 
		 * @return um grau de pertinncia para o parmetro valor
		 */
		public static double  calcularPertinencia(double valor, double centro, int raioDecomposicao) {
			/*double fim = centro + raioDecomposicao;
			double inicio = centro - raioDecomposicao;
			return calcularPertinencia(valor,inicio,centro,fim);*/
			raioDecomposicao++;
			//int d1 = raioDecomposicao+ 1;
			if (valor<centro)
				//return (float) (1.f/raioDecomposicao)*(valor+raioDecomposicao-centro);
				return (float) (valor+raioDecomposicao-centro)/(raioDecomposicao);
			else
				//return (float) (1.f/raioDecomposicao)*(-valor+raioDecomposicao+centro);
				return (float) (-valor+raioDecomposicao+centro)/(raioDecomposicao);

		}
		//forma alternativa de funo triangular
		/*public static double calcularPertinencia2 (double valor, double inicio, double centro, double fim) {
			double a = (valor-inicio)/(centro-inicio);
			double b = (fim-valor)/(fim-centro);
			double min = Math.min(a,b);
			double max = Math.min(0,min);
			return max;
		}*/
	}
	//Fim da classe para funo triangular
	
	//Classe para a funo trapezoidal
	/**
	 * Classe que mantm os mtodos para calcular a pertinncia
	 * usando uma funo trapezoidal
	 */
	public static class Trapezoidal {
		/**
		 * Calcula a pertinncia de um valor.
		 * @param valor o valor ao qual ser dada a pertinncia
		 * @param inicio o incio do trapzio (a) 
		 * @param canto1 o primeiro canto (b)
		 * @param canto2 o segundo canto (c)
		 * @param fim o fim do trapzio (d)
		 * @return um grau de pertinncia para o parmetro valor
		 */
		public static double calcularPertinencia(double valor, double inicio, double canto1, double canto2, double fim) {
			if (valor < inicio || fim <= valor) {
				return 0f;
			} else if (canto1 <= valor && valor < canto2){
				return 1f;
			} else if (inicio <= valor && valor < canto1){
				return ((valor-inicio)/(canto1-inicio));
			} else {
				return ((fim-valor)/(fim-canto2));
			}
		}
		//forma alternativa da funo trapezoidal.
		/*public static double calcularPertinencia2(double valor, double inicio, double canto1, double canto2, double fim) {
			double a = (valor-inicio)/(canto1-inicio);
			double b = (fim-valor)/(fim-canto2);
			double min = Math.min(a,1);
			min = Math.min(min,b);
			double max = Math.max(0,min);
			return max;		
		}*/	
	}
	//Fim da classe para funo trapezoidal
	
	//Classe para funo Gaussiana
	/**
	 * Classe que mantm os mtodos para calcular a pertinncia
	 * usando uma funo gaussiana.
	 */
	public static class Gaussiana {
		/**
		 * Calcula a pertinncia de um valor.
		 * @param valor o valor ao qual ser dada a pertinncia
		 * @param centro o centro da funo (c)
		 * @param largura o "espalhamento" (largura) da funo
		 * @return um grau de pertinncia para o parmetro valor
		 */
		public static double calcularPertinencia(double valor, double centro, double largura) {
			return (Math.exp(-0.5*Math.pow(((valor-centro)/largura),2)));
		}
	}
	//Fim da classe para funo Gaussiana
	
	//Classe para funo sino (bell)
	/**
	 * Classe que mantm os mtodos para calcular a pertinncia
	 * usando uma funo sino generalizada.
	 */
	public static class Sino {
		/**
		 * Calcula a pertinncia de um valor.
		 * @param valor o valor ao qual ser dada a pertinncia
		 * @param centro o centro da funo (c)
		 * @param largura a largura da funo (a), esse parmetro deve ser diferente de 0
		 * @param declive o declive para a funo (b)
		 * @return um grau de pertinncia para o parmetro valor
		 * @throws Exception uma exceo  disparada caso o valor de largura seja 0.
		 */
		public static double calcularPertinencia(double valor, double centro, double largura, double declive) throws Exception {
			if (largura == 0) throw new Exception("O valor de largura da funo de pertinncia sino no pode ser 0.");
			return (1/(1+Math.pow((Math.abs((valor-centro)/largura)),(2*declive))));
		}
	}
	//Fim da classe para funo sino (bell)
	
	//Classe para funo senoidal
	/**
	 * Classe que mantm os mtodos para calcular a pertinncia
	 * usando uma funo senoidal.
	 */
	public static class Senoidal {
		/**
		 * Calcula a pertinncia de um valor.
		 * @param valor o valor ao qual ser dada a pertinncia
		 * @param centro o centro da funo (c)
		 * @param declive o declive para a funo (a)
		 * @return um grau de pertinncia para o parmetro valor
		 */
		public static double calcularPertinencia(double valor, double centro, double declive) {
			return (double)(1/(1+(Math.exp(-declive*(valor-centro)))));
		}
	}
	//Fim da classe para funo senoidal
	
	//Classe para funo simtrica fechada
	/**
	 * Classe que mantm os mtodos para calcular a pertinncia
	 * usando uma funo simtrica fechada baseada em duas funes senoidais.
	 */
	public static class SimetricaFechada {
		/**
		 * Calcula a pertinncia de um valor.
		 * @param valor o valor ao qual ser dada a pertinncia
		 * @param centro1 o centro da senide 1
		 * @param declive1 o declive da senide 1
		 * @param centro2 o centro da senide 2
		 * @param declive2 declive da senide 2
		 * @return um grau de pertinncia para o parmetro valor
		 */
		public static double calcularPertinencia(double valor, double centro1, double declive1, double centro2, double declive2) {
			if ((declive1 < 0 && 0 <= declive2) || (declive1 >= 0 && 0 > declive2)) {
				return (1/((1+Math.exp(-declive1*(valor-centro1)))*(1+Math.exp(-declive2*(valor-centro2)))));
			} else {
				return (((1+Math.exp(-declive2*(valor-centro2)))-(1+Math.exp(-declive1*(valor-centro1))))/((1+Math.exp(-declive1*(valor-centro1)))*(1+Math.exp(-declive2*(valor-centro2)))));
			}
		}
	}
	//Fim da classe para funo simtrica fechada
}
